<?php
namespace MyMeals\Services;

use MyMeals\Util\ParameterBag;
use MyMeals\Util\DateUtil;

class Shifts {

    const APPROVED = "APROVADA";

    protected $dataSourceManager;
    protected $parameterBag;
    protected $connection;

    public function __construct($dataSourceManager, ParameterBag $parameterBag, $connection) {
        $this->dataSourceManager = $dataSourceManager;
        $this->parameterBag = $parameterBag;
        $this->connection = $connection;
    }

    public function findShiftsByMenuDate($filterCriteria) {
        $filterCriteria->setDataSourceName("shifts");
        $newFilterCriteria = $this->parameterBag->getPopulatedFilterFromSession($filterCriteria);
        return $this->dataSourceManager->findBy($newFilterCriteria);
    }

    public function saveShifts($meal, $shifts, &$logCode) {
        foreach ($shifts as $shift) {
            $logCode += 1;
            $this->updateShift($meal, $shift);
            $this->createLog($logCode, $meal, $shift);
        }
    }

    public function updateShift($meal, $shift) {
        if ($meal["IDQTDEAPROVADA"] === self::APPROVED) {
            $shiftToUpdate = "QTESTITURNO" . $shift["CDTURNO"] . " = :QTESTITURNO";
            $update = "UPDATE CARDADIA SET " . $shiftToUpdate . " WHERE NRCARDAPIO = :NRCARDAPIO AND CDSERVFILI = :CDSERVFILI AND CDFILIAL = :CDFILIAL AND NRORG = :NRORG";
            $params = array(
                "QTESTITURNO" => $shift["COMENSAIS"],
                "NRCARDAPIO" => $meal["NRCARDAPIO"],
                "CDSERVFILI" => $meal["CDSERVFILI"],
                "CDFILIAL" => $meal["CDFILIAL"],
                "NRORG" => $meal["NRORG"]
            );
            $sth = $this->connection->prepare($update);
            $sth->execute($params);
        }
    }

    public function createLog($logCode, $meal, $shift) {
        $dsparametros = "NOTIFICA: " . $meal["IDNOTIFICAREFE"] .
                    ' - ' . "HORA ALTERACAO: " . $meal["HRALTERACLI"] .
                    ' - ' . "VALOR ACRESCIMO: " . $meal["VRACRESCIMO"] .
                    ' - ' . "VALOR REDUCAO: " . $meal["VRREDUCAO"] .
                    ' - ' . "NÚMERO DE DIAS ALTERADO: " . $meal["NRDIASALT"];
        $log = array(
            "NRCONFIRMAREFE"=> $logCode,
            "CDFILIAL"=> $meal["CDFILIAL"],
            "CDSERVFILI"=> $meal["CDSERVFILI"],
            "NRCARDAPIO"=> $meal["NRCARDAPIO"],
            "DTCARDAPIO"=> DateUtil::truncDate($meal["DTCARDAPIO"]),
            "CDCLIENTE"=> null,
            "CDDESTFILI"=> null,
            "CDTURNO"=> $shift["CDTURNO"],
            "QTCOMEORIGINAL"=> $meal["QTCOMEORIGINAL"],
            "QTCOMEALTERADA"=> $shift["COMENSAIS"],
            "QTTOTALALTERADA"=> $meal["TOTAL"],
            "IDQTDEAPROVADA"=> $meal["IDQTDEAPROVADA"],
            "DSPARAMETROS"=> $dsparametros,
            "NRORG"=> $meal["NRORG"],
            "IDATIVO"=> "S",
            "DTINCLUSAO"=> DateUtil::currentDateTimeString(),
            "DTULTATU"=> DateUtil::currentDateTimeString(),
            "NRORGINCLUSAO"=> $meal["NRORG"],
            "CDOPERINCLUSAO"=> $meal["CDOPERADOR"],
            "NRORGULTATU"=> $meal["NRORG"],
            "CDOPERULTATU"=> $meal["CDOPERADOR"],
            "CDPRATO"=> null
        );

        $insert = "INSERT INTO CONFIRMAREFE (NRCONFIRMAREFE, CDFILIAL, CDSERVFILI, NRCARDAPIO, DTCARDAPIO, CDPRATO, CDCLIENTE, CDDESTFILI, CDTURNO, QTCOMEORIGINAL, QTCOMEALTERADA, QTTOTALALTERADA, IDQTDEAPROVADA, DSPARAMETROS, NRORG, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU) VALUES (:NRCONFIRMAREFE, :CDFILIAL, :CDSERVFILI, :NRCARDAPIO, :DTCARDAPIO, :CDPRATO, :CDCLIENTE, :CDDESTFILI, :CDTURNO, :QTCOMEORIGINAL, :QTCOMEALTERADA, :QTTOTALALTERADA, :IDQTDEAPROVADA, :DSPARAMETROS, :NRORG, :IDATIVO, :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU)";
        $sth = $this->connection->prepare($insert);
        $sth->execute($log);
    }
}