module.exports = function(grunt) {

    var srcFiles = {
        js: [
            "js/util/**/*.js",
            "js/repositories/**/*.js",
            "js/services/**/*.js",
            "js/controllers/**/*.js"
        ],
        css: [
        	"css/**/*.css"
        ],
        libsCSS: [
			"./bower_components/red-ui/dist/rui.css",
			"./bower_components/red-ui-fields/dist/rui-fields.css",
			"./bower_components/red-ui-header/dist/rui-header.css",
			"./bower_components/red-ui-list/dist/rui-list.css",
			"./bower_components/red-ui-tabs/dist/rui-tabs.css",
			"./bower_components/red-ui-footer/dist/rui-footer.css",
			"./bower_components/red-ui-splash/dist/rui-splash.css"
		],
        libsJS: [
            "./bower_components/zh-customizations-service/dist/zhCustomizationsService.js",
            "./bower_components/zh-filter-preferences/dist/zhFilterPreferences.js",
            "./bower_components/zh-cache-bypass/src/CacheBypass.js",
            "./bower_components/red-ui/dist/rui.js",
            "./bower_components/red-ui-fields/dist/rui-fields.js",
            "./bower_components/red-ui-header/dist/rui-header.js",
            "./bower_components/red-ui-list/dist/rui-list.js",
            "./bower_components/red-ui-tabs/dist/rui-tabs.js",
            "./bower_components/red-ui-footer/dist/rui-footer.js"
        ]
    };
    var typeOfScope = ['Dev', 'Merge', 'Test'];
    var jshintOptions = {
        "globals": {
            "Configuration": true,
            "moment": true,
            "$": true,
            "console": true,
            "Util": true,
            "ZhUtil": true,
            "angular": true,
            "dataSourceHolder": true,
            "$rootScope": true,
            "getLocalVar": true,
            "module": true,
            "GeneralMethods": true,
            "metaDataFactory": true,
            "setLocalVar": true,
            "localforage": true,
            "messages": true,
            "catchError": true,
            "requestErrorCallback": true,
            "buildUserAndHeader": true,
            "RuiNativeInterface": true,
            "require": true
        },
        "unused": true,
        "undef": true,
        "browser": true,
        "bitwise": true,
        "curly": true,
        "eqeqeq": true,
        "freeze": true,
        "loopfunc": false,
        "debug": true,
        "noempty": true,
        "nonew": true,
        "trailing": true,
        "reporter": require('jshint-html-reporter'),
        "reporterOutput": './logs/dev.html'
    };

    function setSpecific(currScope) {
        switch (true) {
            case (currScope === typeOfScope[1]): //merge
                jshintOptions.devel = true;
                break;
            case (currScope === typeOfScope[2]): //test
                jshintOptions.devel = true;
                break;
            default: //dev
                jshintOptions.devel = false;
        }
        return jshintOptions;
    }
    grunt.initConfig({
        jshint: {
            merge: {
                src: ['Gruntfile.js', srcFiles.js],
                "options": setSpecific(typeOfScope[1])
            },
            dev: {
                src: ['Gruntfile.js', srcFiles.js],
                "options": setSpecific(typeOfScope[0])
            }
        },
        jsbeautifier: {
            final: {
                src: ['Gruntfile.js', srcFiles.js],
                options: {
                    js: {
                        keepArrayIndentation: true,
                        maxPreserveNewlines: 0,
                        preserveNewlines: false
                    }
                }
            },
            dev: {
                src: ['Gruntfile.js', srcFiles.js],
                options: {
                    js: {
                        keepArrayIndentation: true,
                        maxPreserveNewlines: 2
                    }
                }
            }
        },
        concat: {
            css: {
                options: {
                    process: function(src, filepath) {
                        return '/* FILE: ' + filepath + ' */\n' + src + '\n';
                    }
                },
                src: srcFiles.css,
                dest: 'dist/app.css'
            },
            js: {
                options: {
                    process: function(src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: srcFiles.js,
                dest: 'dist/app.js'
            },
            libsJS: {
                options: {
                    process: function(src, filepath) {
                        return '// FILE: ' + filepath + '\n' + src + '\n';
                    }
                },
                src: srcFiles.libsJS,
                dest: 'dist/libs.js'
            },
            libsCSS: {
                options: {
                    process: function(src, filepath) {
                        return '/* FILE: ' + filepath + ' */\n' + src + '\n';
                    }
                },
                src: srcFiles.libsCSS,
                dest: 'dist/libs.css'
            }
        },
        uglify: {
            options: {
                mangle: false,
                compress: false,
                report: 'min',
                // the banner is inserted at the top of the output
                banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
            },
            dist: {
                files: {
                    'dist/app.min.js': srcFiles.js,
                    'dist/libs.min.js': srcFiles.libs
                }
            }
        },
        cssmin: {
            dist: {
                src: srcFiles.css,
                dest: 'dist/app.min.css'
            }
        },
        watch: {
            js: {
                files: srcFiles.js,
                tasks: ['jsbeautifier:dev', 'less', 'concat:js', 'copy', 'buildEnvironment', 'jshint:dev', 'jshint:templatesDev']
            },
            css: {
                files: srcFiles.css,
                tasks: ['concat:css']
            }
        },
        // NÂO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
        zhIdGenerator: {
            options: {
                jsonPath: ["./json"]
            }
        },
        buildEnvironment: {
            development: {
                configFile: "./config/devEnvironment.json"
            },
            production: {
                configFile: "./config/prodEnvironment.json"
            },
            test: {
                configFile: "./config/testEnvironment.json"
            },
            srcFolders: ["../backend/config/", "./assets/js", "./assets/json", "./config", "./", "./assets"]
        },
        less: {
            development: {
                options: {
                    compress: true,
                    modifyVars: {
                        splashColor: '#FFFFFF',
                        primaryColor: '#CA3836',
                        accentColor: '#CA3836',
                        lightPrimaryColor: '#FFCDD2',
                        textIconsColor: '#FFFFFF',
                        primaryText: '#212121',
                        secondaryText: '#757575',
                        dividerColor: '#BDBDBD',
                        errorColor: "#DD2C00",
                        fontFamily: 'Roboto, san serif',
                        defaultFontSize: '15px',
                        cartRowSelected: '#AAAAAA',
                        darkPrimaryColor: '#242c2f',
                        listRowBgColor: '#FFFFFF'
                    }
                },
                files: {
                    'bower_components/red-ui/dist/rui.css': "bower_components/red-ui/src/assets/css/**/*.less",
                    'bower_components/red-ui-fields/dist/rui-fields.css': "bower_components/red-ui-fields/src/assets/css/**/*.less",
                    'bower_components/red-ui-header/dist/rui-header.css': "bower_components/red-ui-header/src/assets/css/**/*.less",
                    'bower_components/red-ui-list/dist/rui-list.css': "bower_components/red-ui-list/src/assets/css/**/*.less",
                    'bower_components/red-ui-tabs/dist/rui-tabs.css': "bower_components/red-ui-tabs/src/assets/css/**/*.less",
                    'bower_components/red-ui-footer/dist/rui-footer.css': "bower_components/red-ui-footer/src/assets/css/**/*.less",
                    'bower_components/red-ui-splash/dist/rui-splash.css': "bower_components/red-ui-splash/src/assets/css/**/*.less",
                    'css/custom.css': "css/custom.less"
                }
            }
        }
    });

    grunt.loadNpmTasks('grunt-contrib-jshint');
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks('grunt-contrib-cssmin');
    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-less');
    grunt.loadNpmTasks('grunt-contrib-watch');
    grunt.loadNpmTasks('grunt-zh-id-generator');
    grunt.loadNpmTasks('grunt-zh-environment-builder');
    grunt.loadNpmTasks("grunt-jsbeautifier");
    grunt.loadNpmTasks("grunt-notify");

    // Default task.
    grunt.registerTask('default', ['jsbeautifier:dev', 'jshint:dev', 'less', 'concat', 'buildEnvironment']);

    grunt.registerTask('merge', ['jsbeautifier:dev', 'jshint:merge', 'less', 'concat', 'buildEnvironment']);
    grunt.registerTask('test', ['jsbeautifier:dev', 'less', 'concat', 'buildEnvironment:test']);
    grunt.registerTask('prod', ['jsbeautifier:final', 'less', 'concat', 'buildEnvironment:production', 'cssmin']);
    grunt.registerTask('devCoPilot', ['watch', 'notify']);

};
