/**
 * The controller of header template
 * @param {TemplateManager} templateManager - An instance of TemplateManager
 */
Configuration(function(eventEngine) {
	eventEngine.registerEvent("WidgetOnBack", "widgetOnBack");
});

function ruiHeaderController($scope, templateManager, ScreenService) {

	var self = this;

	var showAsActionTypes = {
		always: "always",
		list: "list",
		view: "view",
		edit: "edit",
		checked_rows: "checked_rows",
		never: "never",
		never_list: "never_list",
		never_view: "never_view",
		never_edit: "never_edit",
		never_checked_rows: "never_checked_rows",
		back: "back",
		back_list: "back_list",
		back_view: "back_view",
		back_edit: "back_edit",
		back_checked_rows: "back_checked_rows"
	};
	var searchStatus = {
		opened: "SO",
		closed: "SC",
		notSearching: "NS"
	};


	/**
	 * Retrieves actions conllection length
	 * @param  {String} type - the action type
	 * @return {Integer} Returns the length of actions collection by type
	 */
	this.getActionsLengthByType = function (type) {
		return getActionsByType(type).length;
	};

	/**
	 * Retrieves actions by type
	 * @param  {String} type - the action type
	 * @return {Array} Array of actions by type
	 */
	this.getActionsByType = getActionsByType;
	function getActionsByType(type) {
		return getActions().filter(function (action) {
			return findShowAsAction(type, action) && action.isVisible;
		});
	}

	function findShowAsAction (types, action) {
		var exist = false;
		types = types.split('|');
		var asAction = action.showAsAction.split('|');
		for  (var i = asAction.length - 1; i >= 0 && exist === false; i--) {
			exist = showAsActionTypes[types[0]] == asAction[i] ? true : showAsActionTypes[types[1]] == asAction[i] ? true : false;
		}
		return exist;
	}

	this.getActionsVisibleLength = function(nameArrayActions) {
		return $scope[nameArrayActions].filter(function(action) {
			return action.isVisible;
		}).length;
	};

	/**
	 * Toggle dropdown visibility with fast animation
	 */
	this.toggleHeaderDropdown = function () {
		$scope.$root.headerDropdownOpened = !$scope.$root.headerDropdownOpened;
	};
	window.toggleHeaderDropdown = this.toggleHeaderDropdown;

	/**
	 * Retrieves the container's active widget
	 * @return {Widget} Active widget
	 */
	this.getActiveWidget = getActiveWidget;
	function getActiveWidget() {
		return getContainer().getActiveWidget();
	}

	/**
	 * Retrieves active widget's current row of current container
	 * @return {Object} Current row
	 */
	this.getCurrentRow = function () {
		return getActiveWidget().currentRow;
	};

	/**
	 * Execute selected action and toggle dropdown
	 * @param  {Action} action - selected action
	 * @param  {Function} actionClick - Zeedhi action click event
	 */
	this.selectDropdownOption = function (action, actionClick) {
		actionClick(action, getActiveWidget());
	};

	/**
	 * Attach actions and methods in container
	 */
	this.updateContainer = updateContainer;
	function updateContainer() {
		var container = getContainer();
		updateCurrentWidget(container);
		container.eachWidget(function(widget){
			widget.getHeaderActions = getHeaderActions;
		});
	}


	function updateCurrentWidget (container) {
		var widget = getActiveWidget();
		widget.headerActions = widget.headerActions || [];
		widget.headerActions = factoryActions(container, widget.headerActions);
		widget.searchCriteria =  widget.searchCriteria || {};
		updateSearching(widget);
		updateHeaderShadow(widget);
		updateHeaderForTWWI(widget, searchStatus.notSearching);
	}
	function updateHeaderForTWWI(widget,search){
		var fatherTemp = widget.getParent().template;
		var srcTemp = 'rui-tabs#widget/tabbedWidgetWithImage.html';
		var ruiHeader = $('.rui-header');
		var ruiContainer = $('.container[zh-fixed]');
		if(search ===  searchStatus.notSearching){
			if(fatherTemp === srcTemp){
				ruiHeader.addClass('header-over-tabs');
				ruiContainer.css('top',0);
			}else{
				ruiHeader.removeClass('header-over-tabs');
				ruiHeader.removeClass('new-bg-header');
				ruiContainer.css('top',ruiHeader.css('height'));
			}
		}else if (search ===  searchStatus.opened && fatherTemp === srcTemp){
			$('.rui-TWWI-Main-img').hide();
			$('.rui-header').removeClass('header-over-tabs');
			$('.rui-header').removeClass('new-bg-header');
			ruiContainer.css('top',ruiHeader.css('height'));
		}
		else if (search ===  searchStatus.closed && fatherTemp === srcTemp){
			$('.rui-TWWI-Main-img').show();
			$('.rui-header').addClass('header-over-tabs');
			$('.rui-header').addClass('new-bg-header');
			ruiContainer.css('top',0);
		}
	}
	function updateHeaderShadow(widget) {
		if (widget.parent && isTabbedTemplate(widget.parent.template)) {
			$('.rui-header').removeClass('searchShadow');
		} else {
			$('.rui-header').addClass('searchShadow');
		}
	}

	function isTabbedTemplate(template) {
		return template.search('tabbedWidget.html') !== -1 || template.search('tabbedWidgetWithImage.html') !== -1;
	}

	function updateSearching(widget) {
		if (widget.showSearch && widget.headerSearchBy) {
			$scope.searchVal = widget.searchCriteria._ALL;
		}

		if (widget.keepHeaderSearchingMode && searchFilled()) {
			$scope.$root.searching = true;
		} else if (searchFilled() && widget.mode == "list") {
			$scope.$root.searching = true;
		} else {
			$scope.$root.searching = false;
		}
	}
	this.getNeverMode = getNeverMode;
	function getNeverMode (){
		var widget = getActiveWidget();
		if (widget.hasCheckedRows()) {
			return "never_checked_rows";
		} else {
			if(widget.mode === "edit") {
				return "never_edit|never";
			} else if(widget.mode === "view") {
				return "never_view|never";
			} else if(widget.mode === "list") {
				return "never_list|never";
			}
		}
	}
	this.getBackMode = getBackMode;
	function getBackMode (){
		var widget = getActiveWidget();
		if(widget.hasCheckedRows()){
			return "back_checked_rows";
		} else {
			if(widget.mode === "edit") {
				return "back_edit|back";
			} else if(widget.mode === "view") {
				return "back_view|back";
			} else if(widget.mode === "list") {
				return "back_list|back";
			}
		}
	}
	this.getCenterMode = getCenterMode;
	function getCenterMode(){
		var widget = getActiveWidget();
		if(widget.hasCheckedRows()){
			return "checked_rows";
		} else {
			if(widget.mode === "edit") {
				return "edit|always";
			} else if(widget.mode === "view") {
				return "view|always";
			} else if(widget.mode === "list") {
				return "list|always";
			}
		}
	}

	/**
	 * Build actions applying mobile and default properties
	 * @param  {Container} container - current container
	 * @param  {Array} actions - array of actions to build
	 * @return {Array} An array of builded actions
	 */
	function factoryActions(container, actions) {
		return actions.map(function (action) {
			action = Util.extend(new Action(), action);
			_extendMobileProperties(action);
			action.container = container;
			action.widget = getActiveWidget();
			return action;
		});
	}

	/**
	 * Set mobile properties if it's a mobile device
	 * @param  {Object} obj - action object
	 */
	function _extendMobileProperties(obj) {
		if (Util.isMobile() && obj.mobile) {
			var mobile = obj.mobile;
			Object.keys(mobile).forEach(function extendMobileProperty(propertyToExtend) {
				if (obj.replaceByMobile || !obj[propertyToExtend] || typeof obj[propertyToExtend] !== 'object') {
					obj[propertyToExtend] = mobile[propertyToExtend];
				} else {
					obj[propertyToExtend] = Util.extend(obj[propertyToExtend], mobile[propertyToExtend]);
				}
			});
		}
	}

	/**
	 * Retrieves header actions of current container
	 * @return {Array} An array of actions
	 */
	function getActions() {
		return getActiveWidget().headerActions || [];
	}

	/**
	 * Retrieves current container
	 * @return {Container} Current container
	 */
	function getContainer() {
		return templateManager.container;
	}

	function widgetOnChangeMode(args) {
		if (getContainer()) {
			var widget = getActiveWidget();
			if (widget.mode != "list") {
				$scope.$root.searching = false;
			}
			updateContainer(getContainer());
		}
	}

	$scope.$watch(function() {
		return getContainer() ? getActiveWidget().mode : null;
	}, widgetOnChangeMode);

	$scope.$watch(function() {
		return getContainer() ? getActiveWidget().hasCheckedRows() : null;
	}, function(args) {
		if (getContainer())
			updateCurrentWidget(getContainer());
	});

	$scope.$watch(function() {
		return getContainer() ? getActiveWidget() : null;
	}, function(args){
		if (getContainer())
			updateCurrentWidget(getContainer());
	});

	$scope.$watch(function() {
		return getContainer();
	}, function(args){
		if (getContainer()){
			updateContainer();
		}
	});
	 
	/**
	 * Change header layout to search mode
	 * @param  {Boolean} type - true: Search mode, false: Normal mode
	 */
	this.toggleHeaderSearch = function(type) {
		var widget = getActiveWidget();
		if(type) {
			$scope.$root.searching = true;
			$('.rui-tabsMobileCont').css('top','-50px');
			$('.rui-TWWI-mainContainer').css('top','-50px');
			setTimeout(function() {
				$('#search-input').attr("placeholder", ScreenService.i18n("Search..."));
				$('#search-input').focus();
				updateHeaderForTWWI(widget, searchStatus.opened);
			}, 150);
		} else {
			if (searchFilled()) {
				clearSearch();
			}
			$('.rui-tabsMobileCont').css('top','0');
			$('.rui-TWWI-mainContainer').css('top','0');
			$('#search-input').attr("placeholder", "");
			$scope.$root.searching = false;
			updateHeaderForTWWI(widget, searchStatus.closed);
		}
	};
	window.toggleHeaderSearch = this.toggleHeaderSearch;

	/**
	 * Update the searchCriteria of active widget
	 */
	this.searchInputChange = searchInputChange;
	function searchInputChange() {
		var inputVal = $scope.searchVal;
		var widget = getActiveWidget();
		var showSearch = widget.showSearch;
		if (showSearch && !widget.headerSearchBy) {
			console.error("ERROR: headerSearchBy property not defined on widget '" + widget.name + "'.");
		} else {
			widget.headerSearchBy = Array.isArray(widget.headerSearchBy) ? widget.headerSearchBy : [widget.headerSearchBy];
			if (!Util.isEmptyOrBlank(inputVal)) {
				widget.searchCriteria._ALL = inputVal;
			} else {
				delete widget.searchCriteria._ALL;
			}

			if ($scope.searchTimeout) {
				clearTimeout($scope.searchTimeout);
			}
			$scope.searchTimeout = setTimeout(setSearchFilter.bind(this), 500);
		}
	}

	/**
	 * Set dataSourceFilter of widget according to searchCriteria
	 */
	function setSearchFilter() {
		var widget = getActiveWidget();

		if (!Object.keys(widget.searchCriteria).length) {
			widget.dataSourceFilter = widget.defaultDataSourceFilter || widget.dataSourceFilter || [];
		} else {
			if (!widget.defaultDataSourceFilter) {
				widget.defaultDataSourceFilter = widget.dataSourceFilter || [];
			}
			var name = widget.headerSearchBy.join("|");
			var operator = "LIKE_ALL";
			var value = '%'+widget.searchCriteria._ALL+'%';
			addFilterCondition(widget, name, operator, value);
		}
		widget.reload();
	}

	/**
	 * Add condition to widget's dataSourceFilter
	 * @param {Widget} widget
	 * @param {String} name
	 * @param {String} operator
	 * @param {String} value
	 */
	function addFilterCondition(widget, name, operator, value) {
		var dataSourceFilter = widget.dataSourceFilter || [];
		dataSourceFilter = dataSourceFilter.filter(function(row) {
			return row.name != name;
		});
		// if (!Util.isUndefinedOrNull(inDataSourceFilter[0])) {
		// 	dataSourceFilter.splice(inDataSourceFilter[0], 1);
		// }
		dataSourceFilter.push({
			"name": name,
			"operator": operator,
			"value": value
		});
		widget.dataSourceFilter = dataSourceFilter;
	}

	/**
	 * Manage visibility of clear search icon
	 * @return {Boolean} Status of widget's searchCriteria property
	 */
	this.searchFilled = searchFilled;
	function searchFilled() {
		var widget = getActiveWidget();
		if (widget.searchCriteria && widget.headerSearchBy) {
			return !Util.isEmptyOrBlank(widget.searchCriteria._ALL);
		} else {
			return false;
		}
	}

	/**
	 * Clear value of search input
	 */
	this.clearSearch = clearSearch;
	function clearSearch() {
		$scope.searchVal = "";
		searchInputChange();
	}

	function getHeaderActions(name) {
		return this.headerActions.filter(function(item) {
			return item.name === name;
		}).shift();
	}

	this.getTitle = function() {
		var container = templateManager.container;
		var widget = getActiveWidget();
		var headerTitle = widget.title || container.label;

		if (widget.showCheckedRowsCounterOnHeader && widget.getCheckedRows().length) {
			headerTitle = widget.getCheckedRows().length;
		}

		return headerTitle;
	};

	this.menuActionVisibility = function() {
		var container = templateManager.container;
		var widget = getActiveWidget();

		if (!container.showMenu || widget.getCheckedRows().length) {
			return false;
		} else {
			if (widget.showMenu === undefined) {
				return true;
			} else {
				return widget.showMenu;
			}
		}
	};
}