function ruiTabbedController($scope, $rootScope, $element, eventEngine, $timeout, ZHPromise, ScreenService){
	var self = this;
	var WIDGET_ON_ACTIVATE_EVENT_NAME = 'WidgetOnActivate';
	$scope.tabbedWidget = $scope.$parent.widget;
	$scope.widgets = $scope.tabbedWidget.widgets;
	var tabs = getTabsList($scope.tabbedWidget);
	$scope.setDefaultTab = function(defaultTab) {
		return ZHPromise.when().then(function() {
			var tabbedWidget = $scope.tabbedWidget;
			if (!defaultTab) {
				if (tabbedWidget.widgets.length) {
					defaultTab = tabbedWidget.widgets[0];
				}
			} else if (Util.isString(defaultTab)) {
				defaultTab = tabbedWidget.container.getWidget(defaultTab);
			}
			if (defaultTab) {
				tabbedWidget.currentWidget = defaultTab;
			}
			if (tabbedWidget.currentWidget) {
				tabbedWidget.currentWidget.activate();
				$timeout(function(){
					setSizes(tabs);	
				}, 300);
			}
			$scope.currentWidget = tabbedWidget.currentWidget;
		});
	};
	var onActivateBoundEventIndex = eventEngine.bindEvent($scope.widget, WIDGET_ON_ACTIVATE_EVENT_NAME, onWidgetActivate);
	$scope.$on('$destroy', function() {
		eventEngine.unbindEvent($scope.widget, WIDGET_ON_ACTIVATE_EVENT_NAME, onActivateBoundEventIndex);
	});
	function setSizes (tabs){
		var tabContainer = $element.find('.rui-TBC-tabs-header-container');
		var marker = $element.find('.rui-active-tab-marker');
		var scrWidth = parseInt($(window).width());
		var totalWidth;
		if(scrWidth > 990){
			totalWidth = 120;
		}else{
			if (tabs.length >2){
				totalWidth = parseInt(tabContainer.css('width'))/3;
			}
			else if (tabs.length === 2){
				totalWidth = parseInt(tabContainer.css('width'))/2;
			}else{
				totalWidth = '100%';
			}
		}
		tabs.forEach(function(tabSG){
			var tabName = tabSG.name;
			$('#header_tab_'+tabName).css('width',totalWidth);
		});
		marker.css('width',totalWidth);
		var actvW = $scope.tabbedWidget.currentWidget;
		$timeout(function(){
			fixPosition(actvW);
		}, 150);
	}
	$(window).resize(function(){
		setSizes (tabs);
	});
	function getTabsList(tabbedWidget) {
		var tabs = tabbedWidget.widgets.slice(0);
		if (tabbedWidget.template.indexOf('tabbedWidget') === -1) {
			tabs.push(tabbedWidget);
			$timeout(function(){
				setSizes(tabs);	
			}, 300);
		}
		return tabs;
	}
	function onWidgetActivate() {
		if ($scope.widget.currentWidget) {
			$scope.widget.currentWidget.activate();
		}
	}
	function doTheMoveAnimation(choosenTab){
		var tabsElement = $element.find('.rui-TBC-tabs-header');
		var tabbedContainer = $element.find('.rui-tabbedContainer');
		var choosenElement = $element.find('#header_tab_'+choosenTab.name);
		var marker = $element.find('.rui-active-tab-marker');
		tabsElement.removeClass('rui-TBC-activeTab');
		choosenElement.addClass('rui-TBC-activeTab');
		$(tabbedContainer).animate({scrollLeft: $(choosenElement).position().left}, 200);
		marker.css('left',$(choosenElement).position().left);
		$('.rui-tabsMobileCont').css('top','0');
	}
	function fixPosition (choosenTab){
		var tabsElement = $element.find('.rui-TBC-tabs-header');
		var tabbedContainer = $element.find('.rui-tabbedContainer');
		var choosenElement = $element.find('#header_tab_'+choosenTab.name);
		var marker = $element.find('.rui-active-tab-marker');
		tabsElement.removeClass('rui-TBC-activeTab');
		choosenElement.addClass('rui-TBC-activeTab');
		$(tabbedContainer).animate({scrollLeft: $(choosenElement).position().left}, 200);
		marker.css('left',$(choosenElement).position().left);
	}
	$scope.onClickTab = function(tabbedWidget, childWidget) {
		if(!$rootScope.searching){
			var tabClickPromise;
			if (childWidget.isEnabled()) {
				tabClickPromise = closeGreaterSwipes(tabbedWidget, childWidget);
				tabbedWidget.currentWidget = childWidget;
				childWidget.activate();
				doTheMoveAnimation(childWidget);
			}
			return tabClickPromise;
		}
	};
	function closeGreaterSwipes(tabbedWidget, childWidget) {
		var closeSwipePromise;
		if (tabbedWidget.currentWidget !== childWidget) {
			var levelToClose = ScreenService.getSwipeLevel(childWidget) + 1;
			if (ScreenService.isSwipeOpen(levelToClose)) {
				closeSwipePromise = ScreenService.closeSwipe(levelToClose).then(function() {
					childWidget.activate();
				});
			}
		}
		return closeSwipePromise;
	}
	var updateTabbed = function(widget) {
		if (widget && widget.onEnterTab) {
			$scope.currentWidget = widget;
			widget.onEnterTab();
		}
	};
	$scope.$parent.$watch('widget.currentWidget', updateTabbed);
	function getNextTab(nextIndex, direction) {
		var nextTab = null;
		while(tabs[nextIndex] && !tabs[nextIndex].enabled) {
			nextIndex += direction;
		}
		if (nextIndex >= 0 && nextIndex < tabs.length) {
			nextTab = tabs[nextIndex];
		}
		return nextTab;
	}
	$scope.scrollSwipe = function(tab, direction){
		var currentIndex = $scope.widgets.indexOf(tab);
		var nextIndex = currentIndex + direction;
		var nextTab = getNextTab(nextIndex, direction);
		if(nextTab){
			if (nextTab !== null) {
				$scope.onClickTab($scope.$parent.widget,nextTab);
			}
		}
	};
}