function SelectController($scope, $rootScope, metaDataFactory, templateManager, ScreenService, ZhUtil, SelectService, ZHPromise, maskEngine, EventChildPropagation) {

	var self = this;

	function extendBehavior(){
		Util.extend(self, new ListTemplatable());
	}
	extendBehavior();

	var _openField = function(field) {

		var defer = ZHPromise.defer();
		var actions = [{
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"name": "ActionEvent",
				"code": "ScreenService.closePopup(); args.owner.widget.afterClosePopup(args.owner.widget.selectField);",
				"id": 9998
			}]
		}];

		if (field.showAddButtonAs == 'action') {
			actions.push({
				"icon": "plus",
				"label": field.addButtonText,
				"showAsAction": "always|edit|checked_rows|view",
				"isVisible": true,
				"showOnForm": true,
				"showOnList": true,
				"events": [{
					"name": "ActionEvent",
					"code": "args.owner.widget.onAdd()",
					"id": 9999
				}]
			});
		}

		innerSearch("", field);

		var template;
		if (!field.showSelectAs) {
			template = "widget/list.html";
		} else if (field.showSelectAs === "tree") {
			template = "widget/tree/grid.html";
		} else {
			template = "widget/" + field.showSelectAs + ".html";
		}

		var fields;
		if (field.fields && field.fields.length) {
			fields = field.fields;
		} else {
			fields = [{
				"label": field.sourceLabel,
				"name": field.descriptionField,
				"showOnList": true,
				"mask": field.mask
			}];
		}

		var isToReload = !!(field.selectWidget && Util.objectHasAnyValue(field.selectWidget.searchCriteria));
		if (!field.selectWidget) {
			var widgetEvents = [{
				"name": "WidgetAfterMoveRow",
				"code": "args.owner.setSelected(args);",
				"id": 9999
			}];
			field.selectWidget = metaDataFactory.widgetFactory({
				"id": 0,
				"name": "Widget",
				"label": field.label,
				"template": template,
				"container": field.widget.container,
				"isVisible": true,
				"popupNoBlock": false,
				"showDescriptionOrder": false,
				"floatingControl": {
					"openSearch": !Util.isMobile(),
					"defaultSearchField": field.defaultSearchField || field.descriptionField,
					"pdfAction": false,
					"xlsAction": false,
					"searchAction": true,
					"customizationAction": {
						"filter": false,
						"layout": false,
						"view": false
					}
				},

				"parentProperty": field.parentProperty,
				"primaryProperty": field.primaryProperty,
				"expandingProperty": field.expandingProperty,
				"breadcrumbProperty": field.breadcrumbProperty,

				"order": field.order,
				"reverse": field.reverse,
				"bringGroupsCollapsed": field.bringGroupsCollapsed,
				"orderFromGroups": field.orderFromGroups,
				"selectField": field,
				"showCheckbox": false,
				"onAdd": field.onAdd,
				"showAddButtonAs": field.showAddButtonAs,
				"addButtonText": field.addButtonText,
				"noMatchText": field.noMatchText,
				"searchNoMatchText": field.searchNoMatchText,
				"fields": fields,
				"dataSource": field.dataSource,
				"dataSourceFilter": field.dataSourceFilter,
				"itemsPerPage": field.itemsPerPage,
				"searchWithPagination": Util.isUndefinedOrNull(field.searchWithPagination) ? true : field.searchWithPagination,
				"hasPagination": Util.isUndefinedOrNull(field.hasPagination) ? true : field.hasPagination,
				"onDemand": field.onDemand,
				"allowSelectOnlyLastChild": field.allowSelectOnlyLastChild,
				"searchCriteria": field.searchCriteria,

				"actions": actions,
				"events": EventChildPropagation.propagate(widgetEvents, field),

				"styles": field.selectWidgetStyles
			});
			field.selectWidget.setSelected = setSelected;
			field.selectWidget.afterClosePopup = afterClosePopup;
			isToReload = !!(field.dataSource && field.dataSource.lazyLoad);
		}

		if (isToReload) {
			field.selectWidget.searchCriteria = {};
			field.selectWidget.searchList = "";
			field.reload().then(defer.resolve, defer.resolve);
		} else {
			defer.resolve();
		}

		ScreenService.openPopup(field.selectWidget);
		return defer.promise;
	};

	$scope.openField = function(field) {
		if (!field.isReadOnly()) {
			if (field.beforeSelectOpen) {
				var e = field.beforeSelectOpen();
				if (e.isDefaultPrevented()) {
					return;
				}
			}
			// remove focus from element
			if (field.element) {
				field.element.blur();
				window.getSelection().removeAllRanges(); //bug webkit
			}

			_openField(field).then(function() {
				if (field.afterSelectOpen) {
					field.afterSelectOpen();
				}
			});
		}
	};

	$scope.field.openField = function () {
		$scope.openField($scope.field);
	};

	$scope.field.clearByOutData = (function () {
		for (var prop in this.outData) {
			this.widget.currentRow[this.outData[prop]] = "";
		}
	}).bind($scope.field);

	$scope.field.clearByName = (function () {
		this.widget.currentRow[this.name] = "";
	}).bind($scope.field);

	if ($scope.selectField) {
		$scope.selectField.openField = function() {
			$scope.openField($scope.selectField);
		};
	}

	var setChanged = function(field) {
		field.widget.currentRow.changed = true;
	};

	$scope.clearField = function(field) {
		if (field.isReadOnly()) {
			return;
		}

		field.clearByOutData();
		field.clearByName();
		field.dataSource.clearCheckedRows();
		setChanged(field);
		if (field.change) {
			field.change();
		}
	};

	$rootScope.search = function(selectListSearch, field) {
		var minLength = field.minimumInputLength || 0;
		if (selectListSearch.length > minLength) {
			innerSearch(selectListSearch, field);
		}
	};

	function innerSearch(selectListSearch, field) {
		if (Util.isArray(field.filterDefinitions) && field.filterDefinitions.length) {
			var dataSource = field.dataSource;
			var filter = ZhUtil.buildFiltersInField(field, selectListSearch);
			dataSource.filter(filter, function(data) {
				dataSource.data = data;
				templateManager.updateTemplate();
			});
		}
	}

	function setSelected(args) {
		if(!$scope.field.allowSelectOnlyLastChild || ($scope.field.allowSelectOnlyLastChild && self.isTree($scope.field.selectWidget) && isLastChild(args.row))){
			ScreenService.closePopup();

			var row = args.row;
			var field = args.owner.selectField;

			if (field.element) {
				field.element.focus();
			}

			(field.outData ? changeByOut : changeByName)(field, row);
			field.dataSource.clearCheckedRows();
			field.dataSource.addCheckedRows(row);
			setChanged(field);

			if (field._afterSelected) {
				field._afterSelected(row);
			}
			if (field.change) {
				field.change();
			}
		}
	}

	var isLastChild = function(row){
		return row._childCount === 0;
	};

	function afterClosePopup(field) {
		if (field.element) {
			field.element.focus();
		}
	}

	function changeByName(field, row) {
		var valueField = field.valueField ? field.valueField : field.name;
		var descriptionField = field.descriptionField ? field.descriptionField : field.name;
		field.widget.currentRow[valueField] = row[valueField];
		field.widget.currentRow[field.name] = row[descriptionField];
	}

	function changeByOut(field, row) {
		if (field.widget.currentRow) {
			var outData = field.outData || {};

			Object.keys(outData).forEach(function(key) {
				if (Array.isArray(outData[key])) {
					outData[key].forEach(function(valueKey) {
						field.widget.currentRow[valueKey] = row[key];
					});
				} else {
					field.widget.currentRow[outData[key]] = row[key];
				}
			});
		}
	}

	this.getSelectionText = function getSelectionText(fieldRow) {
		var mask = $scope.field.mask;
		if(mask){
			var mappedMasks = maskEngine.getMappedMasks();
			var params = mask.params || {};
			var maskMethod = mappedMasks[mask.type].format;
			if (fieldRow && mask && maskMethod) {
				fieldRow = maskMethod.apply(maskEngine, [fieldRow, params]);
			}
		}
		if ($scope.field.placeholder && !fieldRow) {
			selectionText = $scope.field.placeholder;
		} else {
			selectionText = fieldRow;
		}
		return selectionText;
	};

	$scope.maintainWindow = function(field) {
		if (field.maintainWindow) {
			var maintainWindowFilter = SelectService.buildMaintainFilter(field);
			ScreenService.openWindow(field.maintainWindow, maintainWindowFilter);
		}
	};

	$scope.$watchCollection('field.dataSourceFilter', propagateFieldDataSourceFilterToSelectWidget);
	function propagateFieldDataSourceFilterToSelectWidget(fieldDataSourceFilter) {
		if ($scope.field.selectWidget) {
			$scope.field.selectWidget.dataSourceFilter = fieldDataSourceFilter;
		}
	}
}