describe('RequestFactory', function() {

	var templateManager = {
		container: {
			name: 'container',
			getActiveWidget: function() {
				return {
					name: 'widget'
				};
			}
		}
	};
	var requestFactory = new RequestFactory(templateManager);
	var defaultExpectation = {
		origin: {
			containerName: templateManager.container.name,
			widgetName: templateManager.container.getActiveWidget().name
		}
	};

	function expectRequestParams(request, expectation) {
		expect(request.getRequestParams()).toEqual(expectation);
	}

	it('Factory request type filterData', function() {
		var commonProperties = {
			filter: [],
			page: 0,
			itemsPerPage: 40
		};
		var request = requestFactory.factory(Util.extend({
			requestType: 'filterData',
			serviceName: '/filterData',
			callback: null,
			errorCallBack: null,
		}, commonProperties));
		commonProperties.requestType = 'FilterData';
		var expectation = Util.extend(commonProperties, defaultExpectation);
		expectRequestParams(request, expectation);
	});

	it('Factory request type AutoCompleteData', function() {
		var commonProperties = {
			requestType: 'AutoCompleteData',
			value: null
		};
		var request = requestFactory.factory(Util.extend({
			serviceName: '/AutoCompleteData',
			inData: []
		}, commonProperties));
		commonProperties["in"] = [];
		var expectation = Util.extend(commonProperties, defaultExpectation);
		expectRequestParams(request, expectation);
	});

	it('Factory request type Row', function() {
		var commonProperties = {
			requestType: 'Row',
			row: []
		};
		var request = requestFactory.factory(Util.extend({
			serviceName: '/Row'
		}, commonProperties));
		var expectation = Util.extend(commonProperties, defaultExpectation);
		expectRequestParams(request, expectation);
	});

	it('Factory request type File', function() {
		var commonProperties = {
			requestType: 'File',
			row: []
		};
		var request = requestFactory.factory(Util.extend({
			serviceName: '/File'
		}, commonProperties));
		var expectation = Util.extend(commonProperties, defaultExpectation);
		expectRequestParams(request, expectation);
	});

	it('Factory request type CheckRow', function() {
		var commonProperties = {
			requestType: 'CheckRow',
			row: []
		};
		var request = requestFactory.factory(Util.extend({
			serviceName: '/CheckRow'
		}, commonProperties));
		var expectation = Util.extend(commonProperties, defaultExpectation);
		expectRequestParams(request, expectation);
	});

	it('Factory request type Cell', function() {
		var commonProperties = {
			requestType: 'Cell',
			value: null
		};
		var request = requestFactory.factory(Util.extend({
			serviceName: '/Cell'
		}, commonProperties));
		var expectation = Util.extend(commonProperties, defaultExpectation);
		expectRequestParams(request, expectation);
	});

	it('Factory request type dataSource', function() {
		var request = requestFactory.factory({
			requestType: 'dataSource',
			serviceName: '/dataSource',
			dataSource: [],
			callback: null
		});
		var expectation = Util.extend({dataset: [], requestType: 'DataSet'}, defaultExpectation);
		expectRequestParams(request, expectation);
	});

	it('Factory request type Wizard', function() {
		var commonProperties = {
			requestType: 'Wizard',
			dataset: []
		};
		var request = requestFactory.factory(Util.extend({
			serviceName: '/Wizard'
		}, commonProperties));
		var expectation = Util.extend(commonProperties, defaultExpectation);
		expectRequestParams(request, expectation);
	});

	it('Factory request type Empty', function() {
		var request = requestFactory.factory({
			requestType: 'Empty',
			serviceName: '/Empty'
		});
		var expectation = Util.extend({
			requestType: 'Empty'
		}, defaultExpectation);
		expectRequestParams(request, expectation);
	});

});