(function () {

function FilterBuilder(Query, ZHPromise) {
	var _filterType = {};
	var skipFiltering = [];
	var self = this;

	var _buildFilterTypeMap = function (fields, currentRow) {
		fields.forEach(function (field) {
			if (!field.filterProperties) {
				setOperatorFromFieldTemplate(field, currentRow);
			} else {
				var filterProperties = field.filterProperties;
				if (filterProperties.operator) {
					setOperatorFromFieldOperator(_filterType, field);
				} else {
					setOperatorFromFieldTemplate(field, currentRow);
				}
				if (filterProperties.outFilters) {
					Object.keys(filterProperties.outFilters).forEach(function (key) {
						var operator = filterProperties.outFilters[key];
						_filterType[key] = operator ? operator : _filterType[field.name];
					});
				}
			}
		});
	};

	function setOperatorFromFieldOperator(_filterType, field) {
		if (fieldHasSelectTemplate(field)) {
			_filterType[field.outData[field.valueField]] = field.filterProperties.operator;
		} else {
			_filterType[field.name] = field.filterProperties.operator;
		}
	}

	function fieldHasSelectTemplate(field) {
		var template = (field.filterProperties && field.filterProperties.template) || field.template;
		return field.outData && template && ~template.indexOf('select.html');
	}

	var setOperatorFromFieldTemplate = function (field, currentRow) {
		var template = (field.filterProperties && field.filterProperties.template) || field.template;

		if (template && (template.indexOf('between-dates.html') !== -1 || template.indexOf('interval.html') !== -1)) {
			_filterType[field.name] = "BETWEEN";
			var fieldValue = currentRow[field.name];
			if (fieldValue) {
				if (!Object.keys(fieldValue).some(function (element) {
					return fieldValue[element];
				})) {
					if (!~skipFiltering.indexOf(field.name)) {
						skipFiltering.push(field.name);
					}
				}
			}
		} else if (template && (~template.indexOf('number.html') || ~template.indexOf('datetime.html') || ~template.indexOf(
			'calendar.html') || ~template.indexOf('select-native.html'))) {
			_filterType[field.name] = "=";
		} else if (fieldHasSelectTemplate(field)) {
			_filterType[field.outData[field.valueField]] = "=";
		} else if (template && ~template.indexOf('select-multiple.html')) {
			_filterType[field.name] = "IN";
		} else {
			_filterType[field.name] = "LIKE_I";
		}
	};

	var _filterTypeFlow = {
		"BETWEEN": function (query, value) {
			query.between([value.start, value.end]);
		},
		"NOT BETWEEN": function (query, value) {
			query.notBetween([value.start, value.end]);
		},
		"LIKE": function (query, value) {
			query.like('%' + value + '%');
		},
		"LIKE_I": function (query, value) {
			query.likeInsensitive('%' + value + '%');
		},
		"NOT LIKE": function (query, value) {
			query.notLike('%' + value + '%');
		},
		"LIKE ALL": function (query, value) {
			query.likeAll('%' + value + '%');
		},
		"<>": function (query, value) {
			query.notEquals(value);
		},
		"=": function (query, value) {
			query.equals(value);
		},
		">": function (query, value) {
			query.greater(value);
		},
		"<": function (query, value) {
			query.lower(value);
		},
		">=": function (query, value) {
			query.greaterOrEqual(value);
		},
		"<=": function (query, value) {
			query.lowerOrEqual(value);
		},
		"IN": function (query, value) {
			query.in(value);
		},
		"NOT IN": function (query, value) {
			query.notIn(value);
		}
	};

	this.buildFilter = function (fields, currentRow) {
		_filterType = {};
		skipFiltering = [];
		_buildFilterTypeMap(fields, currentRow);
		var query = Query.build();
		for (var key in currentRow) {
			if (typeof currentRow[key] == "boolean") {
				currentRow[key] = currentRow[key].toString();
			}
			if (currentRow[key] && _filterType[key] && (!Array.isArray(currentRow[key]) || currentRow[key].length > 0)) {
				if (!~skipFiltering.indexOf(key)) {
					query.where(key);
					_filterTypeFlow[_filterType[key]](query, currentRow[key]);
				}
			}
		}
		return query.where();
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register("ZhFilterBuilder", FilterBuilder);
});

})();