// FILE: ./bower_components/zh-customizations-service/dist/zhCustomizationsService.js

// FILE: src/config/config.js
(function(){
window.templateUrls = window.templateUrls || [];
var packageName = 'zh-customizations-service';

var hasPackageRegistered = window.templateUrls.some(function(templateUrl){
	return templateUrl.package === packageName;
});

if(!hasPackageRegistered){
	window.templateUrls.push({
		'package': packageName,
		'baseUrl': './bower_components/' + packageName + '/dist/templates/'
	});
}
})();

// FILE: src/registers/registerEvents.js
Configuration(function(eventEngine) {
  eventEngine.registerEvent("FilterPreferenceOnClick", "filterOnClick");
  eventEngine.registerEvent("VisualizationPreferenceOnClick", "visualizationOnClick");
  eventEngine.registerEvent("LayoutPreferenceOnClick", "layoutOnClick");
});

// FILE: src/PreferencesUser.js
function PreferencesUser() {

    var userId = localStorage.getItem('userId');

    this.registerUser = function(primaryKey){
        userId = primaryKey;
    };

    this.getPreferenceUserRegistered = function(){
        return userId;
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register("preferencesUserService", PreferencesUser);
});

// FILE: src/CustomizationActionInjector.js
(function() {

	function CustomizationActionInjector(templateManager, eventAggregator, ApplicationContext) {

		eventAggregator.subscribe("beforeProjectStartup", pushCustomizationAction);

		//------Dependencies
		var zhLayoutPreferences, zhFilterPreferences, zhCustomizationPreferences;

		function getDependencyByCustomizationType(type) {
			var dependency;
			if (type == "layout"){
				dependency = zhLayoutPreferences;
			} else if (type == "filter"){
				dependency = zhFilterPreferences;
			} else if (type =="view"){
				dependency = zhCustomizationPreferences;
			}
			return dependency;
		}

		// function getDefaultVisibilityByTemplate(widget, type) {
		// 	var defaults = [
		// 		{ template: 'widget/list.html', 		options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/grid.html', 		options: { search: true, filter: true, layout: true, view: true },
		// 		{ template: 'widget/tree/grid.html', 	options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/kanban.html', 		options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/crosstab.html', 	options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/form.html', 		options: { search: false, filter: false, layout: false, view: false },
		// 		{ template: 'widget/dashboard.html', 	options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'container/dashboard.html', options: { search: true, filter: true, layout: false, view: false }
		// 	];
		// }

		function isCustomizationActionVisible(widget, type) {
			var dependency = getDependencyByCustomizationType(type);
			return dependency && (Util.isUndefined(widget.floatingControl) || 
								  Util.isUndefined(widget.floatingControl.customizationAction) ||
								  widget.floatingControl.customizationAction[type] !== false);
		}

		function pushCustomizationAction() {
			zhLayoutPreferences = ApplicationContext.zhLayoutPreferences;
			zhFilterPreferences = ApplicationContext.zhFilterPreferences;
			zhCustomizationPreferences = ApplicationContext.zhCustomizationPreferences;

			var customFloatingActions = templateManager.project.customFloatingActions;
			var actionExistence = customFloatingActions.filter(function (action) {
				return action.id == "#hashReportsAction#";
			}).shift();
			if (!actionExistence) {
				templateManager.project.customFloatingActions.push({
					"id": "#hashFilterAction#",
					"name": "FilterAction",
					"label": "Search",
					"type": "filter",
					"order": 20,
					"icon": "search",
					"template": "zh-customizations-service#floating-filter-card.html",
					"expandable": true,
					"isVisible": function(widget) { return isCustomizationActionVisible(widget, 'filter') || Util.isUndefined(widget.floatingControl) || widget.floatingControl.searchAction !== false; }
				});
				templateManager.project.customFloatingActions.push({
					"id": "#hashLayoutsAction#",
					"name": "LayoutAction",
					"label": "Layouts",
					"type": "layout",
					"order": 30,
					"icon": "layout",
					"template": "zh-customizations-service#floating-preferences-card.html",
					"expandable": true,
					"isVisible": function(widget) { return isCustomizationActionVisible(widget, 'layout'); }
				});
				templateManager.project.customFloatingActions.push({
					"id": "#hashReportsAction#",
					"name": "ReportAction",
					"label": "Views",
					"type": "view",
					"order": 40,
					"icon": "view",
					"template": "zh-customizations-service#floating-preferences-card.html",
					"expandable": true,
					"isVisible": function(widget) { return isCustomizationActionVisible(widget, 'view'); }
				});
			}
		}
	}

	Configuration(function(ContextRegister) {
		ContextRegister.register("CustomizationActionInjector", CustomizationActionInjector);
	});

})();

// FILE: src/CustomizationsFloatingCardController.js
function CustomizationsFloatingCardController($scope, $rootScope, ScreenService, templateManager, ApplicationContext, restEngine, ZHPromise, eventAggregator) {
	
	var boundCustomizationUpdateEvent = eventAggregator.subscribe("OnUpdateCustomizationsRepositories", fillFloatingCardRepositories);

	$scope.$on("destroy", function(){
		eventAggregator.unSubscribe("OnUpdateCustomizationsRepositories", boundCustomizationUpdateEvent);
	});
	
	//------Dependencies
	var zhLayoutPreferences = ApplicationContext.zhLayoutPreferences;
	var zhFilterPreferences = ApplicationContext.zhFilterPreferences;
	var zhCustomizationPreferences = ApplicationContext.zhCustomizationPreferences;
	var zhCustomizationsService = ApplicationContext.zhCustomizationsService;
	
	//------ Control viariables
	var editingWidget = $scope.$parent.widget;
	var customizationIndex = 0;
	var floatingControl = editingWidget.floatingControl || {};
	var customizationsWidgetOptions = floatingControl.customizationAction;
	$scope.settingsCardLabel = ScreenService.i18n("Configurations");

	function checkFloatingCardOverflow(e) {
		var actionElement = $(e.target).closest(".float-action");
		var floatingControl = actionElement.parents(".zh-floating-control");
		var widgetContainer = actionElement.closest(".container");
		var actionTop, actionBottom;
		var widgetTop, widgetBottom;

		if (!actionElement.hasClass("opened")) {
			actionElement.removeClass("fix-position-top");
			actionElement.removeClass("fix-position-bottom");

			actionTop = actionElement.offset().top;
			widgetTop = widgetContainer.offset().top;
			widgetBottom = widgetTop + widgetContainer.height();
			if (floatingControl.hasClass("pos-top")) {
				actionTop = actionTop - 200 + actionElement.height();
				if (actionTop < widgetTop) {
					actionElement.addClass("fix-position-top");
				}
			} else {
				actionBottom = actionTop + 200;
				if (actionBottom > widgetBottom) {
					actionElement.addClass("fix-position-bottom");
				}				
			}
		}
	}

	$scope.togglePreferencesCard = function(e){
		checkFloatingCardOverflow(e);
		$scope.toggleAction(e);
	};
	editingWidget.togglePreferencesCard = $scope.togglePreferencesCard;

	var preferencesConfigsClick = function(){
		zhLayoutPreferences.startConfig(editingWidget);
	};

	var filtersConfigsClick = function(){
		zhFilterPreferences.startFilterConfig(editingWidget);
	};

	var viewsConfigsClick = function(){
		zhCustomizationPreferences.startViewSettings(editingWidget);
	};

	var preferencesRestoreClick = function(){
		zhLayoutPreferences.restoreDefault(editingWidget);
	};

	var filtersRestoreClick = function(){
		zhFilterPreferences.restoreDefault(editingWidget);
	};

	var viewsRestoreClick = function(){
		zhCustomizationPreferences.restoreDefault(editingWidget);
	};

	function applyFilter(customization){
		zhCustomizationsService.applyFilter(customization, editingWidget);
		editingWidget.customizationsApplied = editingWidget.customizationsApplied || {};
		delete editingWidget.customizationsApplied.view;
	}

	function applyLayout(customization){
		zhCustomizationsService.applyLayout(customization, editingWidget);
		editingWidget.customizationsApplied = editingWidget.customizationsApplied || {};
		delete editingWidget.customizationsApplied.view;
	}

	function applyVisualization(customization){
		zhCustomizationsService.applyVisualization(customization, editingWidget);
	}

	$scope.isCustomizationApplied = function(customization){
		if(editingWidget.customizationsApplied){
			var appliedCustomization = editingWidget.customizationsApplied[customization.customizationType];
			if(appliedCustomization == customization._id ){
				return true;
			}
		}
	};

	$scope.customizationsTypes = [
		{
			"label": ScreenService.i18n("Views"),
			"type": "view",
			"configsClick": viewsConfigsClick,
			"restoreClick": viewsRestoreClick,
			"applyOnClick": applyVisualization,
			"dataSource": {
				"data": []
			}
		},
		{
			"label": ScreenService.i18n("Layouts"),
			"type": "layout",
			"configsClick": preferencesConfigsClick,
			"restoreClick": preferencesRestoreClick,
			"applyOnClick": applyLayout,
			"dataSource": {
				"data": []
			}
		},
		{
			"label": ScreenService.i18n("Filters"),
			"type": "filter",
			"configsClick": filtersConfigsClick,
			"restoreClick": filtersRestoreClick,
			"applyOnClick": applyFilter,
			"dataSource": {
				"data": []
			}
		}
	];

	$scope.setActiveCustomization = function(type) {
		$scope.activeCustomization = $scope.customizationsTypes.filter(function(customizationType) {
			return customizationType.type == type;
		}).shift();
	};

	function fillFloatingCardRepositories(){
		return ZHPromise.all($scope.customizationsTypes.filter(function(customizationType) {
			return Util.isUndefined(editingWidget.floatingControl) || Util.isUndefined(editingWidget.floatingControl.customizationAction) || editingWidget.floatingControl.customizationAction[customizationType.type] !== false;
		}).map(function(customizationType){
			return zhCustomizationsService.getWidgetCustomizations(editingWidget, customizationType.type).then(function(custo){
				insertDefaultOption(custo, customizationType.type);
				customizationType.dataSource.data = custo;
				ScreenService.updateTemplate();
			});
		}));
	}
	fillFloatingCardRepositories();

	function insertDefaultOption(customizations, customizationType){
		var defaultOption = {
			"_id": "default",
			"label": ScreenService.i18n("Original"),
			"customizationType": customizationType,
			"isDefaultOption": true
		};
		if(defaultOption.customizationType == "filter" && zhFilterPreferences){
			defaultOption.filters = zhFilterPreferences.getDefaultFilterProperties(editingWidget) || [];
		}
		customizations.push(defaultOption);
	}

	$scope.previousCustomizationType = function(){
		if(customizationIndex === 0){
			customizationIndex = $scope.customizationsTypes.length - 1;
		} else {
			customizationIndex--;
		}
		updateCustomizationType(customizationIndex);
	};

	$scope.forwardCustomizationType = function(){
		if(customizationIndex === $scope.customizationsTypes.length - 1){
			customizationIndex = 0;
		} else {
			customizationIndex++;
		}
		updateCustomizationType(customizationIndex);
	};
}

// FILE: src/CustomizationsRequestsBuilder.js
(function() {

	function CustomizationsRequestsBuilder(templateManager) {

		this.buildRequest = function(typeRequest, method, dbProperties, dataset) {
			var request = {
				"typeRequest": typeRequest,
				"baseUrl": getbaseUrl(dbProperties),
				"service": getServiceUrl(dbProperties, method),
				"params": {
					"requestType": "DataSet",
					"dataset": dataset
				}
			};
			return request;
		};

		var getServiceUrl = function(dbProperties, method) {
			return !dbProperties.urlFullPath ? '/' + method + '/' + getRoute(dbProperties) : '';
		};

		var getbaseUrl = function(dbProperties) {
			return dbProperties.url ? dbProperties.url : templateManager.project.zhStoragePath;
		};

		var getRoute = function(dbProperties) {
			return dbProperties.dbName + "/" + dbProperties.collectionName;
		};
	}

	Configuration(function(ContextRegister) {
		ContextRegister.register("CustomizationsRequestsBuilder", CustomizationsRequestsBuilder);
	});

})();

// FILE: src/CustomizationsService.js
(function() {

	function CustomizationService($rootScope, ScreenService, templateManager, RepositoryFactory, Query, ZhUtil, restEngine, ZHPromise, CustomizationsRequestsBuilder, ApplicationContext, containerMerger, eventAggregator, eventEngine, preferencesUserService, ZhLogger, ZhKeyboardService) {

		var self = this;
		var customizationsRepositories;
		var customizationsTypes = ["layout", "filter", "view"];
		var SUFFIX_ORIGINAL_VALUE = "original";
		var userPreference;

		function extendBehavior() {
			Util.extend(self, preferencesUserService);
		}
		extendBehavior();

		var validateRegister = function(register) {
			var valid = register && register.containerId && register.widgetId;
			if (!valid) {
				ZhLogger.warning('The container and the widget must have IDs to save customizations');
			}
			return valid;
		};

		var getCustomizationsRepositories = function() {
			return templateManager.projectPromise.then(function() {
				customizationsRepositories = templateManager.project.customizationsRepositories || [];
			});
		};

		var createCustomizationsRepositories = function() {
			customizationsRepositories.forEach(function(custoRepo) {
				var repositoryName = custoRepo.dbName + custoRepo.collectionName;
				ZhUtil.registerDataSource(repositoryName, "_id");
				repositoryName = "/" + repositoryName;
				custoRepo.repository = RepositoryFactory.factory(repositoryName, "LOCAL");
			});
		};

		self.fillCustomizationsRepositories = function() {
			var promises = customizationsRepositories.map(function(custoRepo) {
				var request = CustomizationsRequestsBuilder.buildRequest("GET", "findAll", custoRepo, []);
				return restEngine.request(request).then(function(responseData) {
					return custoRepo.repository.save(responseData.dataset.foundData);
				});
			});
			return ZHPromise.all(promises);
		};

		self.getRepository = function(custoId) {
			var customizationRepository = getCustomizationRepositoryById(custoId);
			if (!Util.isEmptyOrBlank(customizationRepository) && customizationRepository.repository) {
				return customizationRepository.repository;
			}
		};

		self.removeCustomizations = function(customizationsToDelete) {
			var customizationsMappedById = groupIdsByCustomizationRepository(customizationsToDelete);
			var promises = customizationsMappedById.map(function(customization) {
				var customizationRepository = getCustomizationRepositoryById(customization.custoRepo);
				var query = Query.build()
					.where("_id").in(customization.ids);
				var request = CustomizationsRequestsBuilder.buildRequest("DELETE", "remove", customizationRepository, createCustomizationsIdArrayIndexedById(customization.ids));
				return restEngine.request(request).then(function() {
					return customizationRepository.repository.remove(query);
				});
			});
			return ZHPromise.all(promises)
				.then(function(promisesResult) {
					eventAggregator.publish("OnUpdateCustomizationsRepositories");
					return promisesResult;
				});
		};

		self.saveCustomization = function(rows, customizationType) {
			rows = Util.parseToArray(rows);
			var promises = rows.map(function(row) {
				if (validateRegister(row) && customizationType && ~customizationsTypes.indexOf(customizationType)) {
					row.customizationType = row.customizationType || customizationType;
					var customizationRepository = getCustomizationRepositoryById(row.customizationRepository);
					var request = CustomizationsRequestsBuilder.buildRequest("POST", "save", customizationRepository, [row]);

					return restEngine.request(request).then(function(responseData) {
						var savedPreferences = responseData.dataset.response.saved;

						if (savedPreferences.length === 1) {
							row._id = savedPreferences[0]._id;
							delete row.unsync_id;
						}

						return customizationRepository.repository.save([row]);
					});
				} else {
					return ZHPromise.when({});
				}
			});

			return ZHPromise.all(promises)
				.then(function(promisesResult) {
					eventAggregator.publish("OnUpdateCustomizationsRepositories");
					return promisesResult;
				});
		};

		self.getWidgetCustomizations = function(widget, customizationType) {
			var promises = customizationsRepositories.map(function(repository) {
				var query = Query.build()
					.where("containerId").equals(widget.container.id)
					.where("widgetId").equals(widget.id)
					.where("customizationType").equals(customizationType);
				var userId = preferencesUserService.getPreferenceUserRegistered();
				if (userId && repository.customizationsByUser) {
					query.where("userId").equals(userId);
				}
				return repository.repository.find(query);
			});
			return ZHPromise.all(promises).then(function(reponses) {
				return reponses.reduce(function(accumulatedCustomizations, response) {
					return accumulatedCustomizations.concat(response);
				}, []);
			});
		};

		self.transferCustomizationsRepository = function(widget, rows, destinyRepository) {
			ScreenService.showLoader();
			return self.removeCustomizations(rows).then(function() {
				var syncIds = {};
				promises = rows.map(function(row) {
					return transferViewCustomizations(row, destinyRepository, widget).then(function() {
						row.customizationRepository = destinyRepository;
						row.unsync_id = new Date().getTime();
						removeNotInterestingProperties(row);
						delete row._id;
						return self.saveCustomization(row, row.customizationType).then(function(syncId) {
							syncIds[row.customizationType] = syncId;
						});
					});
				});
				return ZHPromise.all(promises).then(function() {
					ScreenService.hideLoader();
					return syncIds;
				});
			});
		};

		var transferViewCustomizations = function(view, destinyRepository, widget) {
			if (view.customizationType == "view") {
				var viewCustomizationsMapped = [{
					"customizationType": "filter",
					"id": view.filterId
				}, {
					"customizationType": "layout",
					"id": view.preferenceId
				}];
				var viewCustomizations = [];
				var customizationRepository = view.customizationRepository;
				var promises = viewCustomizationsMapped.map(function(customization) {
					return self.getWidgetCustomizationsByRepository(widget, customization.customizationType, customizationRepository, customization.id)
						.then(function(customizationObject) {
							viewCustomizations.push(customizationObject.shift());
						});
				});
				return ZHPromise.all(promises).then(function() {
					return self.transferCustomizationsRepository(widget, viewCustomizations, destinyRepository).then(function(syncIdsMapped) {
						view.filterId = syncIdsMapped.filter;
						view.preferenceId = syncIdsMapped.layout;
					});
				});
			} else {
				return ZHPromise.when();
			}
		};

		self.getCustomizationView = function(widget, customizationRepository, customizationType, id) {
			var idPropertyName = customizationType == "filter" ? "filterId" : "preferenceId";
			return self.getWidgetCustomizationsByRepository(widget, "view", customizationRepository)
				.then(function(viewObjects) {
					var viewObject;
					if (Util.isArray(viewObjects) && viewObjects.length > 0) {
						viewObject = viewObjects.filter(function(view) {
							return view[idPropertyName] == id;
						});
					}
					return viewObject;
				});
		};

		var removeNotInterestingProperties = function(object) {
			Object.keys(object).forEach(function(key) {
				if (~key.indexOf('__')) {
					delete object[key];
				}
			});
		};

		self.getWidgetCustomizationsByRepository = function(widget, customizationType, repositoryType, customizationId) {
			var customizationRepository = getCustomizationRepositoryById(repositoryType);
			var query = Query.build()
				.where("containerId").equals(widget.container.id)
				.where("widgetId").equals(widget.id)
				.where("customizationType").equals(customizationType);
			var userId = preferencesUserService.getPreferenceUserRegistered();
			if (userId && customizationRepository.customizationsByUser) {
				query.where("userId").equals(userId);
			}
			if (customizationId) {
				query.where("_id").equals(customizationId);
			}
			return customizationRepository.repository.find(query);
		};

		self.getCustomizationsRepositoriesTypes = function() {
			return customizationsRepositories;
		};

		self.getUpdatedCustomizationsRepositoriesTypes = function() {
			return self.fillCustomizationsRepositories().then(function() {
				return customizationsRepositories;
			});
		};

		self.getLabelRepositoryType = function(repositoryTypeValue) {
			return customizationsRepositories.filter(function(repository) {
				return repository.value == repositoryTypeValue;
			}).shift().label;
		};

		self.applyLayout = function(customization, editingWidget) {
			var eventData = createEventData(customization, editingWidget);
			if (self.layoutOnClick) {
				self.layoutOnClick(eventData);
			}
		};

		self.applyFilter = function(customization, editingWidget) {
			var eventData = createEventData(customization, editingWidget);
			if (self.filterOnClick) {
				self.filterOnClick(eventData);
			}
		};

		self.applyVisualization = function(customization, editingWidget) {
			var eventData = createEventData(customization, editingWidget);
			if (self.visualizationOnClick) {
				self.visualizationOnClick(eventData);
			}
		};

		function createEventData(customization, editingWidget) {
			return {
				"customization": customization,
				"editingWidget": editingWidget
			};
		}

		var groupIdsByCustomizationRepository = function(customizations) {
			var customizationsMappedById = [];
			customizationsRepositories.forEach(function(custoRepo) {
				var sameCustoRepoIds = customizations.filter(function(customization) {
					return customization.customizationRepository == custoRepo.value;
				});
				var map = {
					"custoRepo": custoRepo.value,
					"ids": sameCustoRepoIds.map(function(customization) {
						return customization._id;
					})
				};
				if (Util.isArray(map.ids) && map.ids.length > 0) {
					customizationsMappedById.push(map);
				}
			});
			return customizationsMappedById;
		};

		function createCustomizationsIdArrayIndexedById(customizationsId) {
			var customizationsToDelete = [];
			customizationsId.forEach(function(id) {
				customizationsToDelete.push({
					"_id": id
				});
			});
			return customizationsToDelete;
		}

		var getCustomizationRepositoryById = function(custoId) {
			return customizationsRepositories.filter(function(custoRepo) {
				return custoRepo.value == custoId;
			}).shift();
		};

		//-------------Begin Execution
		getCustomizationsRepositories().then(function() {
			createCustomizationsRepositories();
			self.fillCustomizationsRepositories().then(function() {
				eventAggregator.publish("OnUpdateCustomizationsRepositories");
			});
		});
	}

	Configuration(function(ContextRegister) {
		ContextRegister.register("zhCustomizationsService", CustomizationService);
	});

})();

// FILE: ./bower_components/zh-filter-preferences/dist/zhFilterPreferences.js

// FILE: src/registers/registerEvents.js
Configuration(function(eventEngine) {
  eventEngine.registerEvent("GridFilterBeforeApply", "beforeGridFilterApply");
  eventEngine.registerEvent("GridFilterAfterApply", "afterGridFilterApply");
  eventEngine.registerEvent("GridFilterAfterCancel", "afterGridFilterCancel");
});

// FILE: src/FilterBuilder.js
(function () {

function FilterBuilder(Query, ZHPromise) {
	var _filterType = {};
	var skipFiltering = [];
	var self = this;

	var _buildFilterTypeMap = function (fields, currentRow) {
		fields.forEach(function (field) {
			if (!field.filterProperties) {
				setOperatorFromFieldTemplate(field, currentRow);
			} else {
				var filterProperties = field.filterProperties;
				if (filterProperties.operator) {
					setOperatorFromFieldOperator(_filterType, field);
				} else {
					setOperatorFromFieldTemplate(field, currentRow);
				}
				if (filterProperties.outFilters) {
					Object.keys(filterProperties.outFilters).forEach(function (key) {
						var operator = filterProperties.outFilters[key];
						_filterType[key] = operator ? operator : _filterType[field.name];
					});
				}
			}
		});
	};

	function setOperatorFromFieldOperator(_filterType, field) {
		if (fieldHasSelectTemplate(field)) {
			_filterType[field.outData[field.valueField]] = field.filterProperties.operator;
		} else {
			_filterType[field.name] = field.filterProperties.operator;
		}
	}

	function fieldHasSelectTemplate(field) {
		var template = (field.filterProperties && field.filterProperties.template) || field.template;
		return field.outData && template && ~template.indexOf('select.html');
	}

	var setOperatorFromFieldTemplate = function (field, currentRow) {
		var template = (field.filterProperties && field.filterProperties.template) || field.template;

		if (template && (template.indexOf('between-dates.html') !== -1 || template.indexOf('interval.html') !== -1)) {
			_filterType[field.name] = "BETWEEN";
			var fieldValue = currentRow[field.name];
			if (fieldValue) {
				if (!Object.keys(fieldValue).some(function (element) {
					return fieldValue[element];
				})) {
					if (!~skipFiltering.indexOf(field.name)) {
						skipFiltering.push(field.name);
					}
				}
			}
		} else if (template && (~template.indexOf('number.html') || ~template.indexOf('datetime.html') || ~template.indexOf(
			'calendar.html') || ~template.indexOf('select-native.html'))) {
			_filterType[field.name] = "=";
		} else if (fieldHasSelectTemplate(field)) {
			_filterType[field.outData[field.valueField]] = "=";
		} else if (template && ~template.indexOf('select-multiple.html')) {
			_filterType[field.name] = "IN";
		} else {
			_filterType[field.name] = "LIKE_I";
		}
	};

	var _filterTypeFlow = {
		"BETWEEN": function (query, value) {
			query.between([value.start, value.end]);
		},
		"NOT BETWEEN": function (query, value) {
			query.notBetween([value.start, value.end]);
		},
		"LIKE": function (query, value) {
			query.like('%' + value + '%');
		},
		"LIKE_I": function (query, value) {
			query.likeInsensitive('%' + value + '%');
		},
		"NOT LIKE": function (query, value) {
			query.notLike('%' + value + '%');
		},
		"LIKE ALL": function (query, value) {
			query.likeAll('%' + value + '%');
		},
		"<>": function (query, value) {
			query.notEquals(value);
		},
		"=": function (query, value) {
			query.equals(value);
		},
		">": function (query, value) {
			query.greater(value);
		},
		"<": function (query, value) {
			query.lower(value);
		},
		">=": function (query, value) {
			query.greaterOrEqual(value);
		},
		"<=": function (query, value) {
			query.lowerOrEqual(value);
		},
		"IN": function (query, value) {
			query.in(value);
		},
		"NOT IN": function (query, value) {
			query.notIn(value);
		}
	};

	this.buildFilter = function (fields, currentRow) {
		_filterType = {};
		skipFiltering = [];
		_buildFilterTypeMap(fields, currentRow);
		var query = Query.build();
		for (var key in currentRow) {
			if (typeof currentRow[key] == "boolean") {
				currentRow[key] = currentRow[key].toString();
			}
			if (currentRow[key] && _filterType[key] && (!Array.isArray(currentRow[key]) || currentRow[key].length > 0)) {
				if (!~skipFiltering.indexOf(key)) {
					query.where(key);
					_filterTypeFlow[_filterType[key]](query, currentRow[key]);
				}
			}
		}
		return query.where();
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register("ZhFilterBuilder", FilterBuilder);
});

})();

// FILE: src/FilterWidgetsCreator.js
function FilterWidgetsCreator(metaDataFactory, ScreenService) {

	this.createFiltersWidget = function (gridWidget, functionsToInject){
		var widget = metaDataFactory.widgetFactory({
			"id": 1,
			"showAsModal": true,
			"container": gridWidget.container,
			"name": "popUpFiltersList",
			"label": ScreenService.i18n("Filters"),
			"isVisible": true,
			"template": "widget/grid.html",
			"showCheckbox": true,
			"actions": actionsFiltersListWidget,
			"styles": [],
			"floatingControl": {
				"refreshAction": false,
				"customizationAction": {
					"layout": false,
					"filter": false,
					"view": false
				},
				"actions": []
			},
			"events": [
				{
					"id": 2,
					"name": "WidgetOnEnter",
					"code": "args.owner.fillFilterList(args.owner);"
				},
				{
					"id": 3,
					"name": "WidgetAfterMoveRow",
					"code": "args.owner.openEditFilter(args.owner, args.row);"
				}
			],
			"dataSource": {
				"localStorage": false,
				"rest": false,
				"data": []
			},
			"fields": [
				{
					"id": 1,
					"name": "label",
					"label": ScreenService.i18n("Filter name"),
					"showOnList": true,
					"template": "field/text-edit.html"
				},
				{
					"id": 2,
					"name": "filterTypeLabel",
					"label": ScreenService.i18n("Type"),
					"showOnList": true,
					"template": "field/text-edit.html"
				},
				{
					"id": 4,
					"name": "defaultPreference",
					"isVisible": false,
					"showOnList": true,
					"label": ScreenService.i18n("Default"),
					"template": "field/checkbox.html",
					"align" : "center",
					"valueToTrue": true,
					"valueToFalse": false,
					"inlineEditable" : "checkbox",
					"hideCheckAllButton": true,
					"events": [{
						"id": 1,
						"name": "FieldOnChange",
						"code": "args.owner.field.widget.setAsDefaultPreference(args.row, args.owner.field.widget);"
					}]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	};

	var actionsFiltersListWidget = [
		{
			"id": 2,
			"name": "NEW",
			"isVisible": true,
			"showAsAction": "always",
			"icon": "plus icon-white",
			"tooltip": ScreenService.i18n("New"),
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.openCreateNewFilterWidget(args.owner.widget)"
			}]
		},
		{
			"id": 3,
			"name": "cancelAction",
			"label": ScreenService.i18n("Close"),
			"showAsAction": "back",
			"hideIcon": true,
			"isVisible": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.closeFilterPopup()"
			}]
		},
		{
			"id": 4,
			"name": "deletePreference",
			"isVisible": true,
			"icon": "trash",
			"showAsAction": "checked_rows",
			"condition": "row.acl_edit",
			"tooltip": ScreenService.i18n("Delete preference"),
			"events": [
				{
					"id": 1,
					"name": "ActionEvent",
					"code": "args.owner.widget.deletePreferenceClick(args.owner.widget);"
				}
			]
		}
	];

	this.createNewFilterWidget = function(gridWidget, functionsToInject) {
		var widget = metaDataFactory.widgetFactory({
			"id": "335689",
			"isVisible": true,
			"name": "saveFilterPopup",
			"showAsModal": true,
			"container": gridWidget.container,
			"label": ScreenService.i18n("Create a new filter"),
			"template": "widget/form.html",
			"currentRow": {},
			"descriptionOrder": false,
			"fields": [
				{
					"id": 1,
					"label": ScreenService.i18n("Name"),
					"name": "name",
					"showOnList": true,
					"class": "6",
					"showOnForm": true,
					"events": [],
					"template": "field/text-edit.html",
					"validations": {
						"required": {
							"message": ScreenService.i18n("Required Field")
						}
					}
				},
				{
					"name": "type",
					"label": ScreenService.i18n("Type"),
					"class": "6",
					"isVisible": true,
					"readOnly": false,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"template": "field/select-native.html",
					"valueField": "value",
					"descriptionField": "label",
					"events": [],
					"hideEmptyValue": false,
					"dataSource": {
						"data": []
					},
					"validations": {
						"required": {
							"message": ScreenService.i18n("Required Field")
						}
					}
				}
			],
			"actions": [{
					"id": 2,
					"name": "cancelAction",
					"label": ScreenService.i18n("Cancel"),
					"showAsAction": "back",
					"hideIcon": true,
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
				}]
				}, {
					"id": 2,
					"name": "saveFilterAction",
					"label": ScreenService.i18n("Continue"),
					"showAsAction": "never",
					"hideIcon": true,
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "args.owner.widget.createNewFilter(args.owner.widget)"
					}]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	};

	this.createChangeFilterTypeWidget = function(gridWidget, functionsToInject){
		var widget = metaDataFactory.widgetFactory({
			"id": 8998,
			"name": "changePreferenceTypeWidget",
			"showAsModal": true,
			"container": gridWidget.container,
			"label": ScreenService.i18n("Change filter type"),
			"template": "widget/form.html",
			"showDescriptionOrder": false,
			"isVisible": true,
			"fields": [
				{
					"name": "filterType",
					"label": ScreenService.i18n("New filter type"),
					"class": "12",
					"isVisible": true,
					"readOnly": false,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"template": "field/select-native.html",
					"valueField": "value",
					"descriptionField": "label",
					"events": [],
					"hideEmptyValue": false,
					"dataSource": {
						"data":[]
					},
					"validations": {
						"required":{
							"message": ScreenService.i18n("Required Field")
						}
					}
				}
			],
			"events": [],
			"actions":[
				{
					"id": 1,
					"name": "canceladd",
					"label": ScreenService.i18n("Cancel"),
					"isVisible": true,
					"hideIcon": true,
					"showAsAction": "back",
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
					}]
				},
				{
					"id": 1,
					"name": "tranferFilter",
					"label": ScreenService.i18n("Save"),
					"isVisible": true,
					"showAsAction": "never",
					"events": [
						{
							"id": 1,
							"name": "ActionEvent",
							"code": "args.owner.widget.saveTransferFilterClick(args.owner.widget)"
						}
					]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	};

	var actionsPropertiesWidget = [
		{
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.backToPreviousWidget(args.owner.widget)"
			}]
		}, {
			"label": ScreenService.i18n("OK"),
			"showAsAction": "never",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.saveFieldChanges(args.owner.widget, args.row)"
			}]
		}
	];

	this.createFormFilterWidget = function(gridWidget, functionsToInject) {
		var widget = metaDataFactory.widgetFactory({
			"id": "31231231231",
			"isVisible": true,
			"name": "filterPopup",
			"orderFieldsBySequenceInForm": true,
			"container": gridWidget.container,
			"label": ScreenService.i18n("Filter"),
			"template": "widget/form.html",
			"fieldGroups": (gridWidget.fieldGroups  || []).concat([{
				"id": 1,
				"name": "COLUMNS_FILTER",
				"opened": false,
				"collapsible": true,
				"label": ScreenService.i18n("More fields")
			}]),
			"fields": [],
			"actions": [
				{
					"id": 1,
					"name": "cancelAction",
					"label": ScreenService.i18n("Cancel"),
					"showAsAction": "back",
					"hideIcon": false,
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
					}]
				},
				{
					"id": 4,
					"name": "saveAndApplyFilterAction",
					"showAsAction": "never",
					"label": ScreenService.i18n("Save & Apply"),
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "args.owner.widget.saveAndApplyFilter(args.owner.widget)"
					}]
				}
			]
		});

		return Util.extend(widget, functionsToInject);
	};

	this.createCurrentFormFilterWidget = function(gridWidget, functionsToInject) {
		var widget = metaDataFactory.widgetFactory({
			"id": "8514214420",
			"orderFieldsBySequenceInForm": true,
			"isVisible": true,
			"name": "filterPopup",
			"container": gridWidget.container,
			"label": ScreenService.i18n("Filter"),
			"template": "widget/form.html",
			"fieldGroups": (gridWidget.fieldGroups  || []).concat([{
				"id": 1,
				"name": "COLUMNS_FILTER",
				"opened": false,
				"collapsible": true,
				"label": ScreenService.i18n("More fields")
			}]),
			"events": Util.getNestedProperty(gridWidget, ['filterProperties', 'events']),
			"fields": [],
			"actions": [
				{
					"id": 1,
					"name": "cancelAction",
					"label": ScreenService.i18n("Back"),
					"showAsAction": "back",
					"hideIcon": false,
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
					}]
				},
				{
					"id": 4,
					"name": "saveAndApplyFilterAction",
					"showAsAction": "never",
					"label": ScreenService.i18n("Filter"),
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "args.owner.widget.currentFilterClickPopup(args.owner.widget)"
					}]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	};

}

Configuration(function(ContextRegister) {
	ContextRegister.register("filterWidgetsCreator", FilterWidgetsCreator);
});

// FILE: src/FilterFloatingCardController.js
var FilterFloatingCardController = (function(_super) {
	__extends(FilterFloatingCardController1, _super);

	function FilterFloatingCardController1($scope, $rootScope, ScreenService, templateManager, ApplicationContext, restEngine, ZHPromise, eventAggregator, $timeout, ZhKeyboardService) {
		var _ref = FilterFloatingCardController.__super__.constructor.apply(this, arguments);

		var zhFilterPreferences = ApplicationContext.zhFilterPreferences;

		function getFilterFromSearchCriteria(filters, searchCriteria) {
			// remove itens starting with "SEARCHLIST_"
			filters = (filters || []).filter(function(filter) {
				return !~filter.name.indexOf("SEARCHLIST_");
			});
			if (searchCriteria) {			
				Object.keys(searchCriteria).forEach(function(key) {
					if (searchCriteria[key]) {				
						filters.push({
							name: "SEARCHLIST_"+key,
							operator: "LIKE",
							value: '%'+searchCriteria[key]+'%'
						});
					}
				});
			}
			return filters;
		}

		$scope.$watchCollection(function() { return $scope.widget.searchCriteria; }, function(current, old) {
			var newFilterApplied = getFilterFromSearchCriteria($scope.widget.filterApplied, current);
			if (!angular.equals($scope.widget.filterApplied, newFilterApplied)) {
				if ($scope.widget.customizationsApplied) {
					delete $scope.widget.customizationsApplied.filter;
				}

				$scope.widget.filterApplied = newFilterApplied;
				zhFilterPreferences.updateFilterInfo($scope.widget, newFilterApplied);
			}
		});

		$scope.clearSearch = function(event, fieldName) {
			if (!fieldName) {
				$scope.widget.searchCriteria = {};
			} else {
				delete $scope.widget.searchCriteria[fieldName];				
			}

			if (event) {
				Util.selectElement($(event.target).closest(".floating-card-input").find("input"));
			}
		};

		$scope.startFiltering = function() {
			zhFilterPreferences.startFiltering($scope.widget);
		};
		$scope.$on('control-info-icon-click', function(event, data) {
			if (data == 'filter') {
				$scope.startFiltering();
			} else {
				$scope.clearSearch();
			}
		});

		$scope.isSearchFieldSelect = function(field) {
			return field && ((field.filterProperties && field.filterProperties.template && ~field.filterProperties.template.indexOf('select-native.html')) || 
							 (field.template && ~field.template.indexOf('select-native.html')));
		};

		$scope.prepareField = function(field) {
			return zhFilterPreferences.prepareFields(Util.parseToArray(field))[0];
		};

		function isMobiScrollOpen() {
			return $('.mbsc-mobiscroll').length > 0;
		}

		$scope.openWidgetFilter = function(event, key) {
			if (!ScreenService.isSideMenuOpen() && !isMobiScrollOpen() && !ScreenService.isAlertShown()) {
				var widget = angular.element('.zh-floating-control:last').scope().widget;
				if (widget.floatingControl && widget.floatingControl.customizationAction && widget.floatingControl.customizationAction.filter) {
					ApplicationContext.zhFilterPreferences.startFiltering(widget);
				}
			}

			// prevent default behaviour
			return false;
		};

		$scope.openWidgetSearch = function(event, key) {
			if (!ScreenService.isSideMenuOpen() && !isMobiScrollOpen() && !ScreenService.isAlertShown()) {
				var scope = angular.element('.zh-floating-control:last').scope();
				scope.openSearchAction();
			}

			// prevent default behaviour
			return false;
		};

		$scope.initKeyboardShortCuts = function() {
			ZhKeyboardService.unbind(templateManager.project.shortcutKeys.openWidgetFilter || 'mod+l');
			ZhKeyboardService.unbind(templateManager.project.shortcutKeys.openWidgetSearch || 'mod+f');
			
			ZhKeyboardService.bind(templateManager.project.shortcutKeys.openWidgetFilter || 'mod+l', $scope.openWidgetFilter);
			ZhKeyboardService.bind(templateManager.project.shortcutKeys.openWidgetSearch || 'mod+f', $scope.openWidgetSearch);
		};
		$scope.initKeyboardShortCuts();

		return _ref;
	}

	return FilterFloatingCardController1;
})(CustomizationsFloatingCardController);

// FILE: src/FilterFloatingControlDirective.js
(function () {
	var ZeedhiDirectives = angular.module("ZeedhiDirectives");

	ZeedhiDirectives.directive(
		'zhFloatingControl',
		["templateManager", "ScreenService", "$timeout", "eventEngine", "ApplicationContext", "$window", "$rootScope", "ZhUtil", "$compile", "ZhKeyboardService", FilterFloatingControlDirective]
	);

	function FilterFloatingControlDirective(templateManager, ScreenService, $timeout, eventEngine, ApplicationContext, $window, $rootScope, zhUtil, $compile, ZhKeyboardService) {
		return {
			priority: 10,
			link: function($scope, element, attrs, controller) {
				var widget = $scope.widget;
				if (widget.floatingControl === false) return;

				var defaultOptions = {
					searchAction: Util.isUndefined(element.parent().attr('data-hide-search')) && Util.isUndefined(element.parent().attr('hide-search')),
					openSearch: false
				};
				widget.floatingControl = Util.extend(defaultOptions, widget.floatingControl);

				widget.floatingControl.addStatus({
					priority: 20,
					classString: 'searching',
					icon: 'search',
					conditionFunction: function() {
						return (widget.filterApplied && widget.filterApplied.length > 0) || Util.objectHasAnyValue($scope.searchCriteria);
					},
					infoTemplate: function() {
						var template;
						if (widget.floatingControl.customizationAction && widget.floatingControl.customizationAction.filter) {
							template = '<div class="searching-control-info"><span data-zh-icon="filter no-border icon-white" data-zh-touchstart="ControlInfoIconClick(\'filter\')"></span><div class="searching-control-info-details" data-ng-if="widget._filterInfo"><span data-ng-bind-html="widget._filterInfo"></span></div></div><div class="searching-control-info-text" data-ng-if="widget._filterInfo" data-zh-bind-translate="\'Filter applied\'"></div>';
						} else {
							template = '<div class="searching-control-info"><span data-zh-icon="close-x no-border icon-white" data-zh-touchstart="ControlInfoIconClick(\'clear\')"></span><div class="searching-control-info-details" data-ng-if="widget._filterInfo"><span data-ng-bind-html="widget._filterInfo"></span></div></div><div class="searching-control-info-text" data-ng-if="widget._filterInfo" data-zh-bind-translate="\'Filter applied\'"></div>';
						}
						return template;
					},
					scope: $scope
				});

				var isTheOnlyActionAvailable = function(propertyName){
					var actionsToCheck= ["configAction", "refreshAction", "xlsAction", "pdfAction", "actions", "searchAction"];
					var filtered = actionsToCheck.filter(function(property){
						var action = widget.floatingControl[property];
						return Util.isArray(action) ? action.length : action;
					});
					return (filtered.length == 1 && filtered[0] == propertyName);
				};

				$scope.$on('floating-control-before-close', function() {
					element.find('.search-action input').blur();
				});

				$scope.$on('floating-control-after-close', function() {
					if (element.hasClass('searching') || isTheOnlyActionAvailable("searchAction")) {
						element.find('.search-action').addClass('opened');
						Util.selectElement(element.find('.search-action input'));
					}
				});

				$scope.openSearchAction = function() {
					$timeout.cancel($scope._closeTimeout);
					if (!element.hasClass('opened')) {
						$scope.toggleActionMenu();
					}
					$scope.closeOpenedActions();
					var searchAction = element.find('.search-action').closest('.float-action');
					if (searchAction && searchAction.length > 0) {
						searchAction.find("span.search-action-icon").click();
						Util.selectElement(searchAction.find('input'));
					}
				};

				if (widget.floatingControl.openSearch) {
					$timeout(function() {
						var floatingControl = (ScreenService.isPopupOpen() ? $('.popup-container .container:visible') : $('.container:visible')).last().parent().find('.zh-floating-control');
						$scope.openSearchAction(floatingControl);
					}, 100);
				}

				$(window).off('click.floatingControl');
				$(window).on('click.floatingControl', function(e) {
					if (!element.find('.search-action').closest('.float-action').hasClass('opened')) {			
						if ($(e.target).parents('.zh-floating-control').length === 0) {
							widget.closeActionMenu();
						}
					}
				});

				$scope.filterFields = function(field) {
					return field.aclIsVisible && !field.skipSearch;
				};

				$scope.toggleSearchFieldsList = function(event) {
					// if filter action exists user cannot choose search field... it will be done in filter popup
					if (!widget.floatingControl.customizationAction || !widget.floatingControl.customizationAction.filter) {
						var el = $(event.target).closest(".floating-card").find('.floating-card-search-field-select');
						el.toggleClass("opened");

						if (el.hasClass("opened")) {
							$(event.target).closest(".floating-card").find('.floating-card-select-field-options').removeClass("opened");
							$(event.target).closest(".floating-card").find('.zh-select-search-floating').removeClass("opened");
						}
					}
				};

				$scope.toggleSelectFieldValueList = function(event) {
					var el = $(event.target).closest(".zh-select-search-floating");
					el.toggleClass("opened");
					var list = $(event.target).closest(".floating-card").find('.floating-card-select-field-options');
					list.toggleClass("opened");

					if (list.hasClass("opened")) {
						$(event.target).closest(".floating-card").find('.floating-card-search-field-select').removeClass("opened");
					}
				};

				$scope.selectSearchField = function(field) {
					$scope.searchField = field;
					element.find('.floating-card-search-field-select').removeClass('opened');
					element.find('input.zh-input-search-floating').focus();
				};

				$scope.selectSearchFieldValue = function(widget, event) {
					var dataValue = $(event.target).attr('data-value');
					var field = $scope.searchField;
					if (dataValue) {
						widget.searchCriteria = widget.searchCriteria || {};
						widget.searchCriteria[field.name] = field.dataSource.data[parseInt(dataValue)][field.valueField].toString();
					} else {
						delete widget.searchCriteria[field.name];
					}

					$(event.target).closest('.floating-card').find('.floating-card-select-field-options').removeClass('opened');
					$(event.target).closest('.floating-card').find('.zh-select-search-floating').removeClass('opened');
				};

				$scope.getSearchFieldValue = function(widget) {
					var field = $scope.searchField;
					var fieldValue;
					var row = field.dataSource.data.filter(function(row) {
						return widget.searchCriteria && widget.searchCriteria[field.name] == row[field.valueField].toString();
					});
					if (row.length > 0) {
						fieldValue = row[0][field.descriptionField];
					}
					return fieldValue;
				};

				if (widget.floatingControl.defaultSearchField) {
					$scope.searchField = widget.getField(widget.floatingControl.defaultSearchField);
				}
			}
		};
	}
})();


// FILE: src/ZhFilterPreferences.js
function ZhFilterPreferences(ScreenService, templateManager, ZhFilterBuilder, ZhUtil, RepositoryFactory, Query, eventEngine, ZHPromise, ZhLogger, ApplicationContext, zhCustomizationsService, filterWidgetsCreator, eventAggregator, preferencesUserService, DecodeValueFieldService) {

	var self = this;
	var gridWidget, filterDescription = [];

	eventAggregator.subscribe("OnListStartUp", getAndApplyDefaultPreference);

	function getAndApplyDefaultPreference(eventData) {
		if (eventData.widget) {
			var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
			self.getDefaultPreferenceByUser(eventData.widget, userRegistered).then(function(defaultPreference) {
				if (defaultPreference) {
					self.applyFilter(defaultPreference, eventData.widget);
				} else {
					eventData.widget.customizationsApplied = eventData.widget.customizationsApplied || {};
					eventData.widget.customizationsApplied.filter = "default";
				}
			});
		}
	}

	this.getDefaultPreferenceByUser = function(widget, userId) {
		return zhCustomizationsService.getWidgetCustomizations(widget, "filter").then(function(preferences) {
			return preferences.filter(function(preference) {
				return preference.mapUserIdDefault && preference.mapUserIdDefault[userId];
			}).shift();
		});
	};

	this.startFilterConfig = function startFiltering(widget) {
		gridWidget = widget;
		gridWidget.__filterConfig = filterWidgetsCreator.createFiltersWidget(gridWidget, functionsFilterListWidget);
		fillFilterList(gridWidget.__filterConfig).then(function() {
			ScreenService.openPopup(gridWidget.__filterConfig);
		});
	};

	this.restoreDefault = function(widget) {
		this.restoreDefaultFilter(widget);
	};

	function getFilterWidget(widget) {
		if (widget.__filterPopup === undefined) {
			widget.__filterPopup = filterWidgetsCreator.createCurrentFormFilterWidget(widget, functionsFormCurrentFilterWidget);
			widget.__filterPopup.showAsModal = false;
			widget.__filterPopup.label = ScreenService.i18n("Filter");
			widget.__filterPopup.fields = prepareFieldstoFormFilter(factoryFields(widget.__filterPopup));
			var searchFields = widget.hideSearchFieldsOnFilter ? [] : prepareFieldstoFormFilter(factorySearchFields(widget.__filterPopup));
			widget.__filterPopup.fields = widget.__filterPopup.fields.concat(searchFields);
			widget.__filterPopup.applyDefaultValue();
			var filters = widget.filterApplied;
			if (filters && filters.length || widget._filterRow) {
				widget.__filterPopup.currentRow = buildCurrentRowByFilters(filters, widget.__filterPopup, widget._filterRow);
			}
		}

		return widget.__filterPopup;
	}

	this.startFiltering = function(widget) {
		gridWidget = widget;
		ScreenService.openPopup(getFilterWidget(gridWidget));
	};

	var filterVisibleFields = function(fields) {
		return fields.filter(function(field) {
			return field.isVisible;
		});
	};

	function currentFilterClickPopup(widget) {
		if (widget.isValid()) {
			var currentRow = widget.currentRow;
			var fields = filterVisibleFields(gridWidget.__filterPopup.fields);
			var filter = {
				_filterRow: ZhUtil.clearRow(currentRow),
				filters: ZhFilterBuilder.buildFilter(fields, currentRow),
				_id: "edited"
			};
			self.applyFilter(filter, gridWidget);
			ScreenService.closePopup();
		}
	}

	function fillFilterList() {
		return zhCustomizationsService.getWidgetCustomizations(gridWidget, "filter").then(function(filters) {
			filters.forEach(function(filter) {
				filter.filterTypeLabel = zhCustomizationsService.getLabelRepositoryType(filter.customizationRepository);
			});
			gridWidget.__filterConfig.dataSource.data = filters;
			ScreenService.updateTemplate();
		});
	}

	this.restoreDefaultFilter = function(widget) {
		var editingWidget = widget || gridWidget;
		var filter = {
			"_id": "default",
			filters: this.getDefaultFilterProperties(editingWidget)
		};
		this.applyFilter(filter, editingWidget);
		this.updateFilterInfo(editingWidget, []);
	};

	function applyFilterEventCallback(eventData) {
		var filter = eventData.e.customization;
		var editingWidget = eventData.e.editingWidget;
		self.applyFilter(filter, editingWidget);
	}

	var fireBeforeBuildFilterEvent = function(editingWidget, filter) {
		var event = editingWidget.beforeGridFilterApply && editingWidget.beforeGridFilterApply(filter);
		return ZHPromise.when(event);
	};

	function _getFilterFields(filters) {
		return filters.filter(function(filter) {
			return !~filter.name.indexOf("SEARCHLIST_");
		});
	}

	function _getSearchFields(filters) {
		var searchFields = Util.clone(filters);
		searchFields = searchFields.filter(function(filter) {
			return ~filter.name.indexOf("SEARCHLIST_");
		});
		searchFields.map(function(filter) {
			filter.name = filter.name.replace("SEARCHLIST_","");
			filter.static = true;
		});
		return searchFields;
	}

	function _applySearchCriteria(searchFields, editingWidget) {
		if (editingWidget.setSearchCriteria) {
			editingWidget.setSearchCriteria(searchFields);
		}
	}

	this.applyFilter = function(filter, editingWidget, callback) {
		editingWidget._filterRow = filter._filterRow;
		var originalFilter = Util.clone(filter.filters);
		fireBeforeBuildFilterEvent(editingWidget, filter).then(function(event) {
			if (!event || !event.isDefaultPrevented()) {
				if (!editingWidget.customizationsApplied) {
					editingWidget.customizationsApplied = {};
				}
				createFilterDescription(editingWidget, originalFilter);
				editingWidget.filterApplied = originalFilter;
				editingWidget.customizationsApplied.filter = filter._id;

				var filterFields;
				var searchFields;
				if (filter.filters) {
					filterFields = _getFilterFields(filter.filters);
					searchFields = _getSearchFields(filter.filters);
				}

				if (filterFields && filterFields.length > 0) {
					filterFields.forEach(function(filter) {
						filter.static = true;
					});
				}
				if (editingWidget.dataSourceFilter) {
					editingWidget.dataSourceFilter = self.getStaticDataSourceFilters(editingWidget.dataSourceFilter);
					if (filterFields && filterFields.length > 0) {
						editingWidget.dataSourceFilter = editingWidget.dataSourceFilter.concat(filterFields);
					}
				}

				_applySearchCriteria(searchFields, editingWidget);

				if (editingWidget.afterGridFilterApply) {
					editingWidget.afterGridFilterApply();
				}

				editingWidget.reload(ZhUtil.getLikeAllFilter(editingWidget, editingWidget.searchCriteria)).then(function() {
					if (originalFilter) {
						self.updateFilterInfo(editingWidget, originalFilter);
					}
					if (callback) {
						callback();
					}
				});
			}
		});
	};

	this.preferencesListOnEnter = function(widgetPreferenceList) {
		var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
		return zhCustomizationsService.getWidgetCustomizations(gridWidget, "filter").then(function(preferences) {
			preferences.forEach(function(preference) {
				preference.defaultPreference = preference.mapUserIdDefault[userRegistered] || false;
				preference.preferenceTypeLabel = zhCustomizationsService.getLabelRepositoryType(preference.customizationRepository);
			});
			widgetPreferenceList.dataSource.data = preferences;
		});
	};

	eventEngine.bindEvent(zhCustomizationsService, "FilterPreferenceOnClick", applyFilterEventCallback);

	var handleSelectMultipleDescription = function(field, currentRow) {
		if (field.template == "field/select-multiple.html" && field.dataSource.data.length > 0) {
			if (currentRow && currentRow[field.name]) {
				field.value(currentRow[field.name]);
			}
			if (field && Util.isArray(field.value())) {
				registers = field.dataSource.data.filter(function(register) {
					return ~field.value().indexOf(register[field.valueField]);
				});
			}
		} else {
			if (field.value() && Util.isArray(field.value()) && field.dataSource.data.length > 0) {
				registers = field.dataSource.data.filter(function(register) {
					return field.value() && register[field.valueField] == field.value();
				}).shift();
				if (registers && Util.isArray(registers)) {
					var teste = [];
					registers.forEach(function(register) {
						teste.push(register[field.descriptionField]);
					});
					return teste.join(", ") || currentRow[field.descriptionField];
				}
			}
		}
	};

	var oldDescriptionCreator = function(field, currentRow) {
		var labelDescription;
		if (field.descriptionField && field.valueField) {
			labelDescription = handleSelectMultipleDescription(field, currentRow);
		} else {
			if (field.outData) {
				labelDescription = currentRow[field.outData[field.valueField]];
			} else {
				labelDescription = currentRow[field.name];
			}
		}
		if (Util.isArray(labelDescription)) {
			labelDescription = labelDescription.map(function(value) {
				if (Util.isString(value)) {
					return value.replace(/^%|%$/g, "");
				}
				return value;
			}).join(" ");
		} else {
			if (Util.isString(labelDescription)) {
				labelDescription = labelDescription.replace(/^%|%$/g, "");
			}
		}
		return labelDescription;
	};

	var createFilterDescription = function(gridWidget, filters) {
		var fields = gridWidget.fields;
		var currentRow = buildCurrentRowByFilters(filters, gridWidget);
		filterDescription = [];
		if (!Util.isEmptyObject(currentRow)) {
			filterVisibleFields(fields).forEach(function(field) {
				var _field = Util.clone(field, ['widget', 'element', 'owner']);
				_field.widget = field.widget;
				_field.template = (_field.filterProperties && _field.filterProperties.template) || _field.template;
				var labelDescription = DecodeValueFieldService.model(_field, currentRow[_field.name]) || oldDescriptionCreator(_field, filters);
				if (labelDescription) {
					filterDescription[field.name] = labelDescription;
				}
			});
		}
	};

	var openCreateNewFilterWidget = function(popupWidget) {
		var createNewFilterWidget = filterWidgetsCreator.createNewFilterWidget(gridWidget, functionsCreateNewFilterWidget);
		createNewFilterWidget.getField('type').dataSource.data = zhCustomizationsService.getCustomizationsRepositoriesTypes();
		createNewFilterWidget.newRow();
		ScreenService.openPopup(createNewFilterWidget);
	};

	var createNewFilter = function(widget) {
		if (widget.isValid()) {
			var filterName = widget.currentRow.name;
			var customizationRepository = widget.currentRow.type;
			var preferenceHeader = createFilterHeader(filterName, customizationRepository);

			ScreenService.closePopup();
			templateManager.updateTemplate();
			var preferencesListWidget = gridWidget.__filterConfig;

			self.preferencesListOnEnter(preferencesListWidget).then(function() {
				openEditFilter(preferencesListWidget, preferenceHeader);
			});
		}
	};

	var createFilterHeader = function(filterName, customizationRepository) {
		var header = {
			"label": filterName,
			"widgetId": gridWidget.id,
			"containerId": gridWidget.container.id,
			"customizationRepository": customizationRepository,
			"unsync_id": new Date().getTime(),
			"customizationType": "filter",
			"mapUserIdDefault": {
				"default": false
			},
			"defaultPreference": false
		};
		return header;
	};

	var deletePreferenceClick = function(widget) {
		return zhCustomizationsService.removeCustomizations(widget.getCheckedRows()).then(function() {
			self.preferencesListOnEnter(widget).then(function() {
				templateManager.updateTemplate();
			});
		});
	};

	var openPopupSelectDestinyRepository = function(filterListWidget) {
		var traferFilterTypeWidget = filterWidgetsCreator.createChangeFilterTypeWidget(gridWidget, functionsTranferFilterType);
		traferFilterTypeWidget.newRow();
		traferFilterTypeWidget.getField('filterType').dataSource.data = zhCustomizationsService.getCustomizationsRepositoriesTypes();
		var toTransferLayouts = filterListWidget.getCheckedRows();
		var isSomeLayoutConnectedToAnVisuzalization = false;
		var promises = toTransferLayouts.map(function(filter) {
			return zhCustomizationsService.getCustomizationView(gridWidget, filter.customizationRepository, filter.customizationType, filter._id).then(function(connectedView) {
				if (connectedView) {
					isSomeLayoutConnectedToAnVisuzalization = true;
				}
			});
		});
		ZHPromise.all(promises).then(function() {
			if (!isSomeLayoutConnectedToAnVisuzalization) {
				ScreenService.openPopup(traferFilterTypeWidget);
			} else {
				ScreenService.showMessage(ScreenService.i18n("Filter connected with a customizations. It can't be transferred."));
			}
		});
	};

	var saveTransferFilterClick = function(widgetChangePreferenceType) {
		var newFilterType = widgetChangePreferenceType.getField("filterType").value();
		zhCustomizationsService.transferCustomizationsRepository(gridWidget, gridWidget.__filterConfig.getCheckedRows(), newFilterType).then(function() {
			fillFilterList(gridWidget.__filterConfig);
			ScreenService.closePopup();
		});
	};

	var openEditFilter = function(filterListWidget, row) {
		var filterFormWidget = filterWidgetsCreator.createFormFilterWidget(gridWidget, functionsFormFilterWidget);
		filterFormWidget.showAsModal = true;
		filterFormWidget.label = ScreenService.i18n("Filter") + " - " + row.label;
		filterFormWidget.filterSelected = row;
		filterFormWidget.fields = prepareFieldstoFormFilter(factoryFields(filterFormWidget));
		var searchFields = gridWidget.hideSearchFieldsOnFilter ? [] : prepareFieldstoFormFilter(factorySearchFields(filterFormWidget));
		filterFormWidget.fields = filterFormWidget.fields.concat(searchFields);
		filterFormWidget.currentRow = buildCurrentRowByFilters(row.filters, gridWidget, row._filterRow);
		ScreenService.openPopup(filterFormWidget);
	};

	var removeConflictingProperties = function(field) {
		var conflictingProperties = ['selectWidget'];
		conflictingProperties.forEach(function(property) {
			delete field[property];
		});
	};

	var factoryFields = function(filterForm) {
		return gridWidget.fields.filter(function(field) {
			return !Util.isUndefined(field.template);
		}).map(function(field) {
			var factoredField = metaDataFactory.fieldFactory(field);
			removeConflictingProperties(factoredField);
			factoredField.widget = filterForm;
			return factoredField;
		});
	};

	var factorySearchFields = function(filterForm) {
		var fields = [];
		fields.push(metaDataFactory.fieldFactory({
			"id": "SEARCHLIST__ALL",
			"label": ScreenService.i18n("All Fields"),
			"name": "SEARCHLIST__ALL",
			"class": 12,
			"template": "field/text-edit.html",
			"isVisible": true,
			"showOnList": true,
			"descending": true,
			"fieldGroup": "COLUMNS_FILTER"
		}, filterForm));

		fields = fields.concat(gridWidget.fields.filter(function(field) {
			return !field.skipSearch && Util.isUndefined(field.template) && !filterForm.getField(field.name);
		}).map(function(field) {
			var factoredField = metaDataFactory.fieldFactory(field);
			delete factoredField.$$hashKey;
			factoredField.id = "SEARCHLIST_"+factoredField.id;
			factoredField.name = "SEARCHLIST_"+factoredField.name;
			factoredField.class = 6;
			factoredField.isVisible = true;
			factoredField.readOnly = false;
			factoredField.validations = {};
			factoredField.template = "field/text-edit.html";
			factoredField.fieldGroup = "COLUMNS_FILTER";
			factoredField.widget = filterForm;
			return factoredField;
		}));

		return fields;
	};

	var prepareFieldstoFormFilter = this.prepareFields = function(fields) {
		return fields.map(function(field) {
			return field.filterProperties ? mergeFieldNewProperties(field, field.filterProperties) : field;
		});
	};

	var saveAndApplyFilter = function(filterFormWidget) {
		if (filterFormWidget.isValid()) {
			saveFilter(filterFormWidget).then(function() {
				self.applyFilter(filterFormWidget.filterSelected, gridWidget, function() {
					ScreenService.closePopup();
				});
			});
		}
	};

	var saveFilter = function(filterFormWidget) {
		filterFormWidget.filterSelected._filterRow = ZhUtil.clearRow(filterFormWidget.currentRow);
		filterFormWidget.filterSelected.filters = ZhFilterBuilder.buildFilter(filterFormWidget.fields.slice(), filterFormWidget.currentRow);
		removeNotInterestingProperties(filterFormWidget.filterSelected);
		return zhCustomizationsService.saveCustomization(filterFormWidget.filterSelected, 'filter').then(function() {
			self.preferencesListOnEnter(gridWidget.__filterConfig).then(function() {
				ScreenService.closePopup();
				templateManager.updateTemplate();
			});
		});
	};

	var removeNotInterestingProperties = function(object) {
		Object.keys(object).forEach(function(key) {
			if (~key.indexOf('__')) {
				delete object[key];
			}
		});
	};

	var applyFilterActionOnClick = function(filter) {
		self.applyFilter(filter, gridWidget);
		ScreenService.closePopup(true);
		gridWidget.activate();
	};

	var mergeFieldNewProperties = function(field, filterProperties) {
		var keys = Object.keys(field);
		keys = keys.concat(
			Object.keys(filterProperties).filter(function(key) {
				return keys.indexOf(key) == -1;
			})
		);
		keys.forEach(function(property) {
			field[property] = filterProperties.hasOwnProperty(property) ? filterProperties[property] : field[property];
		});
		if (field.dataSource && !(field.dataSource instanceof DataSource)) {
			field.dataSource = metaDataFactory.factoryDataSource(field.dataSource, field.dataSourceFilter, field);
		}
		return field;
	};

	var buildCurrentRowByFilters = function(filters, gridWidget, descriptionValues) {
		filters = Util.clone(filters) || [];
		var builtCurrentRow = descriptionValues || {};

		if (gridWidget && gridWidget.fields) {
			gridWidget.fields.forEach(function(field) {
				var fieldFilter = getFieldFilter(field.name, filters);
				if (fieldFilter) {
					if (fieldFilter.operator === 'LIKE' || fieldFilter.operator === 'LIKE_I') {
						if (fieldFilter.value[fieldFilter.value.length - 1] == '%') {
							fieldFilter.value = fieldFilter.value.substring(0, (fieldFilter.value.length - 1));
						}
						if (fieldFilter.value[0] == '%') {
							fieldFilter.value = fieldFilter.value.substring(1, fieldFilter.value.length);
						}
					} else if (fieldFilter.operator && ~fieldFilter.operator.indexOf("BETWEEN")) {
						fieldFilter.value = {
							start: fieldFilter.value[0],
							end: fieldFilter.value[1],
						};
					}
					builtCurrentRow[field.name] = fieldFilter.value;
					/*
					 dumb reload, should remove this in the next release!
					*/
					field.reload();
				}
			});
		}

		// add SearchList values
		filters.filter(function(filter) {
			return ~filter.name.indexOf("SEARCHLIST_");
		}).forEach(function(filter) {
			var filterValue = filter.value;
			if (filterValue[filterValue.length - 1] == '%') {
				filterValue = filterValue.substring(0, (filterValue.length - 1));
			}
			if (filterValue[0] == "%") {
				filterValue = filterValue.substring(1, filterValue.length);
			}
			builtCurrentRow[filter.name] = filterValue;
		});
		return builtCurrentRow;
	};

	this.inArray = function(elem, array) {
		return -1 != (Util.isArray(array) ? array.indexOf(elem) : -1);
	};

	var getFieldFilter = function(fieldName, filters) {
		return filters.filter(function(filter) {
			return filter.name == fieldName;
		}).shift();
	};

	var closeFilterPopup = function() {
		ScreenService.closePopup();
		gridWidget.activate();
		if (gridWidget.afterGridFilterCancel) {
			gridWidget.afterGridFilterCancel();
		}
	};

	this.updateFilterInfo = function(gridWidget, filters) {
		var stripeFilterFields = [];
		var currentFilter = buildCurrentRowByFilters(filters, gridWidget);
		var filterLabel = "";
		gridWidget._filterInfo = filterLabel;

		if (gridWidget.customizationsApplied && gridWidget.customizationsApplied.filter && gridWidget.customizationsApplied.filter != "edited" && gridWidget.customizationsApplied.filter != "default") {
			zhCustomizationsService.getWidgetCustomizations(gridWidget, "filter").then(function(filters) {
				filters.filter(function(filter) {
					if (filter._id == gridWidget.customizationsApplied.filter) {
						filterLabel = "<b>" + ScreenService.i18n("Filter") + ":</b> " + filter.label;
						gridWidget._filterInfo = filterLabel;
						ScreenService.updateTemplate();
					}
				});
			});
		} else {
			filterLabel = [];
			if (Util.isArray(filters)) {
				filters.forEach(function(filter) {
					var field = gridWidget.getField ? gridWidget.getField(filter.name) : undefined;
					if (field) {
						if (hasSelectTemplate(field)) {
							currentFilter[filter.name] = getSelectFieldDescriptionValue(field, currentFilter[filter.name]);
						}
						var fieldLabel = ScreenService.i18n(field.label);
						var nextField;
						if (filterDescription[field.name]) {
							nextField = "<b>" + fieldLabel + ":</b> " + filterDescription[field.name];
						} else if (filter.operator == "BETWEEN") {
							nextField = "<b>" + fieldLabel + ":</b> " + currentFilter[filter.name].start + ' - ' + currentFilter[filter.name].end;
						} else {
							nextField = "<b>" + fieldLabel + ":</b> " + currentFilter[filter.name];
						}

						filterLabel.push(nextField);
					} else if (~filter.name.indexOf("SEARCHLIST_")) {
						if (filter.name == "SEARCHLIST__ALL") {
							filterLabel.push("<b>" + ScreenService.i18n("All Fields") + ":</b> " + currentFilter[filter.name]);
						} else {
							field = gridWidget.getField ? gridWidget.getField(filter.name.substring(11)) : undefined;
							if (field) {
								if (hasSelectTemplate(field)) {
									currentFilter[filter.name] = getSelectFieldDescriptionValue(field, currentFilter[filter.name]);
								}
								filterLabel.push("<b>" + ScreenService.i18n(field.label) + ":</b> " + currentFilter[filter.name]);
							}
						}
					}
				});
			} else if (Util.isObject(filters)) {
				Object.keys(filters).forEach(function(key) {
					if (key == "_ALL") {
						filterLabel.push("<b>" + ScreenService.i18n("All Fields") + ":</b> " + filters[filter.name]);
					} else {
						field = gridWidget.getField(key);
						if (field) {
							if (field.template && hasSelectTemplate(field)) {
								filters[key] = getSelectFieldDescriptionValue(field, filters[key]);
							}
							filterLabel.push("<b>" + ScreenService.i18n(field.label) + ":</b> " + filters[key]);
						}
					}
				});
			}

			filterLabel = filterLabel.join("<br />");
			gridWidget._filterInfo = filterLabel;
			ScreenService.updateTemplate();
		}
	};

	var hasSelectTemplate = function(field) {
		return field.template && ~field.template.indexOf('select-native.html');
	};

	var getSelectFieldDescriptionValue = function(field, value) {
		if (field.dataSource && field.dataSource.data) {
			var selectedRow = field.dataSource.data.filter(function(row) {
				return value == row[field.valueField].toString();
			});
			if (selectedRow.length > 0) {
				value = selectedRow[0][field.descriptionField];
			}
		}
		return value;
	};

	var factoryStripeFields = function(stripe, field) {
		factoredField = metaDataFactory.fieldFactory(field);
		factoredField.widget = stripe;
		if (stripe.__originalFields) {
			stripe.fields = stripe.__originalFields.concat([factoredField]);
		} else {
			stripe.fields = factoredField;
		}
	};

	var defaultFilterProperties = {};

	this.setFilterProperties = function(gridWidget, currentRow) {
		var filter = {
			"_id": "default",
			filters: ZhFilterBuilder.buildFilter(gridWidget.fields, currentRow)
		};
		self.applyFilter(filter, gridWidget);
	};

	this.setDefaultFilterProperties = function(widget, currentRow) {
		gridWidget = widget;
		defaultFilterProperties[gridWidget.id] = ZhFilterBuilder.buildFilter(gridWidget.fields, currentRow);
		if (!gridWidget.filterApplied || !gridWidget.filterApplied.length) {
			self.setFilterProperties(gridWidget, currentRow);
		}
	};

	this.getDefaultFilterProperties = function(gridWidget) {
		return defaultFilterProperties[gridWidget.id];
	};

	this.getStaticDataSourceFilters = function(dataSourceFilter) {
		return dataSourceFilter.filter(function(filter) {
			return !filter.static;
		});
	};

	var setAsDefaultPreference = function(row, widget) {
		var toUpdate = [];
		var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
		var preferenceSetAsDefault = widget.dataSource.data.filter(function(preference) {
			return preference.mapUserIdDefault[userRegistered] === true && preference._id !== row._id;
		}).shift();
		if (preferenceSetAsDefault) {
			preferenceSetAsDefault.mapUserIdDefault[userRegistered] = false;
			removeNotInterestingProperties(preferenceSetAsDefault);
			toUpdate.push(preferenceSetAsDefault);
		}
		removeNotInterestingProperties(row);
		row.mapUserIdDefault[userRegistered] = !row.mapUserIdDefault[userRegistered];
		toUpdate.push(row);
		zhCustomizationsService.saveCustomization(toUpdate, 'filter').then(function() {
			self.preferencesListOnEnter(widget).then(function() {
				ScreenService.hideLoader();
				ScreenService.updateTemplate();
			});
		});
	};

	var functionsFilterListWidget = {
		"fillFilterList": fillFilterList,
		"openEditFilter": openEditFilter,
		"openCreateNewFilterWidget": openCreateNewFilterWidget,
		"deletePreferenceClick": deletePreferenceClick,
		"openPopupSelectDestinyRepository": openPopupSelectDestinyRepository,
		"applyFilterActionOnClick": applyFilterActionOnClick,
		"closeFilterPopup": closeFilterPopup,
		"setAsDefaultPreference": setAsDefaultPreference,
		"restoreDefaultFilter": self.restoreDefaultFilter,
		"preferencesListOnEnter": self.preferencesListOnEnter
	};

	var functionsCreateNewFilterWidget = {
		"createNewFilter": createNewFilter
	};

	var functionsTranferFilterType = {
		"saveTransferFilterClick": saveTransferFilterClick
	};

	var functionsFormFilterWidget = {
		"saveAndApplyFilter": saveAndApplyFilter,
		"saveFilter": saveFilter
	};

	var functionsFormCurrentFilterWidget = {
		"currentFilterClickPopup": currentFilterClickPopup
	};

}

Configuration(function(ContextRegister) {
	var zhFilterPreferences;
	ContextRegister.register("zhFilterPreferences", function(ScreenService, templateManager, ZhFilterBuilder, ZhUtil, RepositoryFactory, Query, eventEngine, ZHPromise, ZhLogger, ApplicationContext, zhCustomizationsService, filterWidgetsCreator, eventAggregator, preferencesUserService, DecodeValueFieldService) {
		zhFilterPreferences = new ZhFilterPreferences(ScreenService, templateManager, ZhFilterBuilder, ZhUtil, RepositoryFactory, Query, eventEngine, ZHPromise, ZhLogger, ApplicationContext, zhCustomizationsService, filterWidgetsCreator, eventAggregator, preferencesUserService, DecodeValueFieldService);
		return zhFilterPreferences;
	});
	ContextRegister.register("ZhGridFilter", zhFilterPreferences);
});

// FILE: ./bower_components/zh-cache-bypass/src/CacheBypass.js
(function() {
    angular.module("ZeedhiApp").config(function($httpProvider) {
        if (Boolean(window.applicationVersion)) {
            $httpProvider.interceptors.push(function() {
                return {
                    'request': function cacheBypass(config) {
                        if (config && config.url && shouldApplyBypass(config.url)) {
                            config.url += '?v='+window.applicationVersion;
                        }
                        return config;
                    }
                };
            });
        } else {
            console.warn('Missing "applicationVersion" global property for CacheBypass.');
        }
    });

    function shouldApplyBypass(url) {
        return (
            Util.stringEndsWith(url, '.json') ||
            Util.stringEndsWith(url, '.html') ||
            Util.stringEndsWith(url, '.svg')
        );
    }
})();

// FILE: ./bower_components/red-ui/dist/rui.js
// FILE: src/assets/js/controllers/ruiController.js
function ruiController($rootScope, ScreenService, eventAggregator, templateManager) {

	eventAggregator.subscribe("beforeProjectStartup", function() {
		var interval = setInterval(function() {
			if (window.__back__) {
				window.__back__ = function() {
					if (ScreenService.isAlertShown()) {
						return;
					} else if ($rootScope.showAbout) {
						ScreenService.hideAbout();
					} else if (ScreenService.isPopupOpen()) {
						ScreenService.closePopup();
					} else if (ScreenService.isSideMenuOpen()) {
						ScreenService.toggleSideMenu();
					} else if ($rootScope.headerDropdownOpened) {
						toggleHeaderDropdown();
						$rootScope.$apply();
					} else if ($rootScope.searching) {
						toggleHeaderSearch();
						$rootScope.$apply();
					} else {
						var widget = getActiveWidget();
						if (widget.mode === 'edit' && widget.onCancel) {
							widget.onCancel();
						} else if (widget.widgetOnBack) {
							widget.widgetOnBack();
						} else if (widget.container && widget.container.onBack) {
							widget.container.onBack();
						} else {
							ScreenService.goBack();
						}
					}
				};
				clearInterval(interval);
			}
		}, 300);
		htmlFixesToRUI();
	});

	function getActiveWidget() {
		return templateManager.container.getActiveWidget();
	}
	function htmlFixesToRUI (){
		$('html').removeClass('splash');
		$('html').addClass('hide-scroll');
	}
}

Configuration(function(ContextRegister) {
	ContextRegister.register("ruiController", ruiController);
});

(function() {
	setInterval (function(){
		$('.container[zh-scroller]').scroll(function(){
			$('html').removeClass('hide-scroll');
			setTimeout(function(){
				$('html').addClass('hide-scroll');
			},1500);
		});
	},800);
})();



// FILE: ./bower_components/red-ui-fields/dist/rui-fields.js
// FILE: src/assets/js/controllers/ruiDateController.js
function ruiDateController($scope, $element, ZhUtil, ScreenService) {

    $scope.ZhUtil = ZhUtil;

    var nomeDosMeses = [
        "janeiro",
        "fevereiro",
        "março",
        "abril",
        "maio",
        "junho",
        "julho",
        "agosto",
        "setembro",
        "outubro",
        "novembro",
        "dezembro"
    ];

    var diasDaSemanaAbreviado = ["dom", "seg", "ter", "qua", "qui", "sex", "sab"];

    var container = $element.get()[0];
    var dateInput = container.querySelector('#date-input');
    var calendarMode = 0x1;
    var yearMode = 0x2;

    $scope.init = function() {
        if (window.orientation)
            detectOrientationAndRotate();
        var fieldValue;
        if ($scope.fieldRow)
            fieldValue = $scope.fieldRow;
        else
            fieldValue = $scope.ZhUtil.evaluate($scope.field.defaultValue);
        if (fieldValue === undefined) {
            fieldValue = '';
        }
        else {
            dateInput.classList.add('rui-has-value');
            var dateArray = fieldValue.split('/'); //[0] == day, [1] == month, [2] == year
            dateContainer.date.day = parseInt(dateArray[0], 10);
            dateContainer.selectedDay = parseInt(dateArray[0], 10);
            dateContainer.date.month = parseInt(dateArray[1], 10) - 1;
            dateContainer.selectedMonth = parseInt(dateArray[1], 10) - 1;
            dateContainer.date.year = parseInt(dateArray[2], 10);
            dateContainer.selectedYear = parseInt(dateArray[2], 10);
            dateContainer.dateWasSelected = true;
            dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
            dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
            var monthYearString = buildMonthWithYearString(dateContainer.selectedMonth, dateContainer.selectedYear);
            dateContainer.monthDisplay.textContent = monthYearString;
            dateInput.children[1].textContent = fieldValue;
            generateCalendar(dateContainer.selectedMonth, dateContainer.selectedYear);
        }
        var label = ScreenService.i18n($scope.field.label);
        if ($scope.field.validations && $scope.field.validations.required) {
            label += " *";
        }
        dateInput.children[0].textContent = label;
        $scope.fieldRow = fieldValue;
        if ($scope.field.hideOverlay) {
            overlay.style.opacity = '0';
        }
        if($scope.field.readOnly){
            dateInput.classList.add('disabled');
        }
    };


    var dateContainer = container.querySelector('#calendar-container');
    var overlay = container.querySelector('#overlay');
    var modal = container.querySelector('#modalWindow');
    dateContainer.okButton = container.querySelector('#ok');
    dateContainer.cancelButton = container.querySelector('#cancel');
    dateContainer.table = container.querySelector('#calendar-table');
    dateContainer.days = container.querySelector('#days-container');
    dateContainer.monthDisplay = container.querySelector('#month-display');
    dateContainer.previousMonthButton = container.querySelector('#previous-month-button');
    dateContainer.nextMonthButton = container.querySelector('#next-month-button');
    dateContainer.topYearDisplay = container.querySelector('#top-year-display');
    dateContainer.topFullDateDisplay = container.querySelector('#top-full-date-display');
    dateContainer.yearPicker = container.querySelector('#year-picker-container');
    dateContainer.yearPickerOutterWrapper = container.querySelector('#year-picker-outter-wrapper');
    dateContainer.currentSelectedYearSpanInYearPicker = undefined;

    dateContainer.dateWasSelected = false;

    dateContainer.mode = calendarMode;

    dateContainer.validate = function() {
        $scope.field.validate($scope.fieldRow);
        $scope.$apply();
    };

    $scope.field.cancel = function() {
        overlay.classList.remove('active');
        modal.classList.remove('active');
        if (dateContainer.dateWasSelected) {
            dateInput.classList.add('rui-has-value');
        }
        dateInput.classList.remove('active');
        dateContainer.validate();
    };

    $scope.field.open = function(e) {
        if (e) {
            e.preventDefault();
        }
        dateInput.classList.add('active');
        dateInput.classList.remove('rui-has-value');
        toggleCalendarMode();
        overlay.classList.add('active');
        modal.classList.add('active');
        generateCalendar(dateContainer.date.month, dateContainer.date.year);
    };

    $scope.field.confirm = function(e) {
        dateContainer.dateWasSelected = true;
        overlay.classList.remove('active');
        $scope.fieldRow =
            ((dateContainer.selectedDay) < 10 ? "0" + (dateContainer.selectedDay) : "" + (dateContainer.selectedDay)) +
            "/" + ((dateContainer.selectedMonth + 1) < 10 ? "0" + (dateContainer.selectedMonth + 1) : "" + (dateContainer.selectedMonth + 1)) +
            "/" + dateContainer.selectedYear;
        dateInput.children[1].textContent = $scope.fieldRow;
        modal.classList.remove('active');
        dateInput.classList.add('rui-has-value');
        dateInput.classList.remove('active');
        dateContainer.validate();
    };


    overlay.onclick = $scope.field.cancel;
    dateInput.onclick = $scope.field.open;
    dateContainer.okButton.onclick = $scope.field.confirm;
    dateContainer.cancelButton.onclick = $scope.field.cancel;


    var dayClickHandler = function(eventInfo) {
        var previousActiveDay = dateContainer.days.children[dateContainer.phantomDaysOffset + dateContainer.selectedDay];
        previousActiveDay.classList.remove('active');
        dateContainer.selectedDay = Number.parseInt(eventInfo.target.textContent, 10);
        var activeDay = dateContainer.days.children[dateContainer.phantomDaysOffset + dateContainer.selectedDay];
        activeDay.classList.add('active');
        dateContainer.date.day = dateContainer.selectedDay;
        dateContainer.date.month = dateContainer.selectedMonth;
        dateContainer.date.year = dateContainer.selectedYear;
        dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
        dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
    };

    function generateCalendar( /*number */ month, /*number */ year) /*number: phantomOffset*/ {
        //debugger;
        //dateContainer.table;
        //dateContainer.days.opacity = 0;
        while (dateContainer.days.firstChild) {
            dateContainer.days.removeChild(dateContainer.days.firstChild);
        }

        var phantomDay = document.createElement('span');
        phantomDay.classList.add('phantom-day');
        phantomDay.textContent = '00';

        var firstDayOfWeek = new Date(year, month, 1).getDay();
        dateContainer.phantomDaysOffset = firstDayOfWeek - 1;

        for (var i = 0; i < firstDayOfWeek; i++) {
            dateContainer.days.appendChild(phantomDay.cloneNode());
        }

        var lastDayOfMonth = new Date(year, month + 1, 0).getDate();

        for (var j = 1; j <= lastDayOfMonth; j++) {
            var day = document.createElement('span');
            day.textContent = "" + j; //((i < 10) ? "0" + i : "" + i);
            day.onclick = dayClickHandler;
            dateContainer.days.appendChild(day);
        }

        if (dateContainer.date.month === month &&
            dateContainer.date.year === year) {
            var phantomDaysOffset = new Date(year, month, 1).getDay() - 1;
            var activeDay = dateContainer.days.children[phantomDaysOffset + dateContainer.date.day];
            activeDay.classList.add('active');
        }

    }

    /*monthsToAdd = 1 vai para o proximo mes, e -1 para o anterior*/
    /* retorna o objeto da nova data*/
    var addMonths = function( /*number*/ monthsToAdd) {
        var newDate = new Date(
            dateContainer.selectedYear,
            dateContainer.selectedMonth,
            dateContainer.selectedDay
        );
        newDate.setMonth(dateContainer.selectedMonth + monthsToAdd);
        var newString = buildMonthWithYearString(newDate.getMonth(), newDate.getFullYear());
        dateContainer.monthDisplay.textContent = newString;
        dateContainer.selectedYear = newDate.getFullYear();
        dateContainer.selectedMonth = newDate.getMonth();
        dateContainer.selectedDay = newDate.getDate();
        generateCalendar(newDate.getMonth(), newDate.getFullYear());
        return newDate;
    };

    var buildMonthWithYearString = function( /*number */ month, /*number */ year) {
        return nomeDosMeses[month] + " de " + year;
    };

    var buildTopDisplayString = function() /* string */ {
        //return nomeDosMeses[month] + " de " + year;
        var date = new Date(
            dateContainer.selectedYear,
            dateContainer.selectedMonth,
            dateContainer.selectedDay
        );
        var nomeDiaDaSemana = diasDaSemanaAbreviado[date.getDay()];
        var nomeDoMesAbreviado = nomeDosMeses[dateContainer.selectedMonth].substr(0, 3);
        return nomeDiaDaSemana + ", " + dateContainer.selectedDay + " de " + nomeDoMesAbreviado;
    };

    var setDateUsingJSDateObject = function( /* Date */ data) {
        dateContainer.date.year = data.getFullYear();
        dateContainer.date.month = data.getMonth();
        dateContainer.date.day = data.getDate();
    };

    dateContainer.previousMonthButton.onclick = function() {
        var newDate = addMonths(-1);
    };

    dateContainer.nextMonthButton.onclick = function() {
        var newDate = addMonths(1);
    };

    var toggleYearMode = function() {
        dateContainer.classList.remove('active');
        dateContainer.yearPickerOutterWrapper.classList.add('active');
        dateContainer.topYearDisplay.classList.add('active');
        dateContainer.topFullDateDisplay.classList.remove('active');
        dateContainer.currentSelectedYearSpanInYearPicker.scrollIntoView();
    };

    var toggleCalendarMode = function() {
        dateContainer.classList.add('active');
        dateContainer.yearPickerOutterWrapper.classList.remove('active');
        dateContainer.topYearDisplay.classList.remove('active');
        dateContainer.topFullDateDisplay.classList.add('active');
    };

    var pickYear = function( /* clickEvent */ e) {
        var newYear = e.target.textContent;
        dateContainer.selectedYear = newYear;
        var newString = buildMonthWithYearString(dateContainer.selectedMonth, newYear);
        dateContainer.currentSelectedYearSpanInYearPicker.classList.remove('active');
        dateContainer.currentSelectedYearSpanInYearPicker = e.target;
        e.target.classList.add('active');
        dateContainer.monthDisplay.textContent = newString;
        dateContainer.topYearDisplay.textContent = newYear;
        dateContainer.date.year = newYear;
        generateCalendar(dateContainer.selectedMonth, dateContainer.selectedYear);
        toggleCalendarMode();
    };

    /*initial setup*/

    var today = new Date();
    var hoje = today.getDate();
    dateContainer.selectedDay = hoje;
    dateContainer.selectedMonth = today.getMonth();
    dateContainer.selectedYear = today.getFullYear();
    dateContainer.date = {
        year: today.getFullYear(),
        day: hoje,
        month: today.getMonth()
    };
    dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
    dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
    dateContainer.monthDisplay.textContent = buildMonthWithYearString(today.getMonth(), today.getFullYear());
    generateCalendar(today.getMonth(), today.getFullYear());
    var activeDay = dateContainer.days.children[dateContainer.phantomDaysOffset + dateContainer.selectedDay];
    activeDay.classList.add('active');

    dateContainer.topYearDisplay.onclick = toggleYearMode;
    dateContainer.topFullDateDisplay.onclick = toggleCalendarMode;

    for (var i = 1970; i < 2051; i++) {
        var n = document.createElement("span");
        n.textContent = i;
        dateContainer.yearPicker.appendChild(n);
        n.onclick = pickYear;
    }

    dateContainer.yearPicker.currentYearNode = dateContainer.yearPicker.children[today.getFullYear() - 1970];
    dateContainer.yearPicker.currentYearNode.classList.add('active');
    dateContainer.currentSelectedYearSpanInYearPicker = dateContainer.yearPicker.currentYearNode;

    /******* ORIENTATION ********/

    if (window.orientation !== undefined) {
        window.addEventListener("orientationchange", function() {
            detectOrientationAndRotate();
        });
    }

    var detectOrientationAndRotate = function() {
        if (orientation === 0 || orientation === 180) {
            modal.classList.remove('landscape');
        }
        else {
            modal.classList.add('landscape');
        }
    };

    var clearButton = container.querySelector('#fake-input-clear');
    clearButton.onclick = function(event) {
        event.stopPropagation();
        //
        dateInput.classList.remove('rui-has-value');
        var today = new Date();
        dateContainer.date.day = today.getDate();
        dateContainer.selectedDay = dateContainer.date.day;
        dateContainer.date.month = today.getMonth();
        dateContainer.selectedMonth = dateContainer.date.month;
        dateContainer.date.year = today.getFullYear();
        dateContainer.selectedYear = dateContainer.date.year;
        dateContainer.dateWasSelected = false;
        dateContainer.topFullDateDisplay.textContent = buildTopDisplayString();
        dateContainer.topYearDisplay.textContent = dateContainer.selectedYear;
        var monthYearString = buildMonthWithYearString(dateContainer.selectedMonth, dateContainer.selectedYear);
        dateContainer.monthDisplay.textContent = monthYearString;
        dateInput.children[1].textContent = '';
        generateCalendar(dateContainer.selectedMonth, dateContainer.selectedYear);
        $scope.fieldRow = '';
        dateContainer.currentSelectedYearSpanInYearPicker.classList.remove('active');
        dateContainer.yearPicker.currentYearNode.classList.add('active');
        dateContainer.currentSelectedYearSpanInYearPicker = dateContainer.yearPicker.currentYearNode;
        dateContainer.validate();
    };
}


// FILE: src/assets/js/controllers/ruiDateNativeController.js
function ruiDateNativeController($scope, $element, ZhUtil, ScreenService) {
	var defaultFormat = "YYYY-MM-DD";
	var field = $scope.field;
	var input = $element.get(0).querySelector(".rui-txt-input-text");
	var range = {
		maxDate: field.maxDate,
		minDate: field.minDate
	};

	field.applyDefaultValue = function() {
		if (field.widget.currentRow) {
			field.widget.currentRow[field.name] = typeof field.defaultValue === "undefined" ? "" : ZhUtil.evaluate(field.defaultValue);
			field.currentDate = getValueWithDefaultFormat(field.widget.currentRow[field.name]);
		}
	};

	if(field.widget.currentRow && field.defaultValue) {
		field.applyDefaultValue();
	}

	$scope.getLabel = function(label, validations) {
		label = ScreenService.i18n(label);
		if (validations && Object.keys(validations).length && validations.required) {
			label += ' *';
		}
		return label;
	};
	$scope.formatValue = function() {
		if (field.currentDate) {
			field.widget.currentRow[field.name] = moment(field.currentDate, defaultFormat).format(field.dateFormat);
		} else {
			field.widget.currentRow[field.name] = field.currentDate;
		}
	};

	updateRange(field.minDate, field.maxDate);

	if (Object.getOwnPropertyDescriptor(field, 'maxDate') && !Object.getOwnPropertyDescriptor(field, 'maxDate').get) {
		Object.defineProperty(field, 'maxDate', {
	        set: function(maxDate) {
	            range.maxDate = maxDate;
	            updateRange(null, maxDate);
	        },
	        get: function(newValue) {
	            return range.maxDate;
	        }
	    });
	}

	if (Object.getOwnPropertyDescriptor(field, 'minDate') && !Object.getOwnPropertyDescriptor(field, 'minDate').get) {
		Object.defineProperty(field, 'minDate', {
	        set: function(minDate) {
	            range.minDate = minDate;
	            updateRange(minDate, null);
	        },
	        get: function(newValue) {
	            return range.minDate;
	        }
	    });
	}

	function updateRange(minDate, maxDate) {
		if (maxDate) {
			var max = moment(ZhUtil.evaluate(maxDate), field.dateFormat).format(defaultFormat);
			input.setAttribute("max", max);
		}
		if (minDate) {
			var min = moment(ZhUtil.evaluate(minDate), field.dateFormat).format(defaultFormat);
			input.setAttribute("min", min);
		}
	}

	function getValueWithDefaultFormat(value) {
		return moment(value, field.dateFormat).format(defaultFormat);
	}
}

// FILE: src/assets/js/controllers/ruiFABController.js
function ruiFABController($rootScope, ApplicationContext,eventAggregator, templateManager) {


    function getNewFAB() {
        var FABRoot = document.createElement('span');
        var FABIcon = document.createElement('span');
        var FABAction = {};

        FABIcon.className = "material-icons fab-icon";
        FABIcon.textContent = "pregnant_woman";

        FABRoot.classList.add('fab-root');
        FABRoot.appendChild(FABIcon);

        FABRoot.setIcon = function( /*string*/ iconName) {
            if (typeof iconName !== 'string') {
                console.log('%cfunction setIcon expects parameter 1 ' +
                    'to be of type string. Received ' + typeof iconName + ' instead',
                    'font-size:20px;background-color:red;color:yellow;');
                return "YOUR CALL WAS REJECTED LOSER";
            }
            FABIcon.textContent = iconName;
        };

        FABRoot.setAction = function( /*object*/ action) {
            if (typeof action !== 'object') {
                console.log('%cfunction setIcon expects parameter 1 ' +
                    'to be of type Object. Received ' + typeof action + ' instead',
                    'font-size:20px;background-color:red;color:yellow;');
                return "YOUR CALL WAS REJECTED LOSER";
            }
            FABAction = action;
        };


        FABRoot.id = "rui-FAB";
        FABRoot.addEventListener('click', function() {
            var eventContext = getApplicationContextWithEventArgs(FABAction);
            $rootScope.$eval(FABRoot.code, eventContext);
            $rootScope.$apply();
        });

        return FABRoot;
    }

    var FABElement = getNewFAB();
    var defaultIconName = "sentimentverysatisfied";

    var getApplicationContextWithEventArgs = function(eventOwner) {
        ApplicationContext.args = {
            owner: eventOwner
        };
        return ApplicationContext;
    };


    var modes = [
        "FAB_edit",
        "FAB_view",
        "FAB_list",
        "FAB_checked_rows",
        "FAB"
    ];

    // find the first action with the FAB mode
    var findActionToDisplay = function(widget) {
        var currentWidgetMode = widget.mode;
        var actions = widget.actions;
        if (widget.hasCheckedRows()) {
            for (var j = actions.length; j--;) {
                if (actions[j].showAsAction === 'FAB_checked_rows') {
                    return actions[j];
                }
            }
        } else {
            for (var i = actions.length; i--;) {
                if (actions[i].showAsAction === 'FAB'||
                    actions[i].showAsAction.indexOf('FAB_'+currentWidgetMode) !== -1) {
                    return actions[i];
                }
            }
        }
    };

    //add a watch to check if the active widget has changed.
    //TODO: find a better solution that DOESN'T use ANGULARJS

    var changeFABVisibility = function() {
        if(!templateManager.container){
            return;
        }
        var widget = templateManager.container.getActiveWidget();
        var action = findActionToDisplay(widget);
        // /*if the*/ TODO: add comment here
        FABElement.className = 'fab-root';
        if (action) {
            FABElement.code = action.code;
            FABElement.setIcon(action.ruiIcon || defaultIconName);
            FABElement.setAction(action);
            FABElement.zhWidgetMode = action.showAsAction;
            FABElement.className += ' active';
            if(action.class){
                FABElement.className += " " + action.class;
            }
        }
    };

    $rootScope.$watch(function() {
        return templateManager.container && templateManager.container.getActiveWidget().hasCheckedRows();
        // else , will return undefined
    }, function(hasCheckedRows) {
        changeFABVisibility();
    });

    $rootScope.$watch(function() {
        return templateManager.container && templateManager.container.getActiveWidget();
        // else , will return undefined
    }, function(hasChanged) {
        changeFABVisibility();
    });

    $rootScope.$watch(function() {
        return templateManager.container && templateManager.container.getActiveWidget().mode;
        // else , will return undefined
    }, function(hasChanged) {
        changeFABVisibility();
    });

    var attachFABToZhContainer = function() {
        var containerToInjectFAB = document.querySelector('div.container');
        if(containerToInjectFAB){
            containerToInjectFAB.appendChild(FABElement);
            return true;
        } else {
            return false;
        }
    };

    // attach the button to the container, using the attempt and error strategy
    var setIntervalReference = setInterval(function(){
        if(attachFABToZhContainer()){
            clearInterval(setIntervalReference);
        }
    },50);


}

Configuration(function(ContextRegister) {
    ContextRegister.register("ruiFABController", ruiFABController);
});


// FILE: src/assets/js/controllers/ruiSelectController.js
function ruiSelect($scope,$element,$timeout) {
	var realInput = $scope.field;
	var FAKE_INPUT_NAME = "#rui_fake_" + realInput.name;
	
	function switchDataDown(fakefieldname,open){
		var selectContainer  = $(fakefieldname).parent().parent();
		if(open){
			var newplaceholder = selectContainer.find('label').text();
			selectContainer.addClass('rui-fixed-at-top');
		}else{
			selectContainer.removeClass('rui-fixed-at-top');
		}
	}
	
	$scope.initAutoCompSelect = function (){
		$timeout(function (){
			var fieldName = realInput.name;
			$('#'+fieldName).on('click',function(){
				switchDataDown(FAKE_INPUT_NAME,true);
				$(FAKE_INPUT_NAME).focus();
			});
		},300);
	};
	
	$scope.closeDataDown = function(){
		switchDataDown(FAKE_INPUT_NAME,false);
		$(FAKE_INPUT_NAME).blur();
	};
	
	$scope.filterComparising = function(row) {
		var filterOptions = realInput.filterOptions;
		var isVisible = false;
		if ($scope.filterQuery) {
			for (var i = 0; i < filterOptions.length; i++) {
				var rowValue = row[filterOptions[i]].toLowerCase();
				rowValue = rowValue.latinize();
				var filterValue = $scope.filterQuery.toLowerCase();
				filterValue = filterValue.latinize();
				if (rowValue.search(filterValue) !== -1) {
					isVisible = true;
					break;
				}
			}
		} else {
			isVisible = true;
		}
		return isVisible;
	};
	
	$scope.chooseAutoComplete = function(row) {
		var outData = realInput.outData;
		if (outData && Object.keys(outData).length) {
			if (outData && angular.isObject(outData)) {
				for(var attr in outData) {
					realInput.widget.currentRow[outData[attr]] = row[attr];
				}
			}
		} else {
			var descriptionField = realInput.descriptionField || realInput.name;
			realInput.widget.currentRow[realInput.name] = row[descriptionField];
			if (realInput.valueField) {
				realInput.widget.currentRow[realInput.valueField] = row[realInput.valueField];
			}
		}
		$scope.closeDataDown();
	};
	
	$scope.cleanField = function() {
		$scope.filterQuery = "";
		$element.find(FAKE_INPUT_NAME).val($scope.filterQuery);
		$(FAKE_INPUT_NAME).focus();
	};
	
	$scope.cleanSelection = function() {
		realInput.widget.currentRow[realInput.name] = '';
	};
}

// FILE: src/assets/js/controllers/ruiTextController.js
function ruiTextController($scope, ScreenService) {
	var defaultFormat = {
		time: "HH:mm"
	};
	var types = {
		time: "time"
	};

	$scope.initField = function() {
		console.log($('#' + $scope.field.name));
	};

	$scope.getFieldVal = function(fieldName) {
		return $("#" + fieldName).val();
	};
	$scope.getLabel = function(label, validations) {
		label = ScreenService.i18n(label);
		if (validations && Object.keys(validations).length && validations.required) {
			label += ' *';
		}
		return label;
	};
	$scope.formatValue = function(filedRow, field) {
		var type = field.type;
		if (type === types.time && field.timeFormat) {
			field.widget.currentRow[field.name] = moment(filedRow, defaultFormat.date).format(field.timeFormat);
		}
	};
}

// FILE: src/assets/js/controllers/ruiTimeController.js
function ruiTimeController($scope, $element, ZhUtil, ScreenService) {

    $scope.ZhUtil = ZhUtil;

    $scope.init = function() {
        if (window.orientation)
            detectOrientationAndRotate();
        var fieldValue;
        if ($scope.fieldRow)
            fieldValue = $scope.fieldRow;
        else
            fieldValue = $scope.ZhUtil.evaluate($scope.field.defaultValue);
        if (fieldValue === undefined) {
            fieldValue = '';
        }
        else {
            var tempoArray = fieldValue.split(':'); //[0] ==  minuto [1] == hora
            clock.time.hora = parseInt(tempoArray[0]);
            clock.time.minuto = parseInt(tempoArray[1]);
            clock.hourView.children[12].style.opacity = '0';
            clock.hourView.children[clock.time.hora].style.opacity = '1';
            clock.minuteView.children[0].style.opacity = '0';
            clock.minuteView.children[clock.time.minuto].style.opacity = '1';
            timeInput.classList.add('rui-has-value');
            clock.hourWasSelected = true;
            toggleHourMode();
        }
        timeInput.children[1].textContent = fieldValue;
        $scope.fieldRow = fieldValue;
        if ($scope.field.hideOverlay) {
            overlay.style.opacity = '0';
        }
        if($scope.field.readOnly){
            timeInput.classList.add('disabled');
        }
    };

    var container = $element.get()[0];
    var clock = container.querySelector('#clock-container');
    var overlay = container.querySelector('#overlay');
    var pointerContainer = container.querySelector('#pointer-container'); /*clock.children*/
    var touchArea = container.querySelector('#touch-area');
    var timeInput = container.querySelector('#time-input');

    /* define a label */
    var label = ScreenService.i18n($scope.field.label);
    if ($scope.field.validations && $scope.field.validations.required) {
        label += " *";
    }
    timeInput.children[0].textContent = label;


    clock.minuteDisplay = container.querySelector('#red-minute');
    clock.hourInnerDisplay = container.querySelector('#red-hours-inner');
    clock.hourOutterDisplay = container.querySelector('#red-hours-outter');
    clock.pointer = pointerContainer;
    clock.HOUR_MODE = 1;
    clock.MINUTE_MODE = 2;
    clock.mode = clock.HOUR_MODE;
    clock.time = {
        hora: 12,
        minuto: 0
    };
    $scope.field.time = clock.time;
    clock.detalhesDoCirculoMinuto = {};
    clock.detalhesDoCirculoHora = {};
    clock.activeMinute = 0; /* Posicao do vetor que indica o minuto selecionado no relogio  */
    clock.activeHour = 0; /* Posicao do vetor que indica a hora selecionado no relogio  */
    clock.activeHourArrayPosition = 2; /*contém a posicao (no array de filhos de relogio, que corresponde ...)*/
    clock.hourWasSelected = false;
    clock.hourView = container.querySelector('#hour-display-container');
    clock.minuteView = container.querySelector('#minute-display-container');
    clock.okButton = container.querySelector('#ok');
    clock.cancelButton = container.querySelector('#cancel');

    clock.validate = function() {
        $scope.field.validate($scope.fieldRow);
        $scope.$apply();
    };

    var clickHander = function(e) {
        var rotationDetails = '';
        if (clock.mode == clock.MINUTE_MODE) {
            var previousMinute = clock.time.minuto;
            var minuto = getMinute(e.clientX, e.clientY, clock.detalhesDoCirculoMinuto);
            if (minuto === 60) {
                minuto = 0;
            }
            clock.time.minuto = minuto;
            rotationDetails = "rotate3d(0,0,1," + (minuto * 6) + "deg)";
            requestAnimationFrame(function() {

                pointerContainer.style.transform = rotationDetails;
                clock.minuteView.children[minuto].style.opacity = '1';
                if (previousMinute !== minuto) {
                    clock.minuteView.children[previousMinute].style.opacity = '0';
                    clock.minuteDisplay.children[clock.activeMinute].children[0].style.color = 'black';
                    if (Number.isInteger(minuto / 5)) {
                        clock.minuteDisplay.children[minuto / 5].children[0].style.color = 'white';
                        clock.activeMinute = minuto / 5;
                    }
                }
            });
        }
        else /* clock.mode = clock.HOUR_MODE*/ {
            /* se o clique for dentro do circulo interno ...*/
            var previousActiveHourArrayPosition = clock.activeHourArrayPosition;
            var hora = getHour(e.clientX, e.clientY, clock.detalhesDoCirculoHora);
            var rawHora = ((hora === 12) ? 0 : hora);
            rotationDetails = "rotate3d(0,0,1," + (hora * 30) + "deg)";
            var previousHour = clock.time.hora;
            /* se o clique for dentro do circulo interno ...*/
            if (isWithinCircumnference(clock.detalhesDoCirculoHora, {
                    x: e.clientX,
                    y: e.clientY
                })) {
                switchToInnerPointer();
                hora = ((hora) ? hora : 12);
                clock.time.hora = hora;
                clock.activeHourArrayPosition = 2;
            }
            else {
                switchToOutterPointer();
                hora = ((hora === 12 || hora === 0) ? 0 : hora + 12);
                clock.time.hora = hora;
                clock.activeHourArrayPosition = 1;
            }
            requestAnimationFrame(function() {
                pointerContainer.style.transform = rotationDetails;
                clock.hourView.children[hora].style.opacity = '1';
                if (previousHour !== hora) {
                    clock.hourView.children[previousHour].style.opacity = '0';
                    clock.children[previousActiveHourArrayPosition].
                    children[clock.activeHour].children[0].style.color = 'black';
                    //if(Number.isInteger(rawHora/5)){
                    clock.children[clock.activeHourArrayPosition].
                    children[rawHora].children[0].style.color = 'white';
                    clock.activeHour = rawHora;
                    //}
                }
            });
        }
    };




    /* se for uma tela de toque, usa o evento padrão para telas de toque*/
    if (('ontouchmove' in window) ||
        (window.navigator.MaxTouchPoints > 0) ||
        (window.navigator.msMaxTouchPoints > 0)) {

        var touchHandler = function(e) {
            //console.log('%cTOCADO','background-color: green; color: yellow');
            //e.preventDefault();
            var touch = {};
            var rotationDetails = '';
            if (clock.mode == clock.MINUTE_MODE) {
                touch = e.targetTouches[0];
                var previousMinute = clock.time.minuto;
                var minuto = getMinute(touch.pageX, touch.pageY, clock.detalhesDoCirculoMinuto);
                if (minuto === 60) {
                    minuto = 0;
                }
                clock.time.minuto = minuto;
                rotationDetails = "rotate3d(0,0,1," + (minuto * 6) + "deg)";
                requestAnimationFrame(function() {
                    pointerContainer.style.transform = rotationDetails;
                    clock.minuteView.children[minuto].style.opacity = '1';
                    if (previousMinute !== minuto) {
                        clock.minuteView.children[previousMinute].style.opacity = '0';
                        clock.minuteDisplay.children[clock.activeMinute].children[0].style.color = 'black';
                        if (Number.isInteger(minuto / 5)) {
                            clock.minuteDisplay.children[minuto / 5].children[0].style.color = 'white';
                            clock.activeMinute = minuto / 5;
                        }
                    }
                });
            }
            else /* clock.mode = clock.HOUR_MODE*/ {
                var previousActiveHourArrayPosition = clock.activeHourArrayPosition;
                touch = e.targetTouches[0];
                var hora = getHour(touch.pageX, touch.pageY, clock.detalhesDoCirculoHora);
                var rawHora = ((hora === 12) ? 0 : hora);
                rotationDetails = "rotate3d(0,0,1," + (hora * 30) + "deg)";
                var previousHour = clock.time.hora;
                /* se o clique for dentro do circulo interno ...*/
                if (isWithinCircumnference(clock.detalhesDoCirculoHora, {
                        x: touch.pageX,
                        y: touch.pageY
                    })) {
                    switchToInnerPointer();
                    hora = ((hora) ? hora : 12);
                    clock.time.hora = hora;
                    clock.activeHourArrayPosition = 2;
                }
                else {
                    switchToOutterPointer();
                    hora = ((hora === 12 || hora === 0) ? 0 : hora + 12);
                    clock.time.hora = hora;
                    clock.activeHourArrayPosition = 1;
                }
                requestAnimationFrame(function() {
                    pointerContainer.style.transform = rotationDetails;
                    clock.hourView.children[hora].style.opacity = '1';
                    if (previousHour !== hora) {
                        clock.hourView.children[previousHour].style.opacity = '0';
                        clock.children[previousActiveHourArrayPosition].
                        children[clock.activeHour].children[0].style.color = 'black';
                        //if(Number.isInteger(rawHora/5)){
                        clock.children[clock.activeHourArrayPosition].
                        children[rawHora].children[0].style.color = 'white';
                        clock.activeHour = rawHora;
                        //}
                    }
                });
            }
        };
        touchArea.ontouchmove = touchHandler;

        touchArea.ontouchend = function(e) {
            touchArea.ontouchmove = null;
            setTimeout(function() {
                touchArea.ontouchmove = touchHandler;
            }, 200);
            toggleMinuteMode();
            return true;
        };

        touchArea.ontouchstart = function(e) {
            //e.preventDefault();
            touchHandler(e);
        };
    }
    else {
        touchArea.onmousedown = function(e) {
            e.preventDefault();
            clickHander(e);
            document.addEventListener("mousemove", clickHander);
        };
        var removeListenerAndSwitchToMinutes = function(e) {
            document.removeEventListener("mousemove", clickHander);
            toggleMinuteMode();
        };
        document.addEventListener('mouseup', removeListenerAndSwitchToMinutes);
    }

    var getMinute = function( /*number*/ x, /*number*/ y,
        /*object {x: pontoX da origem, y: pontoY da origem} */
        detalhesCirculo) {
        var angleRadian = Math.atan2(y - detalhesCirculo.y, x - detalhesCirculo.x);
        var angle = angleRadian * (180 / Math.PI) + 90;
        if (angle < 0) {
            angle += 360;
        }
        return Math.round(angle / 6);
    };


    var getHour = function( /*number*/ x, /*number*/ y,
        /*object {x: pontoX da origem, y: pontoY da origem} */
        detalhesCirculo) {
        var angleRadian = Math.atan2(y - detalhesCirculo.centerPoint.y, x - detalhesCirculo.centerPoint.x);
        var angle = angleRadian * (180 / Math.PI) + 90;
        if (angle < 0) {
            angle += 360;
        }
        return Math.round(angle / 30);
    };


    // document.querySelector('html').appendChild(overlay);
    var modal = container.querySelector('#modalWindow');

    $scope.field.cancel = function() {
        overlay.classList.remove('active');
        modal.classList.remove('active');
        if (clock.hourWasSelected) {
            timeInput.classList.add('rui-has-value');
        }
        timeInput.classList.remove('active');
        toggleHourMode();
        clock.validate();
    };

    $scope.field.open = function(e) {
        if (e) {
            e.preventDefault();
        }
        timeInput.classList.add('active');
        timeInput.classList.remove('rui-has-value');
        overlay.classList.add('active');
        modal.classList.add('active');
        recalculateGeometry();
    };

    $scope.field.confirm = function(e) {
        clock.hourWasSelected = true;
        overlay.classList.remove('active');
        $scope.fieldRow =
            (clock.time.hora < 10 ? "0" + clock.time.hora : clock.time.hora) + ":" +
            (clock.time.minuto < 10 ? "0" + clock.time.minuto : clock.time.minuto);
        timeInput.children[1].textContent = $scope.fieldRow;
        modal.classList.remove('active');
        timeInput.classList.remove('active');
        timeInput.classList.add('rui-has-value');
        //timeInput.classList.remove('active');
        toggleHourMode();
        clock.validate();
    };

    overlay.onclick = $scope.field.cancel;
    timeInput.onclick = $scope.field.open;
    clock.okButton.onclick = $scope.field.confirm;
    clock.cancelButton.onclick = $scope.field.cancel;

    window.addEventListener("orientationchange", function() {
        detectOrientationAndRotate();
    }, false);

    /**
     *  C L O C K  M E T H O D S
     */


    var recalculateGeometry = function() {

        var rect = clock.getBoundingClientRect();
        var raioRelogio = (clock.clientHeight / 2);
        clock.detalhesDoCirculoMinuto = {
            x: rect.left + raioRelogio,
            y: rect.top + raioRelogio
        };
        clock.detalhesDoCirculoHora = {
            centerPoint: {
                x: rect.left + 40 + 90,
                y: rect.top + 40 + 90
            },
            radius: 90
        };
    };

    var toggleMinuteMode = function() {
        if (clock.mode === clock.MINUTE_MODE)
            return;

        clock.minuteDisplay.classList.remove('rui-opacity-hidden');
        clock.hourInnerDisplay.classList.add('rui-opacity-hidden');
        clock.hourOutterDisplay.classList.add('rui-opacity-hidden');
        clock.pointer.classList.remove('rui-inner-pointer');
        clock.mode = clock.MINUTE_MODE;
        clock.minuteView.classList.add('active');
        clock.hourView.classList.remove('active');


        var rotationDetails = "rotate3d(0,0,1," + (clock.time.minuto * 6) + "deg)";
        requestAnimationFrame(function() {
            pointerContainer.style.transform = rotationDetails;
        });
    };

    var toggleHourMode = function() {
        if (clock.mode === clock.HOUR_MODE)
            return;


        // esconde os minutos
        clock.minuteDisplay.classList.add('rui-opacity-hidden');
        // mostra as horas
        clock.hourInnerDisplay.classList.remove('rui-opacity-hidden');
        clock.hourOutterDisplay.classList.remove('rui-opacity-hidden');
        clock.pointer.classList.add('rui-inner-pointer');
        clock.minuteView.classList.remove('active');
        clock.hourView.classList.add('active');
        clock.mode = clock.HOUR_MODE;

        var rotationDetails = "rotate3d(0,0,1," + (clock.time.hora * 30) + "deg)";
        requestAnimationFrame(function() {
            pointerContainer.style.transform = rotationDetails;
            if (clock.time.hora <= 12 && clock.time.hora > 0) {
                clock.pointer.classList.add('rui-inner-pointer');

            }
            else {
                clock.pointer.classList.remove('rui-inner-pointer');
            }
        });
    };

    var switchToInnerPointer = function() {
        clock.pointer.classList.add('rui-inner-pointer');
    };

    var switchToOutterPointer = function() {
        clock.pointer.classList.remove('rui-inner-pointer');
    };

    var distanceBetween = function(pointA, pointB) {
        return (Math.sqrt(Math.pow(pointA.x - pointB.x, 2) + Math.pow(pointA.y - pointB.y, 2)));
    };


    var isWithinCircumnference = function(circleProperties, pointToCheck) {
        return (distanceBetween(circleProperties.centerPoint, pointToCheck) <= circleProperties.radius);
    };

    var detectOrientationAndRotate = function() {
        if (orientation === 0 || orientation === 180) {
            modal.classList.remove('landscape');
        }
        else {
            modal.classList.add('landscape');
        }
        setInterval(recalculateGeometry, 800);
    };

    clock.hourView.onclick = toggleHourMode;
    clock.minuteView.onclick = toggleMinuteMode;


    for (var i = 1; i < 60; i++) {
        var n = clock.minuteView.children[0].cloneNode();
        if (i < 10) {
            i = "0" + i;
        }
        n.textContent = i;
        n.style.opacity = '0';
        clock.minuteView.appendChild(n);
    }

    /* GERA OS MOSTRADORES DE HORA */
    for (var j = 1; j < 24; j++) {
        var node = clock.hourView.children[0].cloneNode();
        if (j < 10) {
            j = "0" + j;
        }
        node.textContent = j;
        node.style.opacity = '0';
        clock.hourView.appendChild(node);
    }
    /* DEFINE A HORA PADRAO COMO 12h*/
    clock.hourView.children[0].style.opacity = '0';
    clock.hourView.children[12].style.opacity = '1';
    /* DEFINE A HORA COMO 'ATIVA' (PARA MOSTRAR A FONTE EM BRANCO)*/
    clock.hourView.classList.add('active');
    /**/

    var clearButton = container.querySelector('#fake-input-clear');
    clearButton.onclick = function(event) {
        event.stopPropagation();
        timeInput.classList.remove('rui-has-value');

        clock.children[clock.activeHourArrayPosition].children[clock.activeHour].children[0].style.color = 'black';

        clock.activeHourArrayPosition = 2;
        clock.activeHour = 11;
        clock.children[clock.activeHourArrayPosition].children[0].children[0].style.color = 'white';

        clock.minuteDisplay.children[clock.activeMinute].children[0].style.color = 'black';
        clock.activeMinute = 0;
        clock.minuteDisplay.children[clock.activeMinute].children[0].style.color = 'white';


        clock.hourView.children[clock.time.hora].style.opacity = '0';
        clock.time.hora = 12;
        clock.hourView.children[12].style.opacity = '1';

        clock.minuteView.children[clock.time.minuto].style.opacity = '0';
        clock.time.minuto = 0;
        clock.minuteView.children[0].style.opacity = '1';

        clock.hourWasSelected = false;
        timeInput.children[1].textContent = '';
        $scope.fieldRow = '';
        /*força re-renderização*/
        clock.mode = clock.MINUTE_MODE;
        toggleHourMode();
        /*******/

        clock.validate();

    };
}



// FILE: ./bower_components/red-ui-header/dist/rui-header.js
// FILE: src/assets/js/controllers/ruiHeaderController.js
/**
 * The controller of header template
 * @param {TemplateManager} templateManager - An instance of TemplateManager
 */
Configuration(function(eventEngine) {
	eventEngine.registerEvent("WidgetOnBack", "widgetOnBack");
});

function ruiHeaderController($scope, templateManager, ScreenService) {

	var self = this;

	var showAsActionTypes = {
		always: "always",
		list: "list",
		view: "view",
		edit: "edit",
		checked_rows: "checked_rows",
		never: "never",
		never_list: "never_list",
		never_view: "never_view",
		never_edit: "never_edit",
		never_checked_rows: "never_checked_rows",
		back: "back",
		back_list: "back_list",
		back_view: "back_view",
		back_edit: "back_edit",
		back_checked_rows: "back_checked_rows"
	};
	var searchStatus = {
		opened: "SO",
		closed: "SC",
		notSearching: "NS"
	};


	/**
	 * Retrieves actions conllection length
	 * @param  {String} type - the action type
	 * @return {Integer} Returns the length of actions collection by type
	 */
	this.getActionsLengthByType = function (type) {
		return getActionsByType(type).length;
	};

	/**
	 * Retrieves actions by type
	 * @param  {String} type - the action type
	 * @return {Array} Array of actions by type
	 */
	this.getActionsByType = getActionsByType;
	function getActionsByType(type) {
		return getActions().filter(function (action) {
			return findShowAsAction(type, action) && action.isVisible;
		});
	}

	function findShowAsAction (types, action) {
		var exist = false;
		types = types.split('|');
		var asAction = action.showAsAction.split('|');
		for  (var i = asAction.length - 1; i >= 0 && exist === false; i--) {
			exist = showAsActionTypes[types[0]] == asAction[i] ? true : showAsActionTypes[types[1]] == asAction[i] ? true : false;
		}
		return exist;
	}

	this.getActionsVisibleLength = function(nameArrayActions) {
		return $scope[nameArrayActions].filter(function(action) {
			return action.isVisible;
		}).length;
	};

	/**
	 * Toggle dropdown visibility with fast animation
	 */
	this.toggleHeaderDropdown = function () {
		$scope.$root.headerDropdownOpened = !$scope.$root.headerDropdownOpened;
	};
	window.toggleHeaderDropdown = this.toggleHeaderDropdown;

	/**
	 * Retrieves the container's active widget
	 * @return {Widget} Active widget
	 */
	this.getActiveWidget = getActiveWidget;
	function getActiveWidget() {
		return getContainer().getActiveWidget();
	}

	/**
	 * Retrieves active widget's current row of current container
	 * @return {Object} Current row
	 */
	this.getCurrentRow = function () {
		return getActiveWidget().currentRow;
	};

	/**
	 * Execute selected action and toggle dropdown
	 * @param  {Action} action - selected action
	 * @param  {Function} actionClick - Zeedhi action click event
	 */
	this.selectDropdownOption = function (action, actionClick) {
		actionClick(action, getActiveWidget());
	};

	/**
	 * Attach actions and methods in container
	 */
	this.updateContainer = updateContainer;
	function updateContainer() {
		var container = getContainer();
		updateCurrentWidget(container);
		container.eachWidget(function(widget){
			widget.getHeaderActions = getHeaderActions;
		});
	}


	function updateCurrentWidget (container) {
		var widget = getActiveWidget();
		widget.headerActions = widget.headerActions || [];
		widget.headerActions = factoryActions(container, widget.headerActions);
		widget.searchCriteria =  widget.searchCriteria || {};
		updateSearching(widget);
		updateHeaderShadow(widget);
		updateHeaderForTWWI(widget, searchStatus.notSearching);
	}
	function updateHeaderForTWWI(widget,search){
		var fatherTemp = widget.getParent().template;
		var srcTemp = 'rui-tabs#widget/tabbedWidgetWithImage.html';
		var ruiHeader = $('.rui-header');
		var ruiContainer = $('.container[zh-fixed]');
		if(search ===  searchStatus.notSearching){
			if(fatherTemp === srcTemp){
				ruiHeader.addClass('header-over-tabs');
				ruiContainer.css('top',0);
			}else{
				ruiHeader.removeClass('header-over-tabs');
				ruiHeader.removeClass('new-bg-header');
				ruiContainer.css('top',ruiHeader.css('height'));
			}
		}else if (search ===  searchStatus.opened && fatherTemp === srcTemp){
			$('.rui-TWWI-Main-img').hide();
			$('.rui-header').removeClass('header-over-tabs');
			$('.rui-header').removeClass('new-bg-header');
			ruiContainer.css('top',ruiHeader.css('height'));
		}
		else if (search ===  searchStatus.closed && fatherTemp === srcTemp){
			$('.rui-TWWI-Main-img').show();
			$('.rui-header').addClass('header-over-tabs');
			$('.rui-header').addClass('new-bg-header');
			ruiContainer.css('top',0);
		}
	}
	function updateHeaderShadow(widget) {
		if (widget.parent && isTabbedTemplate(widget.parent.template)) {
			$('.rui-header').removeClass('searchShadow');
		} else {
			$('.rui-header').addClass('searchShadow');
		}
	}

	function isTabbedTemplate(template) {
		return template.search('tabbedWidget.html') !== -1 || template.search('tabbedWidgetWithImage.html') !== -1;
	}

	function updateSearching(widget) {
		if (widget.showSearch && widget.headerSearchBy) {
			$scope.searchVal = widget.searchCriteria._ALL;
		}

		if (widget.keepHeaderSearchingMode && searchFilled()) {
			$scope.$root.searching = true;
		} else if (searchFilled() && widget.mode == "list") {
			$scope.$root.searching = true;
		} else {
			$scope.$root.searching = false;
		}
	}
	this.getNeverMode = getNeverMode;
	function getNeverMode (){
		var widget = getActiveWidget();
		if (widget.hasCheckedRows()) {
			return "never_checked_rows";
		} else {
			if(widget.mode === "edit") {
				return "never_edit|never";
			} else if(widget.mode === "view") {
				return "never_view|never";
			} else if(widget.mode === "list") {
				return "never_list|never";
			}
		}
	}
	this.getBackMode = getBackMode;
	function getBackMode (){
		var widget = getActiveWidget();
		if(widget.hasCheckedRows()){
			return "back_checked_rows";
		} else {
			if(widget.mode === "edit") {
				return "back_edit|back";
			} else if(widget.mode === "view") {
				return "back_view|back";
			} else if(widget.mode === "list") {
				return "back_list|back";
			}
		}
	}
	this.getCenterMode = getCenterMode;
	function getCenterMode(){
		var widget = getActiveWidget();
		if(widget.hasCheckedRows()){
			return "checked_rows";
		} else {
			if(widget.mode === "edit") {
				return "edit|always";
			} else if(widget.mode === "view") {
				return "view|always";
			} else if(widget.mode === "list") {
				return "list|always";
			}
		}
	}

	/**
	 * Build actions applying mobile and default properties
	 * @param  {Container} container - current container
	 * @param  {Array} actions - array of actions to build
	 * @return {Array} An array of builded actions
	 */
	function factoryActions(container, actions) {
		return actions.map(function (action) {
			action = Util.extend(new Action(), action);
			_extendMobileProperties(action);
			action.container = container;
			action.widget = getActiveWidget();
			return action;
		});
	}

	/**
	 * Set mobile properties if it's a mobile device
	 * @param  {Object} obj - action object
	 */
	function _extendMobileProperties(obj) {
		if (Util.isMobile() && obj.mobile) {
			var mobile = obj.mobile;
			Object.keys(mobile).forEach(function extendMobileProperty(propertyToExtend) {
				if (obj.replaceByMobile || !obj[propertyToExtend] || typeof obj[propertyToExtend] !== 'object') {
					obj[propertyToExtend] = mobile[propertyToExtend];
				} else {
					obj[propertyToExtend] = Util.extend(obj[propertyToExtend], mobile[propertyToExtend]);
				}
			});
		}
	}

	/**
	 * Retrieves header actions of current container
	 * @return {Array} An array of actions
	 */
	function getActions() {
		return getActiveWidget().headerActions || [];
	}

	/**
	 * Retrieves current container
	 * @return {Container} Current container
	 */
	function getContainer() {
		return templateManager.container;
	}

	function widgetOnChangeMode(args) {
		if (getContainer()) {
			var widget = getActiveWidget();
			if (widget.mode != "list") {
				$scope.$root.searching = false;
			}
			updateContainer(getContainer());
		}
	}

	$scope.$watch(function() {
		return getContainer() ? getActiveWidget().mode : null;
	}, widgetOnChangeMode);

	$scope.$watch(function() {
		return getContainer() ? getActiveWidget().hasCheckedRows() : null;
	}, function(args) {
		if (getContainer())
			updateCurrentWidget(getContainer());
	});

	$scope.$watch(function() {
		return getContainer() ? getActiveWidget() : null;
	}, function(args){
		if (getContainer())
			updateCurrentWidget(getContainer());
	});

	$scope.$watch(function() {
		return getContainer();
	}, function(args){
		if (getContainer()){
			updateContainer();
		}
	});
	 
	/**
	 * Change header layout to search mode
	 * @param  {Boolean} type - true: Search mode, false: Normal mode
	 */
	this.toggleHeaderSearch = function(type) {
		var widget = getActiveWidget();
		if(type) {
			$scope.$root.searching = true;
			$('.rui-tabsMobileCont').css('top','-50px');
			$('.rui-TWWI-mainContainer').css('top','-50px');
			setTimeout(function() {
				$('#search-input').attr("placeholder", ScreenService.i18n("Search..."));
				$('#search-input').focus();
				updateHeaderForTWWI(widget, searchStatus.opened);
			}, 150);
		} else {
			if (searchFilled()) {
				clearSearch();
			}
			$('.rui-tabsMobileCont').css('top','0');
			$('.rui-TWWI-mainContainer').css('top','0');
			$('#search-input').attr("placeholder", "");
			$scope.$root.searching = false;
			updateHeaderForTWWI(widget, searchStatus.closed);
		}
	};
	window.toggleHeaderSearch = this.toggleHeaderSearch;

	/**
	 * Update the searchCriteria of active widget
	 */
	this.searchInputChange = searchInputChange;
	function searchInputChange() {
		var inputVal = $scope.searchVal;
		var widget = getActiveWidget();
		var showSearch = widget.showSearch;
		if (showSearch && !widget.headerSearchBy) {
			console.error("ERROR: headerSearchBy property not defined on widget '" + widget.name + "'.");
		} else {
			widget.headerSearchBy = Array.isArray(widget.headerSearchBy) ? widget.headerSearchBy : [widget.headerSearchBy];
			if (!Util.isEmptyOrBlank(inputVal)) {
				widget.searchCriteria._ALL = inputVal;
			} else {
				delete widget.searchCriteria._ALL;
			}

			if ($scope.searchTimeout) {
				clearTimeout($scope.searchTimeout);
			}
			$scope.searchTimeout = setTimeout(setSearchFilter.bind(this), 500);
		}
	}

	/**
	 * Set dataSourceFilter of widget according to searchCriteria
	 */
	function setSearchFilter() {
		var widget = getActiveWidget();

		if (!Object.keys(widget.searchCriteria).length) {
			widget.dataSourceFilter = widget.defaultDataSourceFilter || widget.dataSourceFilter || [];
		} else {
			if (!widget.defaultDataSourceFilter) {
				widget.defaultDataSourceFilter = widget.dataSourceFilter || [];
			}
			var name = widget.headerSearchBy.join("|");
			var operator = "LIKE_ALL";
			var value = '%'+widget.searchCriteria._ALL+'%';
			addFilterCondition(widget, name, operator, value);
		}
		widget.reload();
	}

	/**
	 * Add condition to widget's dataSourceFilter
	 * @param {Widget} widget
	 * @param {String} name
	 * @param {String} operator
	 * @param {String} value
	 */
	function addFilterCondition(widget, name, operator, value) {
		var dataSourceFilter = widget.dataSourceFilter || [];
		dataSourceFilter = dataSourceFilter.filter(function(row) {
			return row.name != name;
		});
		// if (!Util.isUndefinedOrNull(inDataSourceFilter[0])) {
		// 	dataSourceFilter.splice(inDataSourceFilter[0], 1);
		// }
		dataSourceFilter.push({
			"name": name,
			"operator": operator,
			"value": value
		});
		widget.dataSourceFilter = dataSourceFilter;
	}

	/**
	 * Manage visibility of clear search icon
	 * @return {Boolean} Status of widget's searchCriteria property
	 */
	this.searchFilled = searchFilled;
	function searchFilled() {
		var widget = getActiveWidget();
		if (widget.searchCriteria && widget.headerSearchBy) {
			return !Util.isEmptyOrBlank(widget.searchCriteria._ALL);
		} else {
			return false;
		}
	}

	/**
	 * Clear value of search input
	 */
	this.clearSearch = clearSearch;
	function clearSearch() {
		$scope.searchVal = "";
		searchInputChange();
	}

	function getHeaderActions(name) {
		return this.headerActions.filter(function(item) {
			return item.name === name;
		}).shift();
	}

	this.getTitle = function() {
		var container = templateManager.container;
		var widget = getActiveWidget();
		var headerTitle = widget.title || container.label;

		if (widget.showCheckedRowsCounterOnHeader && widget.getCheckedRows().length) {
			headerTitle = widget.getCheckedRows().length;
		}

		return headerTitle;
	};

	this.menuActionVisibility = function() {
		var container = templateManager.container;
		var widget = getActiveWidget();

		if (!container.showMenu || widget.getCheckedRows().length) {
			return false;
		} else {
			if (widget.showMenu === undefined) {
				return true;
			} else {
				return widget.showMenu;
			}
		}
	};
}


// FILE: ./bower_components/red-ui-list/dist/rui-list.js
// FILE: src/assets/js/controllers/FluidListController.js
function FluidListController($scope, $element, ApplicationContext, restEngine) {

	/*
	 * -> fluidList
	 * The widget with this template in the container json
	 *
	 * -> fluidListDOM
	 * The DOM element representing the fluid list template
	 *
	 * -> actionEvents
	 * Array containing the code element from all events with name 'ActionEvent' defined on the jobson.json widget declaration
	 */
	var self = this;
	var fluidListDOM, fluidList;
	this.actionEvents = [];

	/*
	 * Initializes the variables, check if this widget should be visible and calls the createFLuidListElements method.
	 *
	 * returns void
	 */
	this.initializeFluidList = function() {
		fluidList = $scope.widget;
		fluidListDOM = $element.get()[0];

		if (!fluidList.isVisible)
			fluidListDOM.style.visibility = "hidden";

		fluidListDOM.style.columnCount = Math.floor(Math.sqrt(fluidListDOM.clientWidth)/7);

		this.actionEvents = fluidList.events.filter(function(event) {
			return event.name == "ActionEvent";
		});

		pullData().then(createFluidListElements.bind(null, fluidList, fluidListDOM));
	};

	/*
	 * Populates the fluidList.dataSource.data array with data from the backend.
	 * The backend MUST return an object named 'dataset' with an array property named 'data'
	 *
	 * This method orders the data as first element in 0 position, second element in half position, third in 1 position, fourth in half+1 position...
	 * This is due the div columns functionality. Data must be ordered this way for being displayed the first and second element side by side, third and fourth
	 * side by side and so on.
	 *
	 * returns void
	 */
	 var pullData = function() {
		if (!fluidList.dataSource)
			fluidList.dataSource = {};
		if (!fluidList.dataSourceFilter)
			fluidList.dataSourceFilter = [];

		return restEngine.post(fluidList.dataSource.name, {
			"requestType": "FilterData",
			"filter": fluidList.dataSourceFilter
		}).then(function(fluidList, response) {
			var length = response.dataset.data.length;
			for (var i=0, j=0; i<length/2; i++, j+=2) {
				fluidList.dataSource.data[i] = response.dataset.data[j];
				fluidList.dataSource.data[length/2+i] = response.dataset.data[j+1];
			}
			if (length%2 !== 0)
				fluidList.dataSource.data.push(response.dataset.data[length-1]);
			
			// fluidList.dataSource.data = response.dataset.data;
		}.bind(null, fluidList), function(error) {
			console.log("%cFluidList could not retrieve data from given datasource", "background-color: red; color: black;");
		});
	 };

	/*
	 * Iterates over the fluidList.fields array and create a DOM element for each one of them.
	 *
	 * returns void
	 */
	var createFluidListElements = function(/*object*/ fluidList, /*DOM object*/ fluidListDOM) {
		var fluidListElement, fluidListElementTemplate;

		document.querySelector(".container").style.overflowY = "scroll";
		fluidListElementTemplate = fluidListDOM.firstElementChild;

		for (var i=0; i<fluidList.dataSource.data.length; i++) {
			fluidListElement = fluidListElementTemplate.cloneNode(true);

			fluidListElement.classList.add("rui-fluid-list-element");
			fluidListElement.setAttribute("id", fluidList.dataSource.data[i].name);
			fluidListElement.style.height = (fluidList.dataSource.data[i].height * 15 + 100) + "px";
			fluidListElement.lastElementChild.innerHTML = fluidList.dataSource.data[i].label;
			fluidListElement.style.backgroundImage = "url('"+fluidList.dataSource.data[i].image+"')";
			fluidListElement.addEventListener("click", executeActionEvent.bind(self));

			fluidListDOM.appendChild(fluidListElement);
		}

		fluidListDOM.removeChild(fluidListElementTemplate);
	};

	/*
	 * Evaluates the strings in actionEvents.
	 *
	 * returns void
	 */
	var executeActionEvent = function(/*object*/ event) {
		for (var i=0; i<this.actionEvents.length; i++)
			$scope.$eval(
				this.actionEvents[i].code,
				getApplicationContextWithEventArgs(event.currentTarget.getAttribute("id"))
			);
	};

	/*
	 * Adds to ApplicationContext an element called "args" with reference to the event owner.
	 *
	 * returns object
	 */
	var getApplicationContextWithEventArgs = function(/*string*/ eventTargetName) {
		ApplicationContext.args = {
			owner: filterElementsByName(fluidList.dataSource.data, eventTargetName)
		};

		return ApplicationContext;
	};

	/*
	 * Filters the fluidList.dataSource.data array in search of an object with the name passed as parameter.
	 * Returns the first one found.
	 *
	 * returns object
	 */
	var filterElementsByName = function(/*object array*/ array, /*string*/ name) {
		return array.filter(function(element) {
			return element.name == name;
		})[0];
	};
}

// FILE: src/assets/js/controllers/RuiListController.js
/**
 * The controller of including list template
 * @param {ZHPromise} ZHPromise - An instance of ZHPromise.
 */
function RuiListController($scope, $rootScope, ZHPromise, localStorageEngine, templateManager) {

	var previousScroll;
	var container = $("#rui-list-container");
	var self = this;
	var defaultListProperties = {
		showImage: false,
		clearChangedOnReload: true
	};
	$scope.listProperties = $scope.widget.listProperties || defaultListProperties;
	$scope.rows = $scope.widget.dataSource.data;
	$scope.widget.itemsPerPage = $scope.widget.itemsPerPage || $scope.widget.dataSource.itemsPerPage;
    $scope.loadedAllData = false;
    $scope.loadingData = false;
	$scope.showLoadMore = true;
	var overlay = document.querySelector(".rui-list-overlay");

	container.nonbounce();

	this.getContainer = function() {
		return templateManager.container;
	};

	$scope.filterTitle = function (item) {
		if (item.isVisible && item.showOnList && item.showAsTitle) {
			return item;
		}
	};

	$scope.filterSubtitle = function (item) {
		if (item.isVisible && item.showOnList && !item.showAsTitle) {
			return item;
		}
	};

	$scope.noMatchVisibility = function() {
		var widget = self.getActiveWidget();
		return Util.isUndefinedOrNull(widget.searchCriteria) || !Object.keys(widget.searchCriteria).length;
	};

	$scope.searchNoMatchVisibility = function() {
		var widget = self.getActiveWidget();
		return !Util.isUndefinedOrNull(widget.searchCriteria) && Object.keys(widget.searchCriteria).length;
	};

	this.onLongTapRow = function (row) {
		var dataArray = $scope.widget.dataSource.data.filter(function(row) {
			return row.dropdownOpened;
		});
		if (!row.__isSelected && $scope.widget.allowSelectOnLongTap && dataArray.length === 0) {
			var widget = self.getActiveWidget();
			for(var i = 0; i< dataArray.length; i++) {
				dataArray[i].dropdownOpened = false;
				document.getElementById('dropdown_' + dataArray[i].$$hashKey).parentElement.classList.remove('list-dropdown-opened');
			}
			$scope.widget.selectAll([row]);
			if (window.navigator.vibrate) {
				window.navigator.vibrate(25);
			}
		}
	};

	this.closeDropdown = function(row) {
		event.preventDefault();
		event.stopPropagation();
		var dataArray = $scope.widget.dataSource.data.filter(function(row) {
			return row.dropdownOpened;
		});
		if(dataArray.length > 0) {
			for(var i = 0; i < dataArray.length; i++) {
				dataArray[i].dropdownOpened = false;
				document.getElementById('dropdown_'+dataArray[i].$$hashKey).parentElement.classList.remove('list-dropdown-opened');
			}
		} else {
			$scope.widget.setCurrentRow(row);
		}
		overlay.style.display = "none";
	};

	this.getCheckboxIcon = function(row) {
		if (row.__isSelected) {
			return "checkbox";
		} else {
			return "checkboxoutlineblank";
		}
	};

	this.filteredRows = function() {
		var widget = $scope.widget;
		var searchCriteria = widget.searchCriteria || {};
		var data  = widget.dataSource.data || [];

		function filterComparising(row) {
			var isVisible = false;
			if (Util.objectHasAnyValue(searchCriteria)) {
				Object.keys(searchCriteria).forEach(function(filter) {
					if (!isVisible) {
						if (filter === "_ALL") {
							var fields;
							if (widget.headerSearchBy) {
								widget.headerSearchBy = Array.isArray(widget.headerSearchBy) ? widget.headerSearchBy : [widget.headerSearchBy];
								fields = widget.headerSearchBy;
							} else {
								fields = getVisibleFields();
							}
							fields.forEach(function(field) {
								if (!isVisible) {
									isVisible = compareValues(row[field], searchCriteria[filter], isVisible);
								}
							});
						} else {
							isVisible = compareValues(row[filter], searchCriteria[filter], isVisible);
						}
					}
				});
			} else {
				isVisible = true;
			}
			return isVisible;
		}

		function getVisibleFields() {
			return widget.fields.map(function(field) {
				if (field.isVisible && field.showOnList) {
					return field.name;
				}
			});
		}

		function compareValues(rowValue, filterValue) {
			if (!Util.isUndefinedOrNull(rowValue)) {
				rowValue = rowValue.toLowerCase();
				rowValue = rowValue.latinize();
				filterValue = filterValue.toLowerCase();
				filterValue = filterValue.latinize();
				if (rowValue.search(filterValue) !== -1) {
					return true;
				}
			}
			return false;
		}
		return data.filter(filterComparising);
	};

	this.toggleImageCheckbox = function(row) {
		var widget = self.getActiveWidget();
		if (widget.listProperties.showCheckbox) {
			this.toggle(row);
		}
	};


	this.openRow = function(row, widget) {
		if (row.__group) return;

		if ((widget.showCheckbox && widget.dataSource.getCheckedRows().length) || widget.allowMultiSelect || widget.hasCheckedRows()) {
			self.toggle(row);
		} else {
			widget.setCurrentRow(row);
		}
		widget.dataSource.updateCheckedRows();
	};

	this.getActiveWidget = function() {
		return self.getContainer().getActiveWidget();
	};

	this.getClosedStatus = function(row) {
		var status = false;
		if (row && row.__groupHeaders && row.__groupHeaders.length > 0) {
			status = row.__groupHeaders.some(function(group) {
				return group.__groupOpened === false;
			});
		}
		return status;
	};

	this.toggle = function(row) {
		var widget = $scope.widget;
		widget.selectedRow = row;

		var event;
		if (widget.beforeSelectRow && !row.__isSelected) {
			event = widget.beforeSelectRow();
		} else if (widget.beforeUnselectRow && row.__isSelected) {
			event = widget.beforeUnselectRow();
		}
		if (!event || !event.isDefaultPrevented()) {
			var index = getIndex(row);
			if (index === -1) {
				row.__isSelected = true;
				if (!$scope._currentPopup) $rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
			} else {
				delete row.__isSelected;
				widget.dataSource.removeCheckedRows(row);
			}
			if (!$scope._currentPopup && !widget.dataSource.checkedRows.length) {
				$rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
			}

			widget.updateCheckedRows();

			if (widget.afterSelectRow && row.__isSelected) {
				widget.afterSelectRow();
			} else if (widget.afterUnselectRow && !row.__isSelected) {
				widget.afterUnselectRow();
			}
		}
	};

	function getIndex(row) {
		return $scope.widget.dataSource.checkedRows.indexOf(row);
	}

	this.toggleDropdown = function(row, e) {
		if (!$scope.widget.hasCheckedRows()) {
			var dataArray = $scope.widget.dataSource.data.filter(function(row) {
				return row.dropdownOpened === true;
			});

			if(dataArray.length < 1) {
				var htmlElementTarget = e.target;
				var dropdownContent = htmlElementTarget.parentElement.parentElement.parentElement.lastElementChild.firstElementChild;
				var dropdown = htmlElementTarget.parentElement.parentElement.parentElement.lastElementChild;
				var availSpace = screen.availHeight - dropdownContent.getBoundingClientRect().top - (screen.availHeight/15);
				if (availSpace > dropdownContent.offsetHeight) {
					dropdownContent.classList.remove('list-dropdown-last');
				} else {
					dropdownContent.classList.add('list-dropdown-last');
				}
				if(!($scope.widget.hasCheckedRows())) {
					row.dropdownOpened = !row.dropdownOpened;
					if(row.dropdownOpened) {
						dropdown.classList.add('list-dropdown-opened');
					} else {
						dropdown.classList.remove('list-dropdown-opened');
					}
				}
				overlay.style.display = "block";
			} else {
				for(var i=0; i<$scope.widget.dataSource.data.length; i++) {
					$scope.widget.dataSource.data[i].dropdownOpened=false;
					document.getElementById('dropdown_'+i).parentElement.classList.remove('list-dropdown-opened');
				}
			}
			e.preventDefault();
			e.stopPropagation();
		} else {
			e.stopPropagation();
			self.toggle(row);
		}
	};

	this.onActionClick = function(action, actionClick, row) {
		if(!($scope.widget.hasCheckedRows())) {
			var activeWidget = self.getActiveWidget();
			actionClick(action, activeWidget, row);
			event.preventDefault();
			event.stopPropagation();
		} else {
			self.toggle(row);
		}
		self.closeDropdown(row);
	};

   	/**
     * Provide endless scroll on list container
     */
    this.infiniteScroll = function() {
        var scrollHeight = container[0].scrollHeight;
        var scrollPosition = container.height() + container.scrollTop();
        previousScroll = previousScroll || scrollPosition;
        if (((scrollHeight - scrollPosition) / scrollHeight < 0.030) && previousScroll < scrollPosition && !$scope.loadingData && !$scope.loadedAllData) {
            this.loadData(true);
        }
        previousScroll = scrollPosition;
    };

    /**
     * Retrieve content from list dataSource
     * @param  {Boolean} apply - Scope apply selector
     */
    this.loadData = function(apply) {
        $scope.loadingData = true;
    	$('.default-window>.zh-background-loading').addClass("list-hide-loader-overlay");
    	$('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "none");
        if (apply) {
            $scope.$apply();
        }
        var widget = $scope.widget;
        this.nextPage(widget).then(function(callback) {
        	var data;
            if (callback && callback.dataset) {
                Object.keys(callback.dataset).forEach(function (datasetName) {
                    data = callback.dataset[datasetName];
                });
            } else {
                data = callback || [];
            }
            $scope.loadedAllData = data.length < widget.itemsPerPage;
            this.loadMoreVisibility();
            $scope.loadingData = false;
            $('.default-window>.zh-background-loading').removeClass("list-hide-loader-overlay");
	        $('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "inline-block");
        }.bind(this));
    };

    /**
     * Manage load more button visibility
     */
    this.loadMoreVisibility = function() {
    	var listContainer = $('#rui-list-container')[0];
    	if (listContainer) {
	        var scrollHeight = listContainer.scrollHeight + $('.lf-navbar').height();
	        var windowHeight = $(window).height();
	        if (scrollHeight > windowHeight) {
	            $scope.showLoadMore = false;
	        } else {
	            $scope.showLoadMore = true;
	        }

    	}
    };

    /**
     * Retrieve next page of widget's datasource
     * @param  {Widget} widget
     * @return {ZHPromise} Load function callback
     */
    this.nextPage = function(widget) {
        if (widget.dataSource) {
            return widget.dataSource.load(widget.getDataSourceFilter(), widget.dataSource.getNextPage(), widget.itemsPerPage);
        } else {
            return ZHPromise.when();
        }
    };

    this.getRowImage = function(row) {
    	var title = "";
    	if ($scope.listProperties.imageTitleField && row[$scope.listProperties.imageTitleField]) {
    		title = row[$scope.listProperties.imageTitleField].toString().substr(0,1);
    	}
    	return title;
    };

    this.isActionVisible = function(row, action) {
    	var isVisible = action.condition ? eval(action.condition) : action.isVisible; // jshint ignore:line
    	return isVisible;
    };

    $scope.filterRowActions = function(action) {
    	if (action.showAsAction.indexOf("|") === -1) {
    		return action.showAsAction === "row";
    	} else {
    		var showAsAction = action.showAsAction.split("|");
    		var isVisible = false;
    		for (var i = 0; i < showAsAction.length; i++) {
    			if (showAsAction[i] === "row") {
    				isVisible = true;
    				break;
    			}
    		}
    		return isVisible;
    	}
    };
	/**
	 * Overwrite default reload action of widget
	 * @param {Object[]} additionalFilters - (Optional) Filters to be applied on reload
	 * @param {Boolean} shouldRefreshCurrentRow - (Optional) True if should refresh current row
	 */
	if (!$scope.widget.defaultReload) {
		$scope.widget.defaultReload = angular.copy($scope.widget.reload);
	    $scope.widget.reload = function(additionalFilters, shouldRefreshCurrentRow) {
	    	$scope.loadingData = true;
	    	$('.default-window>.zh-background-loading').addClass("list-hide-loader-overlay");
    		$('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "none");
	        if ($scope.listProperties.clearChangedOnReload) {
	        	$scope.widget.dataSource.clearChangedRows();
	        }
	        return $scope.widget.defaultReload(additionalFilters, shouldRefreshCurrentRow).then(function() {
	            container.scrollTop(0);
	            $scope.loadedAllData = false;
	            $scope.loadingData = false;
	            $('.default-window>.zh-background-loading').removeClass("list-hide-loader-overlay");
	        	$('.zh-background-loading>.zh-position-loading>.zh-loading-icon').css("display", "inline-block");
	            $scope.widget.dataSource.updateChangedRows();
	        }.bind(this));
	    };
	}

	if (!$scope.widget.dataSource.updateChangedRows) {
		var dataSource = $scope.widget.dataSource;
		var ignoreKeys = ['__rightActions', '__leftActions'];

		dataSource._changedRows = [];
		dataSource._changedRowsCache = {};

		dataSource.getHashKey = function(row) {
			var VO = this.getVO();
			var hash = "";
			if (VO && VO.localKey) {
				hash = VO.localKey.reduce(function(hash, key) {
					return row[key] ? hash + "_" + row[key].toString() : hash;
				}, "");
			}
			return hash;
		};

		dataSource.addChangedRows = function (rows) {
			rows = Util.parseToArray(rows);
			rows.forEach(function(row) {
				if (this._changedRows.indexOf(row) === -1) {
					this._changedRows.push(row);
				}
				var hash = this.getHashKey(row);
				if (hash) {
					this._changedRowsCache[hash] = Util.clone(row, ignoreKeys);
				}
			}.bind(this));
		};

		dataSource.clearChangedRows = function() {
			this._changedRows = [];
			this._changedRowsCache = {};
		};

		dataSource.getChangedRows = function() {
			var changedRows = this._changedRows;
			if (this.name) {
				var changedRowsHash = changedRows.map(function(row) {
					return this.getHashKey(row);
				}.bind(this));
				changedRows = changedRows.concat(Object.keys(this._changedRowsCache).filter(function(row) {
					return changedRowsHash.indexOf(row) === -1;
				}.bind(this)).map(function(hash) {
					return this._changedRowsCache[hash];
				}.bind(this)));
			}
			return changedRows;
		};

		dataSource.updateChangedRows = function() {
			if (this.data) {
				this._changedRows = this.data.filter(function(row) {
					var hash = this.getHashKey(row);
					if (this._changedRowsCache.hasOwnProperty(hash)) {
						for (var i in this._changedRowsCache[hash]) {
							row[i] = this._changedRowsCache[hash][i];
						}
					}
					return row.changed;
				}.bind(this));
			} else {
				this.clearChangedRows();
			}
			if (templateManager.container) {
				templateManager.container.updateCheckedRows();
			}
		};
	}

	if (!$scope.widget.cancel) {
		$scope.widget.cancel = this.cancel;
	}

    container.on("scroll", this.infiniteScroll.bind(this));
    $(window).on("resize", this.loadMoreVisibility);
}


// FILE: ./bower_components/red-ui-tabs/dist/rui-tabs.js
// FILE: src/assets/js/controllers/ruiTWWIController.js
function ruiTWWIController($scope, $rootScope, $element, eventEngine, $timeout, ZHPromise, ScreenService){
	var self = this;
	var WIDGET_ON_ACTIVATE_EVENT_NAME = 'WidgetOnActivate';
	$scope.tabbedWidget = $scope.$parent.widget;
	$scope.widgets = $scope.tabbedWidget.widgets;
	var tabs = getTabsList($scope.tabbedWidget);

	$scope.getmainImage = function() {
		if($scope.currentWidget){
			var curWidget = $scope.currentWidget;
			var curWImg = curWidget.mainImage;
			if(!curWImg){
				curWImg = $scope.tabbedWidget.mainImage;
			}
			return curWImg;
		}
		return;
	};

	$scope.setDefaultTab = function(defaultTab) {
		return ZHPromise.when().then(function() {
			var tabbedWidget = $scope.tabbedWidget;
			if (!defaultTab) {
				if (tabbedWidget.widgets.length) {
					defaultTab = tabbedWidget.widgets[0];
				}
			} else if (Util.isString(defaultTab)) {
				defaultTab = tabbedWidget.container.getWidget(defaultTab);
			}
			if (defaultTab) {
				tabbedWidget.currentWidget = defaultTab;
			}
			if (tabbedWidget.currentWidget) {
				tabbedWidget.currentWidget.activate();
				$timeout(function(){
					setSizes(tabs);
				}, 300);
			}
			$scope.currentWidget = tabbedWidget.currentWidget;
		});
	};
	var onActivateBoundEventIndex = eventEngine.bindEvent($scope.widget, WIDGET_ON_ACTIVATE_EVENT_NAME, onWidgetActivate);
	$scope.$on('$destroy', function() {
		eventEngine.unbindEvent($scope.widget, WIDGET_ON_ACTIVATE_EVENT_NAME, onActivateBoundEventIndex);
	});
	function setSizes (tabs){
		var tabWithScroll = $element.find('.nonbounce');
		var tabContainer = $element.find('.rui-TWWI-tabs-header-container');
		var marker = $element.find('.rui-TWWI-active-tab-marker');
		var scrWidth = parseInt($(window).width());
		var totalWidth;
		if(scrWidth > 990){
			totalWidth = 120;
		}else{
			if (tabs.length >2){
				totalWidth = parseInt(tabContainer.css('width'))/3;
			}
			else if (tabs.length === 2){
				totalWidth = parseInt(tabContainer.css('width'))/2;
			}else{
				totalWidth = '100%';
			}
		}
		tabs.forEach(function(tabSG){
			var tabName = tabSG.name;
			$('#header_tab_'+tabName).css('width',totalWidth);
		});
		marker.css('width',totalWidth);
		var actvW = $scope.tabbedWidget.currentWidget;
		$timeout(function(){
			fixPosition(actvW);
		}, 150);
	}
	$(window).resize(function(){
		setSizes (tabs);
	});
	function getTabsList(tabbedWidget) {
		var tabs = tabbedWidget.widgets.slice(0);
		if (tabbedWidget.template.indexOf('tabbedWidget') === -1) {
			tabs.push(tabbedWidget);
			$timeout(function(){
				setSizes(tabs);
			}, 300);
		}
		return tabs;
	}
	function onWidgetActivate() {
		if ($scope.widget.currentWidget) {
			$scope.widget.currentWidget.activate();
		}
	}
	function doTheMoveAnimation(choosenTab){
		var tabsElement = $element.find('.rui-TWWI-tabs-header');
		var tabbedContainer = $element.find('.rui-TWWI-tabbedContainer');
		var choosenElement = $element.find('#header_tab_'+choosenTab.name);
		var marker = $element.find('.rui-TWWI-active-tab-marker');
		tabsElement.removeClass('rui-activeTab');
		choosenElement.addClass('rui-activeTab');
		$(tabbedContainer).animate({scrollLeft: $(choosenElement).position().left}, 200);
		marker.css('left',$(choosenElement).position().left);
		$('.rui-TWWI-mainContainer').css('top','0');
	}
	function fixPosition (choosenTab){
		var tabsElement = $element.find('.rui-TWWI-tabs-header');
		var tabbedContainer = $element.find('.rui-TWWI-tabbedContainer');
		var choosenElement = $element.find('#header_tab_'+choosenTab.name);
		var marker = $element.find('.rui-TWWI-active-tab-marker');
		tabsElement.removeClass('rui-activeTab');
		choosenElement.addClass('rui-activeTab');
		$(tabbedContainer).animate({scrollLeft: $(choosenElement).position().left}, 200);
		marker.css('left',$(choosenElement).position().left);
	}
	$scope.onClickTab = function(tabbedWidget, childWidget) {
		if(!$rootScope.searching){
			var tabClickPromise;
			if (childWidget.isEnabled()) {
				tabClickPromise = closeGreaterSwipes(tabbedWidget, childWidget);
				tabbedWidget.currentWidget = childWidget;
				childWidget.activate();
				doTheMoveAnimation(childWidget);
			}
			return tabClickPromise;
		}
	};
	function closeGreaterSwipes(tabbedWidget, childWidget) {
		var closeSwipePromise;
		if (tabbedWidget.currentWidget !== childWidget) {
			var levelToClose = ScreenService.getSwipeLevel(childWidget) + 1;
			if (ScreenService.isSwipeOpen(levelToClose)) {
				closeSwipePromise = ScreenService.closeSwipe(levelToClose).then(function() {
					childWidget.activate();
				});
			}
		}
		return closeSwipePromise;
	}
	var updateTabbed = function(widget) {
		if (widget && widget.onEnterTab) {
			$scope.currentWidget = widget;
			widget.onEnterTab();
		}
	};
	$scope.$parent.$watch('widget.currentWidget', updateTabbed);
	function getNextTab(nextIndex, direction) {
		var nextTab = null;
		while(tabs[nextIndex] && !tabs[nextIndex].enabled) {
			nextIndex += direction;
		}
		if (nextIndex >= 0 && nextIndex < tabs.length) {
			nextTab = tabs[nextIndex];
		}
		return nextTab;
	}
	$scope.scrollSwipe = function(tab, direction){
		var currentIndex = $scope.widgets.indexOf(tab);
		var nextIndex = currentIndex + direction;
		var nextTab = getNextTab(nextIndex, direction);
		if(nextTab){
			if (nextTab !== null) {
				$scope.onClickTab($scope.$parent.widget,nextTab);
			}
		}
	};
	var topOld = 0;
	$('.rui-TWWI-mainContainer').on('scroll', changeBackgroundOpacity);

	function changeBackgroundOpacity() {
		var goingDown;
		var topOffSet = parseFloat($(this).scrollTop());
		var tabsHeader = $element.find('.rui-TWWI-tabbedContainer');
		var tabsKeepHeader = $element.find('.rui-TWWI-keep-header-size');
		var ruiHeader = $('.rui-header');

		var imgDivSrc = $($element.find('.rui-TWWI-Main-src'));
		var imgop = 1-(topOffSet/160);

		if(topOffSet > topOld){
			goingDown = true;
			if(imgop > 1 ){
				imgop = 1;
			}
		}
		else{
			goingDown = false;
			if(imgop < 0 ){
				imgop = 0;
			}
		}
		topOld = topOffSet;
		imgDivSrc.css('opacity',imgop);

		if(topOffSet > 148){
			tabsHeader.addClass('fixedTables');
			tabsKeepHeader.addClass('active-fix');
			ruiHeader.addClass('new-bg-header');
		}
		else if(topOffSet <= 148 && !goingDown){
			tabsHeader.removeClass('fixedTables');
			tabsKeepHeader.removeClass('active-fix');
			ruiHeader.removeClass('new-bg-header');
		}
	}
}

// FILE: src/assets/js/controllers/ruiTabbedController.js
function ruiTabbedController($scope, $rootScope, $element, eventEngine, $timeout, ZHPromise, ScreenService){
	var self = this;
	var WIDGET_ON_ACTIVATE_EVENT_NAME = 'WidgetOnActivate';
	$scope.tabbedWidget = $scope.$parent.widget;
	$scope.widgets = $scope.tabbedWidget.widgets;
	var tabs = getTabsList($scope.tabbedWidget);
	$scope.setDefaultTab = function(defaultTab) {
		return ZHPromise.when().then(function() {
			var tabbedWidget = $scope.tabbedWidget;
			if (!defaultTab) {
				if (tabbedWidget.widgets.length) {
					defaultTab = tabbedWidget.widgets[0];
				}
			} else if (Util.isString(defaultTab)) {
				defaultTab = tabbedWidget.container.getWidget(defaultTab);
			}
			if (defaultTab) {
				tabbedWidget.currentWidget = defaultTab;
			}
			if (tabbedWidget.currentWidget) {
				tabbedWidget.currentWidget.activate();
				$timeout(function(){
					setSizes(tabs);	
				}, 300);
			}
			$scope.currentWidget = tabbedWidget.currentWidget;
		});
	};
	var onActivateBoundEventIndex = eventEngine.bindEvent($scope.widget, WIDGET_ON_ACTIVATE_EVENT_NAME, onWidgetActivate);
	$scope.$on('$destroy', function() {
		eventEngine.unbindEvent($scope.widget, WIDGET_ON_ACTIVATE_EVENT_NAME, onActivateBoundEventIndex);
	});
	function setSizes (tabs){
		var tabContainer = $element.find('.rui-TBC-tabs-header-container');
		var marker = $element.find('.rui-active-tab-marker');
		var scrWidth = parseInt($(window).width());
		var totalWidth;
		if(scrWidth > 990){
			totalWidth = 120;
		}else{
			if (tabs.length >2){
				totalWidth = parseInt(tabContainer.css('width'))/3;
			}
			else if (tabs.length === 2){
				totalWidth = parseInt(tabContainer.css('width'))/2;
			}else{
				totalWidth = '100%';
			}
		}
		tabs.forEach(function(tabSG){
			var tabName = tabSG.name;
			$('#header_tab_'+tabName).css('width',totalWidth);
		});
		marker.css('width',totalWidth);
		var actvW = $scope.tabbedWidget.currentWidget;
		$timeout(function(){
			fixPosition(actvW);
		}, 150);
	}
	$(window).resize(function(){
		setSizes (tabs);
	});
	function getTabsList(tabbedWidget) {
		var tabs = tabbedWidget.widgets.slice(0);
		if (tabbedWidget.template.indexOf('tabbedWidget') === -1) {
			tabs.push(tabbedWidget);
			$timeout(function(){
				setSizes(tabs);	
			}, 300);
		}
		return tabs;
	}
	function onWidgetActivate() {
		if ($scope.widget.currentWidget) {
			$scope.widget.currentWidget.activate();
		}
	}
	function doTheMoveAnimation(choosenTab){
		var tabsElement = $element.find('.rui-TBC-tabs-header');
		var tabbedContainer = $element.find('.rui-tabbedContainer');
		var choosenElement = $element.find('#header_tab_'+choosenTab.name);
		var marker = $element.find('.rui-active-tab-marker');
		tabsElement.removeClass('rui-TBC-activeTab');
		choosenElement.addClass('rui-TBC-activeTab');
		$(tabbedContainer).animate({scrollLeft: $(choosenElement).position().left}, 200);
		marker.css('left',$(choosenElement).position().left);
		$('.rui-tabsMobileCont').css('top','0');
	}
	function fixPosition (choosenTab){
		var tabsElement = $element.find('.rui-TBC-tabs-header');
		var tabbedContainer = $element.find('.rui-tabbedContainer');
		var choosenElement = $element.find('#header_tab_'+choosenTab.name);
		var marker = $element.find('.rui-active-tab-marker');
		tabsElement.removeClass('rui-TBC-activeTab');
		choosenElement.addClass('rui-TBC-activeTab');
		$(tabbedContainer).animate({scrollLeft: $(choosenElement).position().left}, 200);
		marker.css('left',$(choosenElement).position().left);
	}
	$scope.onClickTab = function(tabbedWidget, childWidget) {
		if(!$rootScope.searching){
			var tabClickPromise;
			if (childWidget.isEnabled()) {
				tabClickPromise = closeGreaterSwipes(tabbedWidget, childWidget);
				tabbedWidget.currentWidget = childWidget;
				childWidget.activate();
				doTheMoveAnimation(childWidget);
			}
			return tabClickPromise;
		}
	};
	function closeGreaterSwipes(tabbedWidget, childWidget) {
		var closeSwipePromise;
		if (tabbedWidget.currentWidget !== childWidget) {
			var levelToClose = ScreenService.getSwipeLevel(childWidget) + 1;
			if (ScreenService.isSwipeOpen(levelToClose)) {
				closeSwipePromise = ScreenService.closeSwipe(levelToClose).then(function() {
					childWidget.activate();
				});
			}
		}
		return closeSwipePromise;
	}
	var updateTabbed = function(widget) {
		if (widget && widget.onEnterTab) {
			$scope.currentWidget = widget;
			widget.onEnterTab();
		}
	};
	$scope.$parent.$watch('widget.currentWidget', updateTabbed);
	function getNextTab(nextIndex, direction) {
		var nextTab = null;
		while(tabs[nextIndex] && !tabs[nextIndex].enabled) {
			nextIndex += direction;
		}
		if (nextIndex >= 0 && nextIndex < tabs.length) {
			nextTab = tabs[nextIndex];
		}
		return nextTab;
	}
	$scope.scrollSwipe = function(tab, direction){
		var currentIndex = $scope.widgets.indexOf(tab);
		var nextIndex = currentIndex + direction;
		var nextTab = getNextTab(nextIndex, direction);
		if(nextTab){
			if (nextTab !== null) {
				$scope.onClickTab($scope.$parent.widget,nextTab);
			}
		}
	};
}


// FILE: ./bower_components/red-ui-footer/dist/rui-footer.js
// FILE: src/assets/js/controllers/ruiFooterScreenController.js
function ruiFooterScreenController($scope, $element, eventAggregator, ScreenService) {
	/*
	 * -> footerHeight
	 * Sets the footerHeight in pixels
	 *
	 * -> footerProperties
	 * Contains the footerProperties object from the containers.json file
	 *
	 * -> footerDOM
	 * The DOM Object representing container of this controller
	 *
	 * -> currentWindow
	 * Informs the current opened window's name (in lowercase in order to be valid as window.openWindow() parameter)
	 *
	 * -> currentScrollTop
	 * Informs the current scrollTop property of the current container's DOM element
	 *
	 * -> isHidden
	 * Informs wether the container is currently hidden or not
	 *
	 * -> this.functionsToCallOnContainerOpen
	 * An array that contains the reference for all the functions that should be called by the OnContainerOpen event
	 */
	var footerHeight, footerProperties, footerDOM, currentWindow, currentScrollTop, isHidden;
	this.functionsToCallOnContainerOpen = [];

	/*
	 * Initializes variables and call functions needed for initializing the template
	 * Works like a class constructor
	 *
	 * return void
	 */
	this.initializeFooter = function() {
		footerHeight = 56;
		footerProperties = $scope.templateManager.containers.zeedhi_project.footerProperties;
		footerDOM = $element.get()[0];
		currentScrollTop = 0;
		isHidden = false;

		// var containerElement = document.getElementsByClassName('container')[0];
		// containerElement.classList.add('rui-has-footer');

		if (footerProperties.color == "flat")
			footerDOM.classList.add("rui-footer-flat");

		if (footerProperties.hideOnScrollDown) {
			this.addScrollEventToWindow();
			this.functionsToCallOnContainerOpen.push(this.addScrollEventToWindow.bind(this));
		}

		this.functionsToCallOnContainerOpen.push(this.selectFooterElement.bind(this));

		this.createDOMElementForEachFooterElement();
		this.selectFooterElement();
	};

	/*
	 * Iterates over footerProperties.elements for creating DOM element for each footerElement
	 * This function substiutes the ng-repeat AngularJS directive
	 *
	 * return void
	 */
	this.createDOMElementForEachFooterElement = function() {
		var footerElementDOMTemplate, newFooterElementDOM;

		footerElementDOMTemplate = footerDOM.querySelector("#rui-footer-element");

		for (var i=0; i<footerProperties.elements.length; i++) {
			newFooterElementDOM = footerElementDOMTemplate.cloneNode(true);
			newFooterElementDOM.setAttribute("id", "rui-footer-element-"+footerProperties.elements[i].windowName.toLowerCase());
			newFooterElementDOM.windowName = footerProperties.elements[i].windowName;
			newFooterElementDOM.addEventListener("click", this.changeWindow.bind(this));
			newFooterElementDOM.getElementsByTagName("span")[0].getElementsByTagName("i")[0].innerHTML = footerProperties.elements[i].icon;
			newFooterElementDOM.getElementsByTagName("p")[0].innerHTML = ScreenService.i18n(footerProperties.elements[i].label);

			footerDOM.appendChild(newFooterElementDOM);
		}

		footerDOM.removeChild(footerElementDOMTemplate);
	};

	/*
	 * Applies the selected effect on footerElement with windowName equal to the container's name
	 *
	 * return void
	 */
	this.selectFooterElement = function() {
		var footerElementDOM;

		if (currentWindow) {
			footerElementDOM = footerDOM.querySelector("#rui-footer-element-"+currentWindow.toLowerCase());
			if (footerElementDOM)
				footerElementDOM.classList.remove("selected");
		}

		currentWindow = $scope.templateManager.container.name;
		var newSelectedFooterElement = footerDOM.querySelector("#rui-footer-element-"+currentWindow.toLowerCase());
		if (newSelectedFooterElement) {
			newSelectedFooterElement.classList.add("selected");
		}
	};

	/*
	 * Changes the current window to newly selected one.
	 *
	 * return void
	 */
	this.changeWindow = function(event) {
		var footerElementName;

		event.stopPropagation();

		footerElementName = event.currentTarget.windowName;

		if (footerElementName != currentWindow)
			window.openWindow(footerElementName);
	};

	/*
	 * Adds scroll event to current container.
	 * Changes the container height to occupy the footer space also (but being covered by it when not hidden)
	 *
	 * return void
	 *
	 * TODO: find a better solution than the setTimeout and than adding the scrollEvent to all containers
	 */
	this.addScrollEventToWindow = function() {
		setTimeout(function() {
			var containers = document.querySelectorAll(".container");

			for (var i=0; i<containers.length; i++)
				containers[i].addEventListener("scroll", this.handleScrollEvent.bind(this));

			containers[0].style.height = "auto";
			containers[0].style.height = (containers[0].clientHeight + footerHeight)+"px";
		}.bind(this), 500);
	};

	/*
	 * Receives the event object and decides wether to hide, unhide or do nothing to the footer
	 *
	 * return void
	 */
	this.handleScrollEvent = function(event) {
		if (currentScrollTop != event.currentTarget.scrollTop) {
			if (currentScrollTop > event.currentTarget.scrollTop) {
				if (isHidden)
					this.unhideFooter();
			}
			else if (!isHidden)
				this.hideFooter();

			currentScrollTop = event.currentTarget.scrollTop;
		}
	};

	/*
	 * Hides the footer by setting his height to 0
	 *
	 * return void
	 */
	this.hideFooter = function() {
		footerDOM.style.height = 0;
		isHidden = true;
	};

	/*
	 * Unhides the footer by setting his height back to footerHeight
	 *
	 * return void
	 */
	this.unhideFooter = function() {
		footerDOM.style.height = footerHeight+"px";
		isHidden = false;
	};

	/*
	 * Register this.selectFooterElement and this.addScrollEventToWindow as a callback for onContainerOpen event
	 */
	eventAggregator.subscribe("onContainerOpen", function() {
		for (var i=0; i<this.functionsToCallOnContainerOpen.length; i++)
			this.functionsToCallOnContainerOpen[i]();
	}.bind(this));
}


