function mealsService(ZHPromise, restEngine, GeneralMethods) {
    var scope = this;

    this.getDeliveryPointsAndShifts = function(menuDate, serviceCode) {
        var promises = [];
        promises.push(this.getDeliveryPoints(menuDate, serviceCode));
        promises.push(this.getShifts(menuDate, serviceCode));

        return ZHPromise.all(promises)
            .then(function(response) {
                return {
                    deliveryPoints: response[0],
                    shifts: response[1]
                };
            })
            .catch(catchError);
    };

    this.getDeliveryPoints = function(menuDate, serviceCode) {
        return request("/getDeliveryPoints", "deliveryPoints", menuDate, serviceCode);
    };

    this.getShifts = function(menuDate, serviceCode) {
        return request("/getShifts", "shifts", menuDate, serviceCode);
    };

    this.getPlates = function(menuDate, serviceCode) {
        return request("/getPlates", "plates", menuDate, serviceCode);
    };

    this.save = function(meal, rowEdited) {
        meal.changed = meal.COMENSAIS !== rowEdited.COMENSAIS;
        meal.QTANTERIOR = angular.copy(meal.COMENSAIS);
        meal.COMENSAIS = rowEdited.COMENSAIS;
        updateShifts(rowEdited);
        updateDeliveryPoints(rowEdited);
        return restEngine.request("/meals/save", {
            requestType: "Row",
            row: {
                meal: meal,
                shifts: _shifts,
                deliveryPoints: _deliveryPoints
            }
        }).then(function(response) {
            var mealSaved = response.dataset.meal[0];
            for (var attr in mealSaved) {
                meal[attr] = mealSaved[attr];
            }
        }).catch(catchError);
    };

    function updateShifts(rowEdited) {
        for (var attr in rowEdited) {
            if (attr.indexOf("TURNO") !== -1) {
                var CDTURNO = attr.slice(-1);
                for (var i = 0; i < _shifts.length; i++) {
                    if (_shifts[i].CDTURNO === CDTURNO) {
                        _shifts[i].COMENSAIS = rowEdited[attr];
                    }
                }
            }
        }
    }

    function updateDeliveryPoints(rowEdited) {
        for (var attr in rowEdited) {
            if (attr.indexOf("PONTO") !== -1) {
                var CDDESTFILI = attr.slice(-3);
                for (var i = 0; i < _deliveryPoints.length; i++) {
                    if (_deliveryPoints[i].CDDESTFILI === CDDESTFILI) {
                        _deliveryPoints[i].COMENSAIS = rowEdited[attr];
                    }
                }
            }
        }
    }

    this.savePlates = function(changedPlates, meal) {
        updatePlates(changedPlates, meal.COMENSAIS);
        return restEngine.request("/plates/save", {
            requestType: "Row",
            row: {
                meal: meal,
                plates: getChangedPlates()
            }
        }).then(function() {
            for (var i = 0; i < _editablePlates.length; i++) {
                _editablePlates[i].changed = false;
            }
        }).catch(catchError);
    };

    function updatePlates(changedPlates, serviceQuantity) {
        serviceQuantity = GeneralMethods.valueToFloat(serviceQuantity);
        for (var attr in changedPlates) {
            var platesProperties = attr.split("-");
            for (var i = 0; i < _editablePlates.length; i++) {
                if (_editablePlates[i].CDTIPOPRAT === platesProperties[0] && _editablePlates[i].CDPRATO === platesProperties[1]) {
                    var plateQuantity = GeneralMethods.valueToFloat(changedPlates[attr]);
                    _editablePlates[i].QTCOMEORIGINAL = angular.copy(_editablePlates[i].COMENSAIS);
                    _editablePlates[i].COMENSAIS = plateQuantity;
                    _editablePlates[i].VRPECONSCARD = plateQuantity * 100 / serviceQuantity;
                    _editablePlates[i].changed = true;
                    break;
                }
            }
        }
    }

    function getChangedPlates() {
        return _editablePlates.filter(function(plate) {
            return plate.changed;
        });
    }

    function request(route, dataSourceName, menuDate, serviceCode) {
        return restEngine.request(route, {
            requestType: "FilterData",
            filter: getSessionFilter().concat(getMenuDateFilter(menuDate, serviceCode))
        }).then(function(response) {
            return response.dataset[dataSourceName];
        }).catch(catchError);
    }

    function getMenuDateFilter(menuDate, serviceCode) {
        var filter = [
            {
                name: "CDSERVFILI",
                operator: "=",
                value: serviceCode
			}
		];
        filter.push(scope.getMenuDateObjFilter(menuDate));
        return filter;
    }

    /**
     * Retrives a menu date obj filter
     * @param  {String} menuDate
     * @return {Object} filter
     */
    this.getMenuDateObjFilter = function(menuDate) {
        return {
            name: "DTCARDAPIO",
            operator: "=",
            value: menuDate
        };
    };

    function getSessionFilter() {
        return [
            {
                name: "CDFILIAL",
                operator: "=",
                value: "SESSION"
            },
            {
                name: "CDOPERADOR",
                operator: "=",
                value: "SESSION"
            },
            {
                name: "NRORG",
                operator: "=",
                value: "SESSION"
            }
		];
    }

    var _shifts = [];
    var _deliveryPoints = [];
    var _editablePlates = [];

    Object.defineProperty(this, 'shifts', {
        set: function(collection) {
            _shifts = collection;
        },
        get: function() {
            return _shifts;
        }
    });

    Object.defineProperty(this, 'deliveryPoints', {
        set: function(collection) {
            _deliveryPoints = collection;
        },
        get: function() {
            return _deliveryPoints;
        }
    });

    Object.defineProperty(this, 'editablePlates', {
        set: function(collection) {
            _editablePlates = collection;
        },
        get: function() {
            return _editablePlates;
        }
    });

}

Configuration(function(ContextRegister) {
    ContextRegister.register('MealsService', mealsService);
});
